/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ballistic.h"
#include "utility.h"
#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <errno.h>
#include <assert.h>
#include <math.h>
#include <iomanip>

using namespace arw;
/**
 * must not call from outside
 *
 *
 */
Ballistic::Ballistic() {// kann von aussen nicht aufgerufen werden

}

Ballistic::Ballistic(flightparam &fp) {
	m_max_stepsize = 0.01;
	m_flightparam = fp;
	setup_spline_cd(m_cd);
}

int Ballistic::setup_spline_cd(SplineCurve & spline) {
	errno = 0;

	/**
	 * First column is the Mach number, and the second column is the cd
	 */
	double g1[100][2] = {
			{0.00,  0.2629},
			{0.05,  0.2558},
			{0.10,  0.2487},
			{0.15,  0.2413},
			{0.20,  0.2344},
			{0.25,  0.2278},
			{0.30,  0.2214},
			{0.35,  0.2155},
			{0.40,  0.2104},
			{0.45,  0.2061},
			{0.50,  0.2032},
			{0.55,  0.2020},
			{0.60,  0.2034},
			{0.70,  0.2165},
			{0.725,  0.2230},
			{0.75,  0.2313},
			{0.775,  0.2417},
			{0.80,  0.2546},
			{0.825,  0.2706},
			{0.85,  0.2901},
			{0.875,  0.3136},
			{0.90,  0.3415},
			{0.925,  0.3734},
			{0.95,  0.4084},
			{0.975,  0.4448},
			{1.0,  0.4805},
			{1.025,  0.5136},
			{1.05,  0.5427},
			{1.075,  0.5677},
			{1.10,  0.5883},
			{1.125,  0.6053},
			{1.15,  0.6191},
			{1.20,  0.6393},
			{1.25,  0.6518},
			{1.30,  0.6589},
			{1.35,  0.6621},
			{1.40,  0.6625},
			{1.45,  0.6607},
			{1.50,  0.6573},
			{1.55,  0.6528},
			{1.60,  0.6474},
			{1.65,  0.6413},
			{1.70,  0.6347},
			{1.75,  0.6280},
			{1.80,  0.6210},
			{1.90,  0.6072},
			{1.95,  0.6003},
			{2.00,  0.5934},
			{2.05,  0.5867},
			{2.10,  0.5804},
			{2.15,  0.5743},
			{2.20,  0.5685},
			{2.25,  0.5630},
			{2.30,  0.5577},
			{2.40,  0.5481},
			{2.45,  0.5438},
			{2.50,  0.5397},
			{2.60,  0.5325},
			{2.70,  0.5264},
			{2.80,  0.5211},
			{2.90,  0.5168},
			{3.00,  0.5133},
			{3.10,  0.5105},
			{3.20,  0.5084},
			{3.30,  0.5067},
			{3.40,  0.5054},
			{3.50,  0.5040},
			{3.60,  0.5030},
			{3.70,  0.5022},
			{3.80,  0.5016},
			{3.90,  0.5010},
			{4.00,  0.5006},
			{4.20,  0.4998},
			{4.40,  0.4995},
			{4.60,  0.4992},
			{4.80,  0.4990},
			{5.00,  0.4988},
			{-1.0, -1.0 }    // this is important at the end of the data

	};


	int i = 0;

	spline.clear();
	//std::cout << std::endl;
	double mach, cd;
	for(i = 0; i< 1000; i++) {
		mach = g1[i][0];
		cd = g1[i][1];
		if(mach < 0.0) {
			break;
		}
		spline.add(mach * utility::g_MACH1, cd);
		//std::cout << g1[i][0] << ", " <<  g1[i][1] << std::endl;

	}

	spline.calcSpline();
	return 0;

}

/**
 *
 * @param sol_fs   the function writes in this field the flight snapshot at the range
 * @param curr_fs  release flight snapshot
 * @param range    the range
 * @param stepsize  stepsize in sec.
 * @return int 0 if succedd, otherwise > 0 in case of an error
 */
int Ballistic::flightparam_at_range(flight_snapshot & sol_fs,
		flight_snapshot curr_fs, //
		double range, // relativ range from curr.s.x in x-axis
		double stepsize)// Schrittweite in sec.
{
	errno = 0;
	flight_snapshot prev_fs = curr_fs;
	range += curr_fs.s.x;// abs. Entf.
	if(stepsize > m_max_stepsize)
		stepsize = m_max_stepsize;

	int Error = 1;// Fehlerflag
	double rest_time = 0.0;
	while (curr_fs.t < utility::g_max_flight_time && curr_fs.t >= 0.0) {
		rest_time = (range - curr_fs.s.x) / curr_fs.v.x;
		if (rest_time <= stepsize) {
			stepsize = rest_time;
		}
		if (stepsize < 0.005)
			stepsize = 0.005;
		//assert(curr_rot.arrow.x !=0.0 || curr_rot.arrow.y !=0.0 || curr_rot.arrow.z != 0.0);
		assert(errno == 0);

		do_rk_step(&curr_fs, stepsize);

		if (fabs(curr_fs.s.x) > 1000000.0 || fabs(curr_fs.s.y) > 1000000.0
				|| fabs(curr_fs.s.z) > 1000000.0) {
			return 5;
		}
		if (fabs(curr_fs.v.x) > 1000000.0 || fabs(curr_fs.v.y) > 1000000.0
				|| fabs(curr_fs.v.z) > 1000000.0) {
			return 6;
		}

		if (errno != 0)
			std::cout << "scheisse 567";
		// zu weit unterhalb der ziellinie
		if (curr_fs.s.y < -100.0) // 100 ist zu klein
			return 3; // elevation in y-dir to big

		// Distance is reached:
		if (curr_fs.s.x >= range) {

			Error = 0;// break befor max flighttime is ok!
			double fact = (range - prev_fs.s.x) / (curr_fs.s.x - prev_fs.s.x);
			sol_fs = Ballistic::linint(prev_fs, curr_fs, fact);
			break;

		}
		// save old values for linear interpolation:
		prev_fs = curr_fs;
		//print_flight_snapshot(curr_fs);

	}

	if (errno != 0) {
		errno = 0;
		return 1;
	}

	return Error;
}

/**
* @brief Used to calc the max. range in: \fn calc_horizontal_angle_for_zero_range
* Calc the range, where the bullet pirces the XZ-layer.
*
*  @param   flight_snapshot&
*  @param   flight_snapshot
*  @param   double
*
* @return int 0 if succedd, otherwise > 0 in case of an error
*
*/
int Ballistic::flightparam_at_zero_range(flight_snapshot & sol_fs,
		flight_snapshot curr_fs, double stepsize) {
	errno = 0;
	flight_snapshot prev_fs = curr_fs;
	//range += curr_fs.s.x;// abs. Entf.
	if(stepsize > m_max_stepsize)
		stepsize = m_max_stepsize;

	int Error = 1;// Fehlerflag
	double rest_time = 0.0;
	while (curr_fs.t < utility::g_max_flight_time && curr_fs.t >= 0.0) {

		if (curr_fs.v.y < 0.0 && curr_fs.s.y > 0.0) {
			rest_time = fabs(curr_fs.s.y / curr_fs.v.y);
			if (rest_time < stepsize)
				stepsize = rest_time;
		}
		if (stepsize < 0.005)
			stepsize = 0.005;
		//assert(curr_rot.arrow.x !=0.0 || curr_rot.arrow.y !=0.0 || curr_rot.arrow.z != 0.0);
		assert(errno == 0);

		do_rk_step(&curr_fs, stepsize);

		if (fabs(curr_fs.s.x) > 1000000.0 || fabs(curr_fs.s.y) > 1000000.0
				|| fabs(curr_fs.s.z) > 1000000.0) {
			return 5;
		}
		if (fabs(curr_fs.v.x) > 1000000.0 || fabs(curr_fs.v.y) > 1000000.0
				|| fabs(curr_fs.v.z) > 1000000.0) {
			return 6;
		}

		if (errno != 0)
			std::cout << "scheisse 567";

		// Distance is reached:
		if (curr_fs.s.y < 0.0 && curr_fs.v.y < 0.0) {

			Error = 0;// break befor max flighttime is ok!
			double fact = (prev_fs.s.y) / (prev_fs.s.y - curr_fs.s.y);
			sol_fs = Ballistic::linint(prev_fs, curr_fs, fact);
			break;

		}
		// vertical shoot, the arrow turns to come back
		if (curr_fs.s.x < prev_fs.s.x) {
			//sol_fs = curr_fs;
			break;
		}

		// save old values for linear interpolation:
		prev_fs = curr_fs;
		//print_flight_snapshot(curr_fs);

	}

	if (errno != 0) {
		errno = 0;
		return 1;
	}

	return Error;

}

/**
 *
 ################################################
 gnuplotbefehl:
 ################################################
 # 2D:
 ########
 set multiplot;
 set   autoscale
 set size 1,0.25;

 set origin 0.0,0.0;
 plot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 1:9 title 'Pfeildrehung-z' with linespoints;

 set origin 0.0,0.25;
 plot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 1:8 title 'Pfeildrehung-y' with lines;

 set origin 0.0,0.5;
 plot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 1:3 title 'Weg-z' with lines ;


 set origin 0.0,0.75;
 plot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 1:2 title 'Weg-y' with lines;

 unset multiplot;

 #################################################################
 # 3D:
 #######
 # Drehung vom Pfeil:
 set autoscale
 set size 1,1;
 set origin 0.0,0.0;
 splot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 1:9:8 title 'Pfeildrehung-z' with linespoints;

 # Flugbahn:
 set autoscale
 set size 1,1;
 set origin 0.0,0.0;
 splot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 1:3:2 title 'Weg-z' with linespoints ;




 */

// int Ballistic::calc_alpha(double & alpha, //rad Vorhaltewinkel in rad
//                           flight_snapshot & endvalues,  // Endwerte
//                           double dEntfernung, //m
//                           double dV,   //m/s
//                           double dZielwinkel // rad !!
//                          )
// {
//
//     return 1*10;
//
//
//
//     //v = Pfeilgeschwindigkeit (m/s)
//     //Entfernung = Entfernung (m)
//     //Zielwinkel = wird nur durch die Richtung der Schwerkraft (vector) definiert in flightparam
// }
int Ballistic::calc_release_parameter(flight_snapshot & solution, //
		flight_snapshot & endvalues, // Endwerte
		double zero_range, //m
		double muzzle_velocity) { //m/s

	double dAlpha1;//Vorhaltewinkel in rad at the beginning
	//double dAlpha2;
	double stepsize = 7.0 / muzzle_velocity + 0.001;// alle 7 m ist ausreichend aber min. 0.001
	//stepsize = (dEntfernung / dV) / 3000.0;
	// Kompensationswinkel als Steigung bei einer Länge von 1
	double radial_error;//

	flight_snapshot begin_fs;
	flight_snapshot end_fs;
	/*
	 * Der Winkelvector in rad:
	 */
	utility::vector_3d flightdirection;

	//const double MAX_ANGLE = 85.0 * PI / 180.0; //1.53 in rad, sollte bis 89 sein
	int err;
	errno = 0;

	int iDurchlaeufe = 0;

	//iFehlerflag = 0;
	if (zero_range <= 0.0)
		return 1;
	if (muzzle_velocity <= 0.0) //Siehe dTreffpunktlage
		return 1;

	begin_fs.t = 0.0;
	begin_fs.s.x = 0.0;
	begin_fs.s.y = 0.0;
	begin_fs.s.z = 0.0;
	begin_fs.v.x = 0.0;
	begin_fs.v.y = 0.0;
	begin_fs.v.z = 0.0;
	begin_fs.phi.x = 0.0;
	begin_fs.phi.y = 0.0;
	begin_fs.phi.z = 0.0;
	begin_fs.omega.x = 0.0;
	begin_fs.omega.y = 0.0;
	begin_fs.omega.z = 0.0;

	// Startwert suchen:
	dAlpha1 = -0.2;
	while (dAlpha1 < utility::HALF_OF_PI / 2.0) {// 45
		dAlpha1 += 0.2;

		flightdirection.x = 0.0;
		flightdirection.y = 0.0;
		flightdirection.z = dAlpha1;

		begin_fs.v.x = muzzle_velocity;
		begin_fs.v.y = 0.0;
		begin_fs.v.z = 0.0;
		utility::rotate(begin_fs.v, flightdirection);

		begin_fs.omega.x = 0.0;
		begin_fs.omega.y = 0.0;
		begin_fs.omega.z = 0.0;

		begin_fs.phi = flightdirection;

		begin_fs.s.x = 0.0;
		begin_fs.s.y = 0.0;
		begin_fs.s.z = 0.0;

		err = flightparam_at_range(end_fs, begin_fs, zero_range, stepsize);

		if (err != 0) {
			continue;
		} else {
			break;
		}
	}
	if (err) {

		return err;

	}

	// Treffpunkt ist zu weit unterhalb vom Zielpunkt (über 45°)
	if (end_fs.s.y < (end_fs.s.x * (-1.0))) {
		return 2* 10 ;
	}

	////////////////////////////////////////////
	// Den Abschusswinkel corrigieren:
	flightdirection.x = 0.0;
	flightdirection.y += end_fs.s.z / end_fs.s.x;
	flightdirection.z -= end_fs.s.y / end_fs.s.x;

	//stepsize = 10.0/dV + 0.001;// alle 7 m ist ausreichend aber max. 0.001
	//std::cout << "anfang des durchlaufes\n";


	//                                             (x2 - x1)
	// Sekantenverfahren:  xneu := x2 - f(x2) * -------------
	//                                          f(x2) * f(x1)
	// x1 = die Steigung bei 1 (45° sind demnach 1)
	// f(x1) = die Abweichung auf der gewünschten Entfernung
	for (iDurchlaeufe=0; iDurchlaeufe<30; iDurchlaeufe++)
	{
		begin_fs.v.x = muzzle_velocity;
		begin_fs.v.y = 0.0;
		begin_fs.v.z = 0.0;
		utility::rotate(begin_fs.v, flightdirection);

		begin_fs.omega.x = 0.0;
		begin_fs.omega.y = 0.0;
		begin_fs.omega.z = 0.0;

		begin_fs.phi = flightdirection;

		begin_fs.s.x = 0.0;
		begin_fs.s.y = 0.0;
		begin_fs.s.z = 0.0;

		err = flightparam_at_range(end_fs, begin_fs, zero_range, stepsize );
		if(err != 0)
		{
			std::cout << "[Ballistic::calc_alpha] Treffpunktlage - Fehler=" << err << "durchlaufe=" << iDurchlaeufe << std::endl;
			return err;
		}

		// Abbruchkriterium:
		radial_error = sqrt(pow(end_fs.s.y,2.0) + pow(end_fs.s.z, 2.0));
		if(radial_error < 0.001) {
			solution = begin_fs;
			endvalues = end_fs;
			return 0;
		}
		if(stepsize > 0.001) {
			if(radial_error < 0.001) {
				stepsize = 0.1/muzzle_velocity + 0.001;
			}
			else if(radial_error < 0.01) {
				stepsize = 0.5/muzzle_velocity + 0.001;

			}
			else if(radial_error < 0.1) {
				stepsize = 1.0/muzzle_velocity + 0.001;
			}

		}

		// Verbesserte Abschusswinkel:
		// ist nicht exakt und gilt nur für kleine Korrekturen:
		flightdirection.x = 0.0;
		flightdirection.y += end_fs.s.z / end_fs.s.x;
		flightdirection.z -= end_fs.s.y / end_fs.s.x;

		//         std::cout << std::endl;

	}
	//std::cout << "max durchlaufzahl" << std::endl;
	return 1*10;

}

int Ballistic::calc_flight_table(std::vector<flight_snapshot> &table,
		double max_time, // s
		double max_range, // m in x-Axis
		double zero_range, double muzzle_velocity) {

	int iError = 0;
	flight_snapshot curr_fs;
	flight_snapshot endvalues;
	double stepsize_time = 1.0 / muzzle_velocity + 0.001;
	if(stepsize_time > m_max_stepsize)
		stepsize_time = m_max_stepsize;
	iError = calc_release_parameter(curr_fs, //
			endvalues, // Endwerte
			zero_range, //m
			muzzle_velocity //m/s
			);
	if (iError) {
		return 1;
	}
	table.push_back(curr_fs);
	while (curr_fs.t < max_time && curr_fs.s.x < max_range) {
		do_rk_step(&curr_fs, stepsize_time);
		table.push_back(curr_fs);
	}
	if (errno != 0) {
		errno = 0;
		return 7;
	}
	return iError;
}

void Ballistic::do_euler_step(struct flight_snapshot * snapshot, double step) {
	struct accel_snapshot accel;
	struct flight_snapshot k_flight = *snapshot;
	struct flight_snapshot orig_snapshot = *snapshot;

	get_accel(accel, k_flight);
	snapshot->s += snapshot->v * step + accel.a * step * step / 2.0;
	snapshot->phi += snapshot->omega * step + accel.alpha * step * step / 2.0;

	snapshot->v += accel.a * step;
	snapshot->omega += accel.alpha * step;
	snapshot->a = accel.a;
	snapshot->alpha = accel.alpha;
	snapshot->t += step;
}

///////////////////////////////////////////////////////////////////
/** This function takes a single RK step across the flight path

 */
void Ballistic::do_rk_step(struct flight_snapshot * snapshot, double step) {
	//assert(snapshot->arrow.x !=0.0 || snapshot->arrow.y !=0.0 || snapshot->arrow.z != 0.0);
	struct accel_snapshot accel;
	struct flight_snapshot k_flight;
	flight_snapshot orig_snapshot = *snapshot;

	// the velocities:
	struct rk {
		utility::vector_3d delta_v;
		utility::vector_3d delta_omega;
	} k0, k1, k2, k3;
	////////////////////////////////////////////////////////////
	// Determine K0
	k_flight = *snapshot;
	get_accel(accel, k_flight);

	snapshot->a = accel.a;
	snapshot->alpha = accel.alpha;

	k0.delta_v.x = step * accel.a.x;
	k0.delta_v.y = step * accel.a.y;
	k0.delta_v.z = step * accel.a.z;
	k0.delta_omega.x = step * accel.alpha.x;
	k0.delta_omega.y = step * accel.alpha.y;
	k0.delta_omega.z = step * accel.alpha.z;

	///////////////////////////////////////////////////////////
	// Determine K1
	k_flight = *snapshot;
	k_flight.v.x += k0.delta_v.x / 2.0;
	k_flight.v.y += k0.delta_v.y / 2.0;
	k_flight.v.z += k0.delta_v.z / 2.0;
	k_flight.omega.x += k0.delta_omega.x / 2.0;
	k_flight.omega.y += k0.delta_omega.y / 2.0;
	k_flight.omega.z += k0.delta_omega.z / 2.0;

	k_flight.s.x += step * snapshot->v.x / 2.0;
	k_flight.s.y += step * snapshot->v.y / 2.0;
	k_flight.s.z += step * snapshot->v.z / 2.0;
	k_flight.phi.x += step * snapshot->omega.x / 2.0;
	k_flight.phi.y += step * snapshot->omega.y / 2.0;
	k_flight.phi.z += step * snapshot->omega.z / 2.0;

	get_accel(accel, k_flight);
	k1.delta_v.x = step * accel.a.x;
	k1.delta_v.y = step * accel.a.y;
	k1.delta_v.z = step * accel.a.z;
	k1.delta_omega.x = step * accel.alpha.x;
	k1.delta_omega.y = step * accel.alpha.y;
	k1.delta_omega.z = step * accel.alpha.z;
	assert(errno == 0);

	////////////////////////////////////////////////////////
	// Determine K2
	k_flight = *snapshot;
	k_flight.v.x += k1.delta_v.x / 2.0;
	k_flight.v.y += k1.delta_v.y / 2.0;
	k_flight.v.z += k1.delta_v.z / 2.0;
	k_flight.omega.x += k1.delta_omega.x / 2.0;
	k_flight.omega.y += k1.delta_omega.y / 2.0;
	k_flight.omega.z += k1.delta_omega.z / 2.0;
	assert(errno == 0);

	k_flight.s.x += step * snapshot->v.x / 2.0 + step * k0.delta_v.x / 4.0;
	k_flight.s.y += step * snapshot->v.y / 2.0 + step * k0.delta_v.y / 4.0;
	k_flight.s.z += step * snapshot->v.z / 2.0 + step * k0.delta_v.z / 4.0;
	assert(errno == 0);
	k_flight.phi.x += step * snapshot->omega.x / 2.0 + step * k0.delta_omega.x
			/ 4.0;
	k_flight.phi.y += step * snapshot->omega.y / 2.0 + step * k0.delta_omega.y
			/ 4.0;
	k_flight.phi.z += step * snapshot->omega.z / 2.0 + step * k0.delta_omega.z
			/ 4.0;
	assert(errno == 0);

	get_accel(accel, k_flight);
	assert(errno == 0);
	k2.delta_v.x = step * accel.a.x;
	k2.delta_v.y = step * accel.a.y;
	k2.delta_v.z = step * accel.a.z;
	k2.delta_omega.x = step * accel.alpha.x;
	k2.delta_omega.y = step * accel.alpha.y;
	k2.delta_omega.z = step * accel.alpha.z;
	assert(errno == 0);

	///////////////////////////////////////////////////////
	// Determine K3
	k_flight = *snapshot;
	k_flight.v.x += k2.delta_v.x;
	k_flight.v.y += k2.delta_v.y;
	k_flight.v.z += k2.delta_v.z;
	k_flight.omega.x += k2.delta_omega.x;
	k_flight.omega.y += k2.delta_omega.y;
	k_flight.omega.z += k2.delta_omega.z;

	k_flight.s.x += step * snapshot->v.x + step * k1.delta_v.x / 2.0;
	k_flight.s.y += step * snapshot->v.y + step * k1.delta_v.y / 2.0;
	k_flight.s.z += step * snapshot->v.z + step * k1.delta_v.z / 2.0;

	k_flight.phi.x += step * snapshot->omega.x + step * k1.delta_omega.x / 2.0;
	k_flight.phi.y += step * snapshot->omega.y + step * k1.delta_omega.y / 2.0;
	k_flight.phi.z += step * snapshot->omega.z + step * k1.delta_omega.z / 2.0;

	get_accel(accel, k_flight);
	k3.delta_v.x = step * accel.a.x;
	k3.delta_v.y = step * accel.a.y;
	k3.delta_v.z = step * accel.a.z;
	k3.delta_omega.x = step * accel.alpha.x;
	k3.delta_omega.y = step * accel.alpha.y;
	k3.delta_omega.z = step * accel.alpha.z;
	assert(errno == 0);

	////////////////////////////////////////////////////////////////////
	// determine the new values with the the 3 k-values (runge-kutta)

	snapshot->s.x += step * snapshot->v.x + (k0.delta_v.x + k1.delta_v.x
			+ k2.delta_v.x) * (step / 6.0);
	snapshot->s.y += step * snapshot->v.y + (k0.delta_v.y + k1.delta_v.y
			+ k2.delta_v.y) * (step / 6.0);
	snapshot->s.z += step * snapshot->v.z + (k0.delta_v.z + k1.delta_v.z
			+ k2.delta_v.z) * (step / 6.0);
	snapshot->phi.x += step * snapshot->omega.x + (k0.delta_omega.x
			+ k1.delta_omega.x + k2.delta_omega.x) * (step / 6.0);
	snapshot->phi.y += step * snapshot->omega.y + (k0.delta_omega.y
			+ k1.delta_omega.y + k2.delta_omega.y) * (step / 6.0);
	snapshot->phi.z += step * snapshot->omega.z + (k0.delta_omega.z
			+ k1.delta_omega.z + k2.delta_omega.z) * (step / 6.0);
	//assert(snapshot->arrow.x >0.0 || snapshot->arrow.y >0.0 || snapshot->arrow.z > 0.0);
	assert(errno == 0);

	//assert(snapshot->arrow.x != 0.0 || snapshot->arrow.y !=0.0 || snapshot->arrow.z != 0.0);


	snapshot->v.x += (k0.delta_v.x + 2.0 * k1.delta_v.x + 2.0 * k2.delta_v.x
			+ k3.delta_v.x) / 6.0;
	snapshot->v.y += (k0.delta_v.y + 2.0 * k1.delta_v.y + 2.0 * k2.delta_v.y
			+ k3.delta_v.y) / 6.0;
	snapshot->v.z += (k0.delta_v.z + 2.0 * k1.delta_v.z + 2.0 * k2.delta_v.z
			+ k3.delta_v.z) / 6.0;
	snapshot->omega.x += (k0.delta_omega.x + 2.0 * k1.delta_omega.x + 2.0
			* k2.delta_omega.x + k3.delta_omega.x) / 6.0;
	snapshot->omega.y += (k0.delta_omega.y + 2.0 * k1.delta_omega.y + 2.0
			* k2.delta_omega.y + k3.delta_omega.y) / 6.0;
	snapshot->omega.z += (k0.delta_omega.z + 2.0 * k1.delta_omega.z + 2.0
			* k2.delta_omega.z + k3.delta_omega.z) / 6.0;

	snapshot->t += step;
	//assert(errno == 0);

}


/**
* @brief This routine determines the current x, y and z accelerations of translation and rotation
* @see http://de.wikipedia.org/wiki/Luftwiderstand#Turbulente_Str.C3.B6mung
*
*  @param   enclosing_method_arguments
*
*  @return  return_type
*
*/
void Ballistic::get_accel(struct accel_snapshot & accel,
		const struct flight_snapshot & snapshot) {

	//rotate the airflow in the arrow coordinate system:
	utility::vector_3d inertial_system = snapshot.phi;
	inertial_system *= -1.0;

	utility::vector_3d wind = m_flightparam.wind;
	utility::rotate(wind, inertial_system);

	utility::vector_3d gravity = m_flightparam.gravity;
	utility::rotate(gravity, inertial_system);

	utility::vector_3d v = snapshot.v;
	utility::rotate(v, inertial_system);

	utility::vector_3d omega = snapshot.omega;// not used
	utility::rotate(omega, inertial_system);



	// relativ Geschwindigkeit zur Luft
	utility::vector_3d v_air;
	v_air = wind - v;

	double airspeed = sqrt(v_air.x * v_air.x + v_air.y * v_air.y + v_air.z
			* v_air.z);

	/* First we calc only the accelaration because of the wind (without the gravity,
	 * which has no influence in the torsion)
	 *
	 * This is _not_ an exact solution, because the formfactor depends of the angle to the wind
	 * */

	utility::vector_3d force_wind;

	if (fabs(airspeed) > 1.0e-100 ) {

		if(fabs(m_flightparam.arrow.cp) > 0.0001) {


			double fact = 0.5 * m_flightparam.rho  * m_cd.spline(airspeed)
					* airspeed * airspeed;
			force_wind.x = fact * m_flightparam.arrow.A_frontal	* m_flightparam.arrow.ff_frontal  * v_air.x / airspeed;

			force_wind.y = fact * m_flightparam.arrow.A_cross * m_flightparam.arrow.ff_cross * v_air.y / airspeed;

			force_wind.z = fact * m_flightparam.arrow.A_cross * m_flightparam.arrow.ff_cross  * v_air.z / airspeed;


		}else{
			/* If the arrow.cp == 0 then take only the frontal area for the drag. Cross area is ignored.
			 *
			 *
			 */
			double force = 0.5 * m_flightparam.rho * m_flightparam.arrow.A_frontal
					* m_flightparam.arrow.ff_frontal * m_cd.spline(airspeed)
					* airspeed * airspeed;
			force_wind.x = force  * v_air.x / airspeed;
			force_wind.y = force  * v_air.y / airspeed;
			force_wind.z = force  * v_air.z / airspeed;

		}

	}else {
		force_wind.x = 0.0;
		force_wind.y = 0.0;
		force_wind.z = 0.0;
	}


	////////////////////////////////////////////
	// angular accelaration:
	if(fabs(m_flightparam.arrow.cp) > 0.0001){
		accel.alpha.x = 0.0;// ignore the rotation of the arrow around the x-axis
		accel.alpha.y = -m_flightparam.arrow.cp * force_wind.z
		/ m_flightparam.arrow.J;
		accel.alpha.z = m_flightparam.arrow.cp * force_wind.y
		/ m_flightparam.arrow.J;
	}


	///////////////////////////////////////////////////
	// drag and gravity:
	accel.a.x = force_wind.x / m_flightparam.arrow.m + gravity.x;
	accel.a.y = force_wind.y / m_flightparam.arrow.m + gravity.y;
	accel.a.z = force_wind.z / m_flightparam.arrow.m + gravity.z;

	///////////////////////////////////////////////////
	// turn back from the arrow inertial system to the global coordinate system:
	utility::rotate(accel.a, snapshot.phi);
	utility::rotate(accel.alpha, snapshot.phi);

}

/** @brief linear interpolation between 2 flight positions
 * @param flight_snapshot
 * @param flight_snapshot
 */
struct flight_snapshot Ballistic::linint(const struct flight_snapshot & fs1,
		const struct flight_snapshot & fs2, double fact) {
	flight_snapshot mid_fs;
	mid_fs.a = fs1.a + (fs2.a - fs1.a) * fact;
	mid_fs.alpha = fs1.alpha + (fs2.alpha - fs1.alpha) * fact;
	mid_fs.omega = fs1.omega + (fs2.omega - fs1.omega) * fact;
	mid_fs.phi = fs1.phi + (fs2.phi - fs1.phi) * fact;
	mid_fs.s = fs1.s + (fs2.s - fs1.s) * fact;
	mid_fs.t = fs1.t + (fs2.t - fs1.t) * fact;
	mid_fs.v = fs1.v + (fs2.v - fs1.v) * fact;
	return mid_fs;

}

/** @brief only a helper funktion for debugging
 * @param flight_snapshot
 */
void Ballistic::print_flight_snapshot(flight_snapshot & snapshot) {

	std::cout << std::setw(15) << snapshot.t  <<  snapshot.s  <<  snapshot.v
			<<  snapshot.a  <<  snapshot.phi
			<< snapshot.omega <<  snapshot.alpha
			<< std::endl;

}
/** @brief only a helper funktion for debugging
 *
 */
void Ballistic::print_flight_snapshot_header() {
	std::cout  << "\n#    t              "
			   << "s.x            s.y            s.z            v.x            v.y            v.z            "
	           << "a.x            a.y            a.z            phi.x          phi.y          phi.z          "
	           << "omega.x        omega.y        omega.z        alpha.x        alpha.y        alpha.z        "
	<< std::endl;

}
/** @brief rotate the gravity vector for uphill or downhill shoots
 * @param  flightparameter
 * @param angle  zero is a shoot in horizontal direction. [rad]
 *               greater than zero is uphill
 *               lesser than zero is downhill
 */
void Ballistic::flightparam_gravity(flightparam & fp, double angle) {

	utility::vector_3d rot;
	rot.x = 0.0;
	rot.y = 0.0;
	rot.z = (-1) * angle;

	fp.gravity.x = 0.0;
	fp.gravity.y = -utility::g_G;
	fp.gravity.z = 0.0;
	utility::rotate(fp.gravity, rot);
}
/** @brief modify the wind vector
 * @param  flightparameter
 * @param angle  zero: wind comes from back (6 o'clock)
 *               clockwise,
 *               greater than zero: wind comes from left side
 *               lower than zero: wind comes from right side
 *               [rad]
 * @param speed  wind speed  [m/s]
 *
 */
void Ballistic::flightparam_wind(flightparam &fp, double angle, double speed) {
	utility::vector_3d wind_angle;
	wind_angle.x = 0.0;
	wind_angle.y = (-1) * angle;// im Uhrzeigersinn, 0rad ist in x-axis
	wind_angle.z = 0.0;

	fp.wind.x = speed; // Windgeschwindigkeit in x-axis
	fp.wind.y = 0.0;
	fp.wind.z = 0.0;
	utility::rotate(fp.wind, wind_angle);
}
/** @brief init the flightparam vector for horizontal shoots without wind
 * @param  flightparameter
 * @param  arrow
 *
 */
void Ballistic::flightparam_init(flightparam &fp, const arrow & a) {
	fp.gravity.x = 0.0;
	fp.gravity.y = (-1) * utility::g_G;
	fp.gravity.z = 0.0;

	fp.rho = utility::g_AIR_DENSITY;
	fp.arrow = a;

	fp.wind.x = 0.0; // Windgeschwindigkeit in x-axis
	fp.wind.y = 0.0;
	fp.wind.z = 0.0;

}

/**
 * @param  flight_snapshot impact parameter at traverse the zero plane
 * @param  flight_snapshot release parameter
 * @param  double release speed of the arrow
 * @param  double vertical (yaw) release angle of the arrow [rad]
 * @param  stepsize   time steps [sec]
 * @return int 0 for success, otherwise <>0 in case of an error
 *
 */
int Ballistic::calc_horizontal_angle_for_zero_range(flight_snapshot & _end_fs,
		flight_snapshot & _begin_fs, double v, double vertical_angle,
		double stepsize) {

	flight_snapshot end_fs, begin_fs;
	utility::vector_3d release_angle;
	release_angle.y = 0.0;
	release_angle.z = vertical_angle;
	release_angle.x = 0.0;
	int err;
	bool ok = false;

	for (int i = 0; i < 10; i++) {
		begin_fs.s.x = 0.0;
		begin_fs.s.y = 0.0;
		begin_fs.s.z = 0.0;
		begin_fs.v.x = v;
		begin_fs.v.y = 0.0;
		begin_fs.v.z = 0.0;
		utility::rotate(begin_fs.v, release_angle);
		begin_fs.t = 0.0;
		begin_fs.phi = release_angle;
		begin_fs.omega.x = 0.0;
		begin_fs.omega.y = 0.0;
		begin_fs.omega.z = 0.0;

		err = flightparam_at_zero_range(end_fs, begin_fs, stepsize);
		if (err) {
			return 2;
		}
		if (fabs(end_fs.s.z) < 0.01) {
			ok = true;
			break;

		}
		release_angle.y += end_fs.s.z / end_fs.s.x;
	}
	if (ok == false) {
		return 1;
	}
	_begin_fs = begin_fs;
	_end_fs = end_fs;
	return 0;
}
/**
 * @todo Testen, in_angle_max optimieren, wegen den Schwerkrafteinfluss
 * alle Winkel in rad
 */
int Ballistic::calc_max_range(flight_snapshot & _begin_fs,
		flight_snapshot & _end_fs, double v) {

	int anz_points = 10; //20 //laufzeitvorteil
	double in_angle_min = 0.0;
	double in_angle_max = 135.0 * utility::PI / 180.0;

	std::vector<flight_snapshot> seq_end_fs(3);
	std::vector<flight_snapshot> seq_begin_fs(3);

	std::vector<utility::vector_3d> seq_begin_angle(3);

	flight_snapshot begin_fs, end_fs;
	double begin_angle = in_angle_min;
	double stepsize_angle;

	bool ok = false;

	double stepsize_time = 10.0 / v + 0.001;

	int iFehler = calc_horizontal_angle_for_zero_range(end_fs, begin_fs, v,
			begin_angle, stepsize_time);
	if (iFehler) {
		return 1;
	}
	seq_end_fs[0] = end_fs;
	seq_begin_fs[0] = begin_fs;
	seq_begin_angle[0] = begin_fs.phi;

	for (int i = 0; i < 20; i++) {

		stepsize_angle = fabs(in_angle_max - in_angle_min) / (double) anz_points;
		if (stepsize_angle > 0.1) {
			stepsize_angle = 0.1;
		}

		begin_angle += stepsize_angle;

		iFehler = calc_horizontal_angle_for_zero_range(end_fs, begin_fs, v,
				begin_angle, stepsize_time);
		if (iFehler) {
			return 2;
		}
//		std::cout << "             winkel=" << begin_angle << "     entf="
//				<< end_fs.s.x << std::endl;


		seq_end_fs[1] = end_fs;
		seq_begin_fs[1] = begin_fs;
		seq_begin_angle[1] = begin_fs.phi;



		do {

			begin_angle += stepsize_angle;
			iFehler = calc_horizontal_angle_for_zero_range(end_fs, begin_fs, v,
					begin_angle, stepsize_time);
			if (iFehler) {
				return 3;
			}
//			std::cout << "             winkel=" << begin_angle << "     entf="
//					<< end_fs.s.x << std::endl;

			seq_end_fs[2] = end_fs;
			seq_begin_fs[2] = begin_fs;
			seq_begin_angle[2] = begin_fs.phi;

			if (seq_end_fs[2].s.x < seq_end_fs[1].s.x) {
				std::cout << "-------" << seq_end_fs[2].s.x << " < " << seq_end_fs[1].s.x << std::endl;
				break;
			}


			// rotate the ring buffer:
			seq_end_fs[0] = seq_end_fs[1];
			seq_end_fs[1] = seq_end_fs[2];
			seq_end_fs[2] = end_fs;

			seq_begin_fs[0] = seq_begin_fs[1];
			seq_begin_fs[1] = seq_begin_fs[2];
			seq_begin_fs[2] = begin_fs;

			seq_begin_angle[0] = seq_begin_angle[1];
			seq_begin_angle[1] = seq_begin_angle[2];
			seq_begin_angle[2] = begin_fs.phi;


		} while (begin_angle <= in_angle_max);


		//std::cout << "winkel=" << seq_begin_angle[1].z << "    enft="
		//		<< seq_end_fs[1].s.x << std::endl;
		if (fabs(seq_begin_angle[2].z - seq_begin_angle[0].z) < 0.001) {
			ok = true;
			_begin_fs = seq_begin_fs[1];
			_end_fs = seq_end_fs[1];
			break;

		}

		in_angle_max = seq_begin_fs[2].phi.z;
		in_angle_min = seq_begin_fs[0].phi.z;

		begin_angle = seq_begin_fs[0].phi.z;

	}
	if (ok == false) {
		return 4;
	}
	return 0;
}

