/***************************************************************************
 *
 *   Copyright (C) 2009 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * crossbow.cpp
 *  Created on: 30.05.2009
 */

#include "crossbow.h"
#include <errno.h>
#include <assert.h>
#include <iostream>
#include "utility.h"
#include <vector>

using namespace arw;

crossbow::crossbow() {
	m_scso.trans.x = 0.0;
	m_scso.trans.y = 0.05;
	m_scso.trans.z = 0.0;
	m_scso.rot.x = 0.0;
	m_scso.rot.y = 0.0;
	m_scso.rot.z = -0.01;
	m_energy = 80.0;
}

crossbow::~crossbow() {
	// TODO Auto-generated destructor stub
}
/**
 * This function calc the flightparam at zerorange. It test only the vertical direction
 * @param _endparam      function writes the flightsnapshot when reach the zero range, in this field
 * @param _beginparam    function writes the muzzle flightsnapshot in this field.
 * @param sight_setting  the directon vector to the target from the sight system, only the x and the y is necessary,
 *                       the z value is ignored.
 * @param fp             flightparameter
 * @return               zero at success, otherwise <> 0
 */
int crossbow::flightparam_at_zero_range(flight_snapshot & _endparam,
		flight_snapshot & _beginparam, utility::vector_3d sight_setting,
		flightparam fp)

{
	double Entfernung1;
	double Entfernung2;
	double A1;//Einstellwertdifferenz
	double A2;
	double k, d;//Tangentensteigung, Tangentenabst. auf der y-Achse
	int iZaehler;
	int iFehler;

	errno = 0;
	bool success = false;
	utility::vector_3d setting;
	flight_snapshot endparam, beginparam;
	sight_setting /= sight_setting.x;

	/** only for testing

	 plot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 1:2  with linespoints;

	 */

	double velocity = sqrt(2.0 * m_energy / fp.arrow.m);
	Entfernung1 = velocity; //Startwert = Geschwindigkeit hat sich bewährt
	iFehler = this->sight_setting(setting, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
			endparam, beginparam, Entfernung1, //m
			fp);

	setting /= setting.x;
	setting.y -= sight_setting.y;
	A1 = setting.y;

	Entfernung2 = Entfernung1 + 0.1;
	//std::cout << "flightparam_at_zero_range" << std::endl;
	for (iZaehler = 1; iZaehler < 30; iZaehler++) {
		iFehler = this->sight_setting(setting, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
				endparam, beginparam, Entfernung2, //m
				fp);
		if (0 != iFehler)
			return iFehler;
		setting /= setting.x;
		setting.y -= sight_setting.y;
		A2 = setting.y;

		if ((Entfernung1 - Entfernung2) == 0.0)
			return 1;
		k = (A1 - A2) / (Entfernung1 - Entfernung2);
		//if(k > -0.002)
		//	k = -0.002;

		//std::cout << "k = " << k << ", A2= " << A2 << std::endl;

		if (k > 0.0)//Wenn Tangentensteig. gegen 0 geht ist entweder die
			return 1;// Geschwindigkeit zu hoch od. der Scheitel ist erreicht

		d = A1 - k * Entfernung1;

		Entfernung1 = Entfernung2;
		A1 = A2;

		Entfernung2 = -d / k; //new, more accurate range
		//std::cout << Entfernung2 << ", " << A1 << std::endl;
		if (Entfernung2 <= 0.0)
			return 1;
		if (fabs(A1) < 0.00001)//Abweichung von der 0-Linie
		{
			_endparam = endparam;
			_beginparam = beginparam;
			success = true;
			break;
		}
	}
	//std::cout << "fertig" << std::endl;
	if (success == false)
		return 1;

	return 0; // Bei 100 Durchläufe wir 0 zurckgegeben (Fehler)
}
/**
* @brief @see \fn flightparam_at_zero_range
*
*  @param   flight_snapshot&, flight_snapshot&, utility::vector_3d, flightparam
*
*  @return  int
*
*/
int crossbow::flightparam_at_init_range(flight_snapshot & _endparam,
		flight_snapshot & _beginparam, utility::vector_3d sight_setting,
		flightparam fp) {
	double Entfernung1;
	double Entfernung2;
	double A1;//Einstellwert
	double A2;
	double k, d;//Tangentensteigung, Tangentenabst. auf der y-Achse
	int iZaehler;
	int iFehler;
	errno = 0;
	bool success = false;
	utility::vector_3d setting;
	flight_snapshot endparam, beginparam;
	sight_setting /= sight_setting.x;

	/** only for testing

	 plot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 1:2  with linespoints;

	 */
	//		double velo = 80.0;
	//		for (double entf = 1.0; entf < 200.0; entf += 2.0) {
	//			int error = this->sight_setting(setting, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
	//					endparam, beginparam, entf, //m
	//					velo, //m/s
	//					scso // Koordinatensystemverschiebung vom Armbrustkoordinatensystem aus
	//					);
	//			setting /= setting.x;
	//			setting.y -= sight_setting.y;
	//			std::cout << entf << ",  " << setting.y << std::endl;
	//		}

	Entfernung1 = 0.001; //Startwert

	iFehler = this->sight_setting(setting, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
			endparam, beginparam, Entfernung1, //m
			fp);

	setting /= setting.x;
	setting.y -= sight_setting.y;
	A1 = setting.y;

	Entfernung2 = Entfernung1 + 0.001;
	for (iZaehler = 1; iZaehler < 50; iZaehler++) {
		iFehler = this->sight_setting(setting, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
				endparam, beginparam, Entfernung2, //m
				fp);
		if (0 != iFehler)
			return iFehler;
		setting /= setting.x;
		setting.y -= sight_setting.y;
		A2 = setting.y;

		if ((Entfernung1 - Entfernung2) == 0.0)
			return 1;
		k = (A1 - A2) / (Entfernung1 - Entfernung2);
		if (k < 0.0)//Wenn Tangentensteig. gegen 0 geht ist entweder die
			return 1;// Geschwindigkeit zu hoch od. der Scheitel ist erreicht

		d = A1 - k * Entfernung1;

		Entfernung1 = Entfernung2;
		A1 = A2;

		Entfernung2 = -d / k; //new, more accurate range
		if (Entfernung2 <= 0.0)
			return 1;
		if (fabs(A1) < 0.00001)//Abweichung von der 0-Linie
		{
			_endparam = endparam;
			_beginparam = beginparam;
			success = true;
			break;
		}
	}
	if (success == false)
		return 1;

	return 0; // Bei 100 Durchläufe wir 0 zurckgegeben (Fehler)
}

int crossbow::sight_setting(utility::vector_3d & targetpoint_from_sight, // Richtungsvector vom zielgerätkoordinatensystem aus
		utility::vector_3d & targetpoint_from_crossbow, // Richtungsvector vom Armbrustkoordinatensystem aus
		flight_snapshot & endparam, flight_snapshot & beginparam, double range, //m
		flightparam fp, double v) {

	Ballistic b(fp);

	int error = b.calc_release_parameter(beginparam, //rad Vorhaltewinkel in rad
			endparam, // Endwerte
			range, //m
			v //m/s
			);
	if (error)
		return error;
	sight_coordinate_system_origin scso = m_scso;
	scso.rot *= -1.0;
	scso.trans *= -1.0;

	// Zielpunkt  vom globalen Koordinatensystem aus. (Nullpunkt der Armbrust ist der globale Nullpunkt und die x-Achse
	// schaut genau in Richtung zum Ziel.
	// Die x_achse ist genau die entfernung, und die beiden anderen
	// Achsen sind Null, was aber nicht genau stimmt, wegen den Rechenfehler.
	// Eigentlich müsste endparam.s genau diese Werte beinhalten.
	utility::vector_3d targetpoint;
	//targetpoint.x = endparam.s;
	targetpoint.x = range;
	targetpoint.y = 0.0;
	targetpoint.z = 0.0;

	/*
	 Weltkoordinatensystem umwandeln Armbrustkoordiantensystem durch
	 drehen, daß die X-Achse genau in Abschussrichtung zeigt:
	 */
	// Ist für die Verdrehwinkel aller 3 Achsen.
	utility::vector_3d verdrehwinkel = utility::rotate2xAxis(beginparam.v);
	utility::rotate(targetpoint, verdrehwinkel);
	targetpoint_from_crossbow = targetpoint;

	/*
	 Armbrustkoordinatensystem umwandeln in Visierkoordinatensystem
	 */

	/** @todo testen ob die Reihenfolge egal ist, und ob das überhaupt stimmen kann
	 eigentlich müsste zuerst die translation kommen, weil dabei der 0-punkt seine Position ändert,
	 was bei einer Drehung nicht der Fall ist.
	 */
	utility::translate(targetpoint, scso.trans);
	utility::rotate(targetpoint, scso.rot);

	targetpoint_from_sight = targetpoint;
	return 0;

}



int crossbow::sight_setting(utility::vector_3d & targetpoint_from_sight, // Richtungsvector vom zielgerätkoordinatensystem aus
		flight_snapshot & endparam, flight_snapshot & beginparam, double range, //m
		flightparam fp, double v) {

	utility::vector_3d targetpoint_from_crossbow;// not used
	return this->sight_setting(targetpoint_from_sight,
			targetpoint_from_crossbow, endparam, beginparam, range, fp, v);
}

int crossbow::sight_setting(utility::vector_3d & targetpoint_from_sight, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
		flight_snapshot & endparam, flight_snapshot & beginparam, double range, //m
		flightparam fp) {
	if (fp.arrow.m == 0.0) {
		return 1;
	}
	utility::vector_3d targetpoint_from_crossbow;// not used
	double v = sqrt(2.0 * m_energy / fp.arrow.m);
	return crossbow::sight_setting(targetpoint_from_sight,
			targetpoint_from_crossbow, endparam, beginparam, range, //m
			fp, v);

}

int crossbow::sight_setting(utility::vector_3d & targetpoint_from_sight,
		utility::vector_3d & targetpoint_from_crossbow,
		flight_snapshot & endparam, flight_snapshot & beginparam, double range, //m
		flightparam fp) {
	if (fp.arrow.m == 0.0) {
		return 1;
	}
	double v = sqrt(2.0 * m_energy / fp.arrow.m);
	return crossbow::sight_setting(targetpoint_from_sight,
			targetpoint_from_crossbow, endparam, beginparam, range, //m
			fp, v);

}

/**
 *
 calcualte the muzzle velocity of an arrow.
 It need the sight settings for 2 different ranges.
 It also calculate the angle of the sight to the crossbow.
 There are 2 loops, the outer loop improve the angle of the sight.
 The inner loop improve the muzzle velocity of the arrow.


 Berechnet die Geschwindigkeit nach dem Sekantenverfahren.
 \see sight_coordinate_system_origin
 plot "~/kdeproj_neu/arrowmatcher_seitenwind/debug/src/output.csv" using 2:1 title 'Pfeildrehung-z' with linespoints;


 * @param _v          function writes the velocity in this field [m/s]
 * @param _scso_rot   the function writes a improved value in this field [rad]
 * @param _small_range    small range (global coordinaten system) [m]
 * @param _wide_range    large range (global coordinaten system) [m]
 * @param _setting_small_range   setting for the small range [m/m]
 * @param _setting_wide_range   setting for the large range [m/m]
 * @param _devitation_small_range   devitation of the impactpoint from the target system,
 *                             therefore x have to be zero! [m]
 * @param _devitation_wide_range   devitation of the impactpoint from the target system,
 *                             therefore x have to be zero! [m]
 *
 * @param fp          Flight parameter @see flightparam
 * @return            0 if no error
 */
int crossbow::calc_velocity(double & _v, //m/s
		utility::vector_3d & _scso_rot, //angle
		double _small_range, //m
		double _wide_range, //m
		utility::vector_3d _setting_small_range, //m/m
		utility::vector_3d _setting_wide_range, //m/m
		utility::vector_3d _devitation_small_range, // m
		utility::vector_3d _devitation_wide_range, // m
		flightparam _fp) {
	double v1, v2, v_new, y1, y2, k, d;
	int iFehler;
	crossbow c;
	bool scso_rot_is_ok = false;
	bool velocity_is_ok = false;
	c = *this;
	// set this value to zero at start, because this will recalculated this value later:
	c.m_scso.rot.x = 0.0;
	c.m_scso.rot.y = 0.0;
	c.m_scso.rot.z = 0.0;

	_setting_small_range /= _setting_small_range.x;
	_setting_wide_range /= _setting_wide_range.x;

	errno = 0;
	flight_snapshot endparam, beginparam;
	utility::vector_3d setting_small_1, setting_wide_1, setting_small_2,
			setting_wide_2, setting_small_real, setting_wide_real;

	for (int i = 0; i < 30; i++) {

		if (_wide_range < 350.0)
			v1 = 10.0 + _wide_range;//Startwert; durch probieren daraufgekommen
		else
			v1 = 350.0;
		v2 = v1 * 1.1;

		iFehler = c.sight_setting(setting_small_1, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
				endparam, beginparam, _small_range, //m
				_fp, v1);
		if (0 != iFehler)
			return iFehler;

		iFehler = c.sight_setting(setting_wide_1, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
				endparam, beginparam, _wide_range, //m
				_fp, v1);
		if (0 != iFehler)
			return iFehler;

		v2 = v1 * 1.1;
		//bool max_iterations = true;
		velocity_is_ok = false;
		for (int iterations = 1; iterations < 100; iterations++) {
			iFehler = c.sight_setting(setting_small_2, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
					endparam, beginparam, _small_range, //m
					_fp, v2);
			if (0 != iFehler)
				return iFehler;

			iFehler = c.sight_setting(setting_wide_2, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
					endparam, beginparam, _wide_range, //m
					_fp, v2);
			if (0 != iFehler)
				return iFehler;
			//////////////////////////////////////////////////
			// The real setting of the sight
			// include the devitation. (sight coordinaten system)
			setting_small_real.x = setting_small_2.x;
			setting_small_real.y = _setting_small_range.y * setting_small_2.x
					+ _devitation_small_range.y;
			setting_small_real.z = _setting_small_range.z * setting_small_2.x
					+ _devitation_small_range.z;

			setting_wide_real.x = setting_wide_2.x;
			setting_wide_real.y = _setting_wide_range.y * setting_wide_2.x
					+ _devitation_wide_range.y;
			setting_wide_real.z = _setting_wide_range.z * setting_wide_2.x
					+ _devitation_wide_range.z;
			////////////////////////////////////////////////////////
			//			setting_small_real /= setting_small_real.x;
			//			setting_wide_real /= setting_wide_real.x;
			//			setting_wide_1 /= setting_wide_1.x;
			//			setting_wide_2 /= setting_wide_2.x;
			//			setting_small_1 /= setting_small_1.x;
			//			setting_small_2 /= setting_small_2.x;


			y1 = (setting_wide_1.y - setting_small_1.y) - (setting_wide_real.y
					- setting_small_real.y);
			y2 = (setting_wide_2.y - setting_small_2.y) - (setting_wide_real.y
					- setting_small_real.y);

			k = y1 / (v1 - v2) - y2 / (v1 - v2); //'Tangentensteigung
			d = y1 - k * v1;
			if (k <= 0.0)
				return 1;

			v_new = -d / k;

			if (v_new < (0.2 * v2))
				return 2;
			if (fabs(y1) < 0.0000001 || fabs(y2) < 0.0000001) //((v2 / v1) < 1.01 && (v2 / v1) > 0.99)
			{
				//max_iterations = false;
				velocity_is_ok = true;
				//_v = v_new;
				break;
			}
			v1 = v2;
			v2 = v_new;
			setting_small_1 = setting_small_2;
			setting_wide_1 = setting_wide_2;

		}

		if (!velocity_is_ok) {
			return 3;
		}
		/////////////////////////////////////////////////////////////
		// improve the inclination of the sight (the zero axis)
		utility::vector_3d small_diff;
		utility::vector_3d wide_diff;
		utility::vector_3d delta_diff;

		small_diff = setting_small_2 / setting_small_2.x - setting_small_real
				/ setting_small_real.x;
		wide_diff = setting_wide_2 / setting_wide_2.x - setting_wide_real
				/ setting_wide_real.x;
		// middlevalue:
		delta_diff = (small_diff + wide_diff) / 2.0;
		c.m_scso.rot.z = c.m_scso.rot.z + delta_diff.y;
		c.m_scso.rot.y = c.m_scso.rot.y - delta_diff.z;

		//		double small_diff_y = setting_small_2.y / setting_small_2.x - setting_small_real.y / setting_small_real.x;
		//		double wide_diff_y =  setting_wide_2.y / setting_wide_2.x - setting_wide_real.y / setting_wide_real.x;
		//		double middle_diff_y = (small_diff_y + wide_diff_y) / 2.0;
		//c.m_scso.rot.z = c.m_scso.rot.z + middle_diff_y;
		if (fabs(delta_diff.y) < 0.00001) {
			scso_rot_is_ok = true;
			break;
		}

	}
	if (!scso_rot_is_ok) {
		return 5;
	}
	_scso_rot = c.m_scso.rot;
	_v = v_new;
	return 0;
}

int crossbow::calc_sight_inclination(utility::vector_3d & _scso_rot, //angle
		double _range, //m
		utility::vector_3d _setting, //m/m
		utility::vector_3d _devitation, //m (from the targetpoint system, x is zero)
		flightparam _fp) {

	double v;
	int iFehler;
	crossbow c;
	bool scso_rot_is_ok = false;
	c = *this;
	v = sqrt(2.0 * c.m_energy / _fp.arrow.m);
	c.m_scso.rot *= 0.0;// set all values to zero

	_setting /= _setting.x;

	errno = 0;
	flight_snapshot endparam, beginparam;
	utility::vector_3d setting, setting_real;

	for (int i = 0; i < 30; i++) {
		iFehler = c.sight_setting(setting, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
				endparam, beginparam, _range, //m
				_fp, v);
		if (0 != iFehler)
			return iFehler;
		//////////////////////////////////////////////////
		// The real setting of the sight
		// include the devitation. (sight coordinaten system)
		setting_real.x = setting.x;
		setting_real.y = _setting.y * setting.x + _devitation.y;
		setting_real.z = _setting.z * setting.x + _devitation.z;

		/////////////////////////////////////////////////////////////
		// improve the inclination of the sight (the zero axis)
		utility::vector_3d diff;

		diff = setting / setting.x - setting_real / setting_real.x;

		c.m_scso.rot.z = c.m_scso.rot.z + diff.y;
		c.m_scso.rot.y = c.m_scso.rot.y - diff.z;

		if (fabs(diff.y) < 0.00001) {
			scso_rot_is_ok = true;
			break;
		}

	}
	if (!scso_rot_is_ok) {
		return 5;
	}
	_scso_rot = c.m_scso.rot;
	return 0;

}

/**
 *
 */
int crossbow::calc_max_range(utility::vector_3d & setting,
		flight_snapshot & begin_fs, flight_snapshot & end_fs, flightparam fp) {
	int iFehler;
	double v = sqrt(2.0 * this->m_energy / fp.arrow.m);
	Ballistic b(fp);
	iFehler = b.calc_max_range(begin_fs, end_fs, v);
	if (iFehler) {
		return iFehler;
	}
	sight_coordinate_system_origin scso = m_scso;
	scso.rot *= -1.0;
	scso.trans *= -1.0;

	// Zielpunkt  vom globalen Koordinatensystem aus. (Nullpunkt der Armbrust ist der globale Nullpunkt und die x-Achse
	// schaut genau in Richtung zum Ziel.
	// Die x_achse ist genau die entfernung, und die beiden anderen
	// Achsen sind Null, was aber nicht genau stimmt, wegen den Rechenfehler.
	utility::vector_3d targetpoint = end_fs.s;

	/*
	 Weltkoordinatensystem umwandeln Armbrustkoordiantensystem durch
	 drehen, daß die X-Achse genau in Abschussrichtung zeigt:
	 */
	// Ist für die Verdrehwinkel aller 3 Achsen.
	utility::vector_3d verdrehwinkel = utility::rotate2xAxis(begin_fs.v);
	utility::rotate(targetpoint, verdrehwinkel);

	/*
	 Armbrustkoordinatensystem umwandeln in Visierkoordinatensystem
	 */

	/** @todo testen ob die Reihenfolge egal ist, und ob das überhaupt stimmen kann
	 eigentlich müsste zuerst die translation kommen, weil dabei der 0-punkt seine Position ändert,
	 was bei einer Drehung nicht der Fall ist.
	 */
	utility::translate(targetpoint, scso.trans);
	utility::rotate(targetpoint, scso.rot);

	// umwandeln, daß die x-Achse genau 1.0 ist:
	//targetpoint.y /= targetpoint.x;
	//targetpoint.z /= targetpoint.x;
	//targetpoint.x = 1.0;

	setting = targetpoint;
	return 0;
}
/**
 *
 * @param global_table
 * @param crossbow_table
 * @param sight_table
 * @param sight_setting
 * @param line_of_sight_table
 * @param max_time
 * @param max_range
 * @param fp
 * @return  zero if success
 */
int crossbow::calc_flight_table(std::vector<flight_snapshot> &global_table,
		std::vector<flight_snapshot> &crossbow_table, std::vector<
				flight_snapshot> &sight_table,
		std::vector<flight_snapshot> &line_of_sight_table,

		double max_time, // s
		double max_range, // m in x-Axis
		double zero_range, flightparam fp) {
	global_table.clear();
	crossbow_table.clear();
	sight_table.clear();
	line_of_sight_table.clear();

	sight_coordinate_system_origin scso = m_scso;
	scso.rot *= -1.0;
	scso.trans *= -1.0;
	utility::vector_3d release_angle;
	//std::list<flight_snapshot> table;
	double v = sqrt(2.0 * m_energy / fp.arrow.m);
	Ballistic b(fp);
	b.calc_flight_table(global_table, max_time, max_range, zero_range, v);
	if (global_table.begin() == global_table.end()) {
		return 1;

	}
	// calc the angle of sight setting for let the line of sight the zero axis
	flight_snapshot dummy1, dummy2;
	utility::vector_3d setting;
	sight_setting(setting, dummy1, dummy2, zero_range, fp);
	utility::vector_3d setting_angle = utility::rotate2xAxis(setting);

	flight_snapshot begin = *(global_table.begin());

	release_angle = utility::rotate2xAxis(begin.v);
	flight_snapshot current;

	for (std::vector<flight_snapshot>::iterator iter = global_table.begin(); iter
			!= global_table.end(); iter++) {

		current = (*iter);

		utility::rotate(current.s, release_angle);
		utility::rotate(current.v, release_angle);
		utility::rotate(current.phi, release_angle);
		utility::rotate(current.omega, release_angle);

		crossbow_table.push_back(current);

		utility::translate(current.s, scso.trans);
		utility::translate(current.v, scso.trans);
		utility::translate(current.phi, scso.trans);
		utility::translate(current.omega, scso.trans);

		utility::rotate(current.s, scso.rot);
		utility::rotate(current.v, scso.rot);
		utility::rotate(current.phi, scso.rot);
		utility::rotate(current.omega, scso.rot);

		sight_table.push_back(current);

		utility::rotate(current.s, setting_angle);
		utility::rotate(current.v, setting_angle);
		utility::rotate(current.phi, setting_angle);
		utility::rotate(current.omega, setting_angle);

		line_of_sight_table.push_back(current);

	}

	return 0;

}

