/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "utility.h"
#include <math.h>
#include <stdlib.h>
#include <iomanip>

using namespace arw;

utility::vector_3d utility::operator+(const utility::vector_3d &v1, const utility::vector_3d &v2) {
	vector_3d temp = v1;
	temp += v2;
	return temp;
}
utility::vector_3d utility::operator-(const utility::vector_3d &v1, const utility::vector_3d &v2) {
	utility::vector_3d temp = v1;
	temp -= v2;
	return temp;
}
utility::vector_3d utility::operator*(const utility::vector_3d &v1, const utility::vector_3d &v2) {
	utility::vector_3d temp = v1;
	temp *= v2;
	return temp;
}
utility::vector_3d utility::operator/(const utility::vector_3d &v1, const utility::vector_3d &v2) {
	utility::vector_3d temp = v1;
	temp /= v2;
	return temp;
}

utility::vector_3d utility::operator+(const utility::vector_3d &v1, double d) {
	utility::vector_3d temp = v1;
	temp += d;
	return temp;
}
utility::vector_3d utility::operator-(const utility::vector_3d &v1, double d) {
	utility::vector_3d temp = v1;
	temp -= d;
	return temp;

}
utility::vector_3d utility::operator*(const utility::vector_3d &v1, double d) {
	utility::vector_3d temp = v1;
	temp *= d;
	return temp;

}
utility::vector_3d utility::operator/(const utility::vector_3d &v1, double d) {
	utility::vector_3d temp = v1;
	temp /= d;
	return temp;

}

std::ostream & utility::operator<<(std::ostream & os, const utility::vector_3d &v) {
	os << std::setw(15) << v.x << std::setw(15) << v.y << std::setw(15) << v.z;
	return os;
}

/* Linear interpolation is close enough */
double utility::linint(double x0, double y0, double x1, double y1, double x) {
	return (x - x0) * (y1 - y0) / (x1 - x0) + y0;
}

void utility::rotate(vector_3d & point, double axis[], double angle) {

	double r[3];
	r[0] = axis[0];
	r[1] = axis[1];
	r[2] = axis[2];

	double m[4][4];
	matrix_identity(m);
	double rot[4][4], p[4][4];

	utility::matrix_identity(rot);
	utility::matrix_rotate(m, r, angle);

	utility::matrix_product(p, m, rot);

	utility::point_transform(point, p);

}
void utility::rotate(utility::vector_3d & point, utility::vector_3d axis, double angle) {
	double _axis[3];
	_axis[0] = axis.x;
	_axis[1] = axis.y;
	_axis[2] = axis.z;
	utility::rotate(point, _axis, angle);
}
void utility::rotate(utility::vector_3d & point, utility::vector_3d axis) {
	double _axis[3];
	_axis[0] = axis.x;
	_axis[1] = axis.y;
	_axis[2] = axis.z;
	double angle = sqrt(axis.x * axis.x + axis.y * axis.y + axis.z * axis.z);
	utility::rotate(point, _axis, angle);
}

void utility::translate(utility::vector_3d & point, utility::vector_3d t) {
	double r[3];
	r[0] = t.x;
	r[1] = t.y;
	r[2] = t.z;

	double m[4][4];
	utility::matrix_identity(m);
	utility::matrix_translate(m, r);

	point_transform(point, m);

}

void utility::vector_normalize(utility::vector_3d & v) {
	double length = sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
	if (length <= 0.0) {
		return;
	}
	v.x /= length;
	v.y /= length;
	v.z /= length;
}
void utility::vector_normalize(double v[3]) {
	double length = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
	if (length <= 0.0) {
		return;
	}
	v[0] /= length;
	v[1] /= length;
	v[2] /= length;
}
/** @todo testing
 * angle between 2 vectors, Reihenfolge ist egal
 * @param p
 * @param q
 * @return  the angle
 */
double utility::angle(const utility::vector_3d &a, const utility::vector_3d &b) {
	double cosine = (a.x * b.x + a.y * b.y + a.z * b.z) / (sqrt(a.x * a.x + a.y
			* a.y + a.z * a.z) * sqrt(b.x * b.x + b.y * b.y + b.z * b.z));
	return acos(cosine);
}

/** cross product
 *  berechnet den Normalvector, der in die Richtung schaut,
 *  dass die umlaufrichtung im Uhrzeigersinn geht: p,q,Nullpunkt,
 *  wenn man in richtung des Normalvectors schaut.
 * @param p
 * @param q
 * @return normal vector
 */
utility::vector_3d utility::normal_vector(const utility::vector_3d &p, const utility::vector_3d &q) {
	utility::vector_3d v;
	v.x = p.y * q.z - p.z * q.y;
	v.y = p.z * q.x - p.x * q.z;
	v.z = p.x * q.y - p.y * q.x;
	return v;
}
// Liefert einen Vector, mit dem man den übergebenen Vector in die x-Achse drehen kann
utility::vector_3d utility::rotate2xAxis(const utility::vector_3d &v1) {
	//utility::vector_3d rot;
	utility::vector_3d rot;
	rot.x = 0.0;
	rot.y = 0.0;
	rot.z = 0.0;
	utility::vector_3d x_axis;
	x_axis.x = 1.0;
	x_axis.y = 0.0;
	x_axis.z = 0.0;

	// Vector hat Länge 0:
	if (v1.y == 0.0 && v1.z == 0.0 && v1.x == 0.0) {
		return rot;
	}

	//vector zeigt schon in x-Richtung:
	if (v1.y == 0.0 && v1.z == 0.0) {
		if (v1.x > 0.0) {
			return rot;
		} else {
			rot.y = utility::PI;// um 180° drehen
			return rot;

		}

	}

	rot = utility::normal_vector(x_axis, v1);
	double angle = utility::angle(x_axis, v1);
	//double angle = atan2(sqrt(v1.z*v1.z + v1.y*v1.y), v1.x);
	angle *= (-1.0);
	utility::vector_normalize(rot);

	rot.x *= angle;
	rot.y *= angle;
	rot.z *= angle;
	return rot;
}

/**
 * gts_matrix_rotate:
 * @m: a #GtsMatrix or %NULL.
 * @r: the rotation axis.
 * @angle: the angle (in radians) to rotate by.
 *
 * Initializes @m to a rotation matrix around @r by @angle.
 * Allocates a new matrix if @m is %NULL.
 *
 * Returns: the rotation matrix.
 */
void utility::matrix_rotate(double m[4][4], double r[3], double angle) {
	double c, c1, s;

	// Bei drehung um x-Achse: v[0] = 1, v[1]=0, v[2]=0
	// Einfach alle werte durch die Länge vom Vector dividieren, damit Vector eine Länge von 1 bekommt:
	utility::vector_normalize(r);

	c = cos(angle);
	c1 = 1. - c;
	s = sin(angle);

	m[0][0] = r[0] * r[0] * c1 + c;
	m[0][1] = r[0] * r[1] * c1 - r[2] * s;
	m[0][2] = r[0] * r[2] * c1 + r[1] * s;
	m[0][3] = 0.;

	m[1][0] = r[1] * r[0] * c1 + r[2] * s;
	m[1][1] = r[1] * r[1] * c1 + c;
	m[1][2] = r[1] * r[2] * c1 - r[0] * s;
	m[1][3] = 0.;

	m[2][0] = r[2] * r[0] * c1 - r[1] * s;
	m[2][1] = r[2] * r[1] * c1 + r[0] * s;
	m[2][2] = r[2] * r[2] * c1 + c;
	m[2][3] = 0.;

	m[3][0] = 0.;
	m[3][1] = 0.;
	m[3][2] = 0.;
	m[3][3] = 1.;

}

void utility::point_transform(utility::vector_3d & p, double m[4][4]) {
	double x, y, z;

	x = m[0][0] * p.x + m[0][1] * p.y + m[0][2] * p.z + m[0][3];
	y = m[1][0] * p.x + m[1][1] * p.y + m[1][2] * p.z + m[1][3];
	z = m[2][0] * p.x + m[2][1] * p.y + m[2][2] * p.z + m[2][3];
	p.x = x;
	p.y = y;
	p.z = z;
}

/**
 * gts_matrix_zero:
 * @m: a #GtsMatrix or $NULL.
 *
 * Initializes @m to zeros. Allocates a matrix if @m is %NULL.
 *
 * Returns: the zero'ed matrix.
 */
void utility::matrix_zero(double m[4][4]) {

	m[0][0] = m[1][0] = m[2][0] = m[3][0] = 0.;
	m[0][1] = m[1][1] = m[2][1] = m[3][1] = 0.;
	m[0][2] = m[1][2] = m[2][2] = m[3][2] = 0.;
	m[0][3] = m[1][3] = m[2][3] = m[3][3] = 0.;

}

/**
 * gts_matrix_identity:
 * @m: a #GtsMatrix or %NULL.
 *
 * Initializes @m to an identity matrix. Allocates a matrix if @m is %NULL.
 *
 * Returns: the identity matrix.
 */
void utility::matrix_identity(double m[4][4]) {
	matrix_zero(m);
	m[0][0] = m[1][1] = m[2][2] = m[3][3] = 1.;

}

/**
 * gts_matrix_product:
 * @m1: a #GtsMatrix.
 * @m2: another #GtsMatrix.
 *
 * Returns: a new #GtsMatrix, product of @m1 and @m2.
 */
void utility::matrix_product(double m[4][4], double m1[4][4], double m2[4][4]) {
	unsigned int i, j;

	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			m[i][j] = m1[i][0] * m2[0][j] + m1[i][1] * m2[1][j] + m1[i][2]
					* m2[2][j] + m1[i][3] * m2[3][j];

}

void utility::matrix_translate(double m[4][4], double t[3]) {
	utility::matrix_zero(m);
	m[0][3] = t[0];
	m[1][3] = t[1];
	m[2][3] = t[2];
	m[3][3] = 1.;
	m[0][0] = m[1][1] = m[2][2] = 1.;

}
