/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * CrossbowTreeModel.cpp
 *  Created on: 23.12.2010
 */


#include "CrossbowTreeModel.h"
#include <QDebug>

CrossbowTreeModel::CrossbowTreeModel(QObject *parent) :
	QAbstractItemModel(parent) {
	m_unitconverter = new UnitConverter();
	QList<QString> path;
	rootItem = new TreeItem<CArmbrust>(NULL);
	rootItem->isRootItem = true;
	rootItem->base_item = NULL;
	rootItem->parentItem = NULL;
	rootItem->base_item_list = &(this->m_list);
}

CrossbowTreeModel::~CrossbowTreeModel() {
	delete rootItem;
}

int CrossbowTreeModel::columnCount(const QModelIndex &parent) const {
	if (parent.isValid())
		return static_cast<TreeItem<CArmbrust>*> (parent.internalPointer())->columnCount();
	else
		return rootItem->columnCount();
}

QVariant CrossbowTreeModel::data(const QModelIndex &index, int role) const {
	if (!index.isValid())
		return QVariant();

	TreeItem<CArmbrust> *item = static_cast<TreeItem<CArmbrust>*> (index.internalPointer());
	if(role == Qt::DecorationRole){
		if(item->parentItem->isRootItem && index.column() == 0){
			QPixmap *image = new QPixmap(":/toolbar/tlb_crossbow.png");
			return QVariant(*image);
		}

	}
	if (role != Qt::DisplayRole)
		return QVariant();


	return item->data(index.column());
}

Qt::ItemFlags CrossbowTreeModel::flags(const QModelIndex &index) const {
	if (!index.isValid())
		return Qt::NoItemFlags;

	TreeItem<CArmbrust> *item = static_cast<TreeItem<CArmbrust>*> (index.internalPointer());
	if (item->isRootItem)
		return Qt::ItemIsEnabled;
	// item is an arrow:
	if (item->parentItem == this->rootItem)
		return Qt::ItemIsEnabled | Qt::ItemIsSelectable;

	return Qt::ItemIsEnabled;
}

QVariant CrossbowTreeModel::headerData(int section, Qt::Orientation orientation,
		int role) const {
	if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
		QVariant header;
		switch (section) {
		case 0:
			header = QString(tr("title"));
			break;
		case 1:
			header = QString(tr("value"));
			break;
		case 2:
			header = QString(tr("unit"));
			break;
		}
		return header;
		//return rootItem->data(section);
	}
	return QVariant();
}

QModelIndex CrossbowTreeModel::index(int row, int column,
		const QModelIndex &parent) const {
	if (!hasIndex(row, column, parent))
		return QModelIndex();

	TreeItem<CArmbrust> *parentItem;

	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<TreeItem<CArmbrust>*> (parent.internalPointer());

	TreeItem<CArmbrust> *childItem = parentItem->child(row);
	if (childItem)
		return createIndex(row, column, childItem);
	else
		return QModelIndex();
}

QModelIndex CrossbowTreeModel::parent(const QModelIndex &index) const {
	if (!index.isValid())
		return QModelIndex();

	TreeItem<CArmbrust> *childItem =
			static_cast<TreeItem<CArmbrust>*> (index.internalPointer());
	if (childItem == NULL)
		return QModelIndex();
	TreeItem<CArmbrust> *parentItem = childItem->parent();

	if (parentItem == rootItem)
		return QModelIndex();
	if (parentItem == NULL)
		return QModelIndex();

	return createIndex(parentItem->row(), 0, parentItem);
}

int CrossbowTreeModel::rowCount(const QModelIndex &parent) const {
	TreeItem<CArmbrust> *parentItem;
	if (parent.column() > 0)
		return 0;

	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<TreeItem<CArmbrust>*> (parent.internalPointer());

	int i = parentItem->childCount();
	return i;
}


bool CrossbowTreeModel::create_data(TreeItem<CArmbrust> *child, TreeItem<CArmbrust> *parent,
		CArmbrust *base_item, const QMap<QString, QVariant> &v) {

	QMap<QString, QVariant>::const_iterator i = v.constBegin();
	child->base_item = base_item;
	child->base_item_list = &m_list;
	child->isRootItem = false;
	child->parentItem = parent;

	QVariant entry = QString(tr("untitled"));
	child->itemData.append(entry);
	child->itemData.append(QString());
	child->itemData.append(QString());

	for (i = v.constBegin(); i != v.constEnd(); i++) {

		if (i.value().type() == QVariant::Map) {
			if( ! i.value().toMap()["hidden"].toInt()  ) {
				TreeItem<CArmbrust> *new_child = new TreeItem<CArmbrust>(child);
				if (create_data(new_child, child, base_item, i.value().toMap())) {

					child->childItems.append(new_child);
				}
			}
		} else {
			if (i.key() == QString("title")) {
				if (i.value().toString().length() > 0)
					child->itemData[0] = this->trUtf8(i.value().toString().toUtf8());

			}
			if (i.key() == QString("value")) {
				child->itemData[1] = i.value();
			}
			if (i.key() == QString("unit")) {
				if(m_unitconverter != NULL)
				{
					Unit unit;
					m_unitconverter->get_unit(unit, i.value().toString());
					child->itemData[2] = unit.m_symbol;
				}
			}
			if(i.key() == QString("hidden")) {
				//child->setHidden(true);
				if(i.value().toInt() == 1) {
					//child->setHidden(true);
				}
			}
		}
		//std::cout << i.key().toStdString().c_str() << " = " << i.value().toString().toStdString().c_str() << std::endl;

	}
	return true;
}

void CrossbowTreeModel::write_item(int &depth, TreeItem<CArmbrust> *item) {
	//std::cout << "depth =" << depth << "  " << std::endl;
	TreeItem<CArmbrust> *child;
	for (int i = 0; i < item->childItems.size(); i++) {
		child = item->childItems.value(i);
		int child_depth = depth + 1;
		write_item(child_depth, child);
	}

}
CArmbrust* CrossbowTreeModel::getItem(QModelIndex index) {
	if (!index.isValid())
		return NULL;

	TreeItem<CArmbrust> *item = static_cast<TreeItem<CArmbrust>*> (index.internalPointer());
	if (item == this->rootItem)
		return NULL;
	return item->base_item;

}
bool CrossbowTreeModel::setItem(QModelIndex index, CArmbrust &a) {
	if (!index.isValid())
		return false;

	TreeItem<CArmbrust> *item = static_cast<TreeItem<CArmbrust>*> (index.internalPointer());
	if (item == this->rootItem)
		return false;
	*(item->base_item) = a;
	QModelIndex begin = createIndex(item->row(), 0, rootItem);
	QModelIndex end = createIndex(item->row(), item->columnCount() - 1,
			rootItem);

	emit
	dataChanged(begin, end);

	return true;

}
bool CrossbowTreeModel::setItem(CArmbrust &a) {
	if (this->rootItem == NULL)
		return false;
	TreeItem<CArmbrust> *item = NULL;
	for (int i = 0; i < this->rootItem->childItems.size(); i++) {
		item = rootItem->childItems.value(i);
		if (item->base_item->getGuid() == a.getGuid()) {
			*(item->base_item) = a;
			item->childItems.clear();
			this->create_data(item, rootItem, item->base_item, item->base_item->m_data);
			//TreeItem<CArmbrust> *child = this->create_item(item->arrow, item, item->arrow->m_data);

			QModelIndex begin = createIndex(item->row(), 0, rootItem);
			QModelIndex end = createIndex(item->row(), item->columnCount(),
					rootItem);


			//emit dataChanged(begin, end);
			reset();// must call reset, beause all child items are deleted and new created in create_data

		}

	}
	return true;

}
bool CrossbowTreeModel::appendRow(CArmbrust &a) {
	//QModelIndex begin = createIndex(rootItem->childCount(),0, rootItem);
	//emit beginInsertRows(begin, rootItem->childCount(), rootItem->childCount());

	this->m_list.push_back(a);
	std::list<CArmbrust>::iterator iter = this->m_list.end();
	iter--;
	CArmbrust *ptr = &(*(iter));
	TreeItem<CArmbrust> *item = new TreeItem<CArmbrust>(this->rootItem);
	this->create_data(item, this->rootItem, ptr, ptr->m_data);
	this->rootItem->childItems.append(item);

	//emit endInsertRows();
	reset();// must call reset, beause all child items are  new created in create_data
	return true;

}
/**
 * @todo: the new element is only append to the list of arrows (m_list)
 * therefore m_list don't mirror the right order of elements!
 */
bool CrossbowTreeModel::insertRow ( int row, const QModelIndex & parent )
{

	if(parent.isValid()){
		TreeItem<CArmbrust> *item = static_cast<TreeItem<CArmbrust>*> (parent.internalPointer());
		if (item == NULL)
			return false;
		if(item->parentItem != this->rootItem)
			return false;
	}else{
		row = 0;

	}

	CArmbrust p;
	this->m_list.push_back(p);
	std::list<CArmbrust>::iterator iter = this->m_list.end();
	iter--;
	CArmbrust *ptr = &(*(iter));

	TreeItem<CArmbrust> *new_item = new TreeItem<CArmbrust>(this->rootItem);
	this->create_data(new_item, this->rootItem, ptr, ptr->m_data);
	this->rootItem->childItems.insert(row, new_item);
	return true;
}
bool CrossbowTreeModel::insertRows ( int row, int count, const QModelIndex & parent  )
{
	QModelIndex index = parent;;
	if (!parent.isValid()){
		index = QModelIndex();
		row = 0;
	}

	emit beginInsertRows(index, row, row + count -1);
	for(int i=0;i<count; i++){
		if(!this->insertRow(row, index))
			return false;
	}
	emit endInsertRows();
	reset();
	return true;

}

void CrossbowTreeModel::clear() {
	//qDeleteAll(this->rootItem->childItems);
	this->rootItem->base_item = NULL;
	this->rootItem->childItems.clear();
	reset();

}
bool CrossbowTreeModel::removeRow ( int row, const QModelIndex & parent )
{
	if (!parent.isValid())
		return false;

	TreeItem<CArmbrust> *item = static_cast<TreeItem<CArmbrust>*> (parent.internalPointer());
	if (item == NULL)
		return false;
	if(item->parentItem != this->rootItem)
		return false;

	item = this->rootItem->childItems.value(row);
	if(item->base_item == NULL)
		return false;


	this->m_list.remove(*(item->base_item));
	this->rootItem->childItems.removeAt(row);
	return true;

}
bool CrossbowTreeModel::removeRows ( int row, int count, const QModelIndex & parent  )
{
	if (!parent.isValid())
		return false;
	emit beginRemoveRows(parent, row, count);
	for(int i=0;i<count; i++){
		if(!this->removeRow(row, parent))
			return false;
	}
	emit endRemoveRows();
	reset();
	return true;
}

/**
* @brief test only the guid, the value must be a QVariant which includes a QString. Other parmas are ignored.
*
*  @param   const QModelIndex&, int, const QVariant&, int, Qt::MatchFlags
*
*  @return  QModelIndexList
*
*/
QModelIndexList CrossbowTreeModel::match ( const QModelIndex & start, int role, const QVariant & value, int hits , Qt::MatchFlags flags ) const
{
	QModelIndexList matching_list;
	if (this->rootItem == NULL)
		return matching_list;
	if(! value.canConvert<QString>()) {
		return matching_list;
	}
	QUuid guid(value.toString());
	TreeItem<CArmbrust> *item = NULL;
	for (int i = 0; i < this->rootItem->childItems.size(); i++) {
		item = rootItem->childItems.value(i);
		if (item->base_item->getGuid() == guid) {
			QModelIndex index = createIndex(item->row(), 0, rootItem);
			matching_list.append(index);
		}

	}
	return matching_list;


}

