/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * Datastore.cpp
 *  Created on: 30.12.2010
 */

#include "Datastore.h"
#include <QStringList>
#include <QList>
#include <QDebug>


Datastore::Datastore() {
	m_guid = QUuid::createUuid () ;

}
Datastore::Datastore(const Datastore & d) {
    if(this == &d)
        return ;
	m_data = d.m_data;
	m_guid = d.m_guid;
}
Datastore::Datastore(const QString guid) {
	m_guid = QUuid(guid);
	if(m_guid.isNull()){
		m_guid = QUuid::createUuid();
	}

}

Datastore::~Datastore() {
	// TODO Auto-generated destructor stub
}

Datastore & Datastore::operator = (const Datastore & a)
{
    if(this == &a)
        return *this;
	m_data = a.m_data;
    m_guid = a.m_guid;
    return *this;

}
/** only test, if the guid is the same
 * @param Datastore a
 *
 */
bool Datastore::operator == (const Datastore & a)
{
	if(m_guid != a.m_guid)
		return false;
	return true;
}

/**
* @brief copy all data, without the m_guid!
*
*  @param   const Datastore&
*
*  @return  Datastore&
*
*/
Datastore & Datastore::take_data(const Datastore & a) {
	   if(this == &a)
	        return *this;
		m_data = a.m_data;
	    return *this;
}
/**
* @brief set a value in the map, this is an overloaded function, @see Datastore#setValue
*
*  @param   QString       path     A string like: sight/screw/z
*  @param   QVariant      value    a QString or a QMap
*
*  @return  bool
*
*/
bool Datastore::setValue(QString path, QVariant value) {

	return this->setValue(path, value, this->m_data);
}
/**
* @brief set a value in the map
*
*  @param   QString       path     A string like: sight/screw/z
*  @param   QVariant      value
*  @param   QMap<QString,QVariant>&   data
*
*  @return  bool
*
*/
bool Datastore::setValue(QString path, QVariant value, QMap<QString, QVariant> & data) {
	QStringList list;
	//qDebug() << path;
	//qDebug() << data;
	list = path.split("/");
	if(list.isEmpty()) {
		if(value.canConvert<QMap<QString, QVariant> >()) {
			data = value.toMap();// root Element is a map
			return true;
		}
		return false;
	}
	QString key = list.takeFirst();
	//qDebug() << key;

	QMap<QString, QVariant> map;
	if(list.size() == 0) {
		//qDebug() << data;
		data[key] = value;
		//qDebug() << data;
	}else {
		map = data[key].toMap();
		bool ok = this->setValue(list.join("/"), value, map);
		if(!ok) {
			return false;
		}
		data[key] = map;
	}
	return true;


}

QVariant Datastore::getValue(QString path) {
	return getValue(path, m_data);
}

QVariant Datastore::getValue(QString path, QMap<QString, QVariant> & data) {
	QVariant ret;
	QStringList list;
	//qDebug() << path;
	//qDebug() << data;
	list = path.split("/");
	if(list.isEmpty()) {
		return data;
	}
	QString key = list.takeFirst();
	//qDebug() << key;

	if(list.size() == 0) {
		//qDebug() << data;
		return data[key];
		//qDebug() << data;
	}
	if(data[key].canConvert<QMap<QString, QVariant> >()) {
		QMap<QString, QVariant> map;
		map = data[key].toMap();
		return this->getValue(list.join("/"),  map);
	}else{
		return QVariant();
	}

}

int Datastore::removeAllChildren(QString path) {
	QVariant var = getValue(path);
	if( ! var.canConvert<QMap<QString, QVariant> >()) {
		return 0;
	}
	int count = 0;
	QMap<QString, QVariant> map = var.toMap();
	QMap<QString, QVariant>::iterator i ;
	for(i=map.begin(); i != map.end(); i++ ) {
		if( ! (*i).canConvert<QMap<QString, QVariant> >()) {
			continue;
		}

		map.erase(i);
		count++;
	}
	setValue(path, map);
	return count;
}

void Datastore::map2string(QString &str, const QMap<QString, QVariant> &v)
{

	QMap<QString, QVariant>::const_iterator i = v.constBegin();

	while (i != v.constEnd()) {
		 if(i.value().type() == QVariant::Map)
	     {
			 str = str + i.key() + QString("map ...\n");
			 map2string(str, i.value().toMap() );
	     }
	     else if(i.value().type() == QVariant::Double)
	     {
	    	 str = str + i.key() + QString(" = ") + QString().setNum(i.value().toDouble()) + QString("\n");
	     }
	     else if(i.value().type() == QVariant::Int)
	     {
	    	 str = str + i.key() + QString(" = ") + QString().setNum(i.value().toInt()) + QString("\n");
	     }
	     else if(i.value().type() == QVariant::String)
	     {
	    	 str = str + i.key() + QString(" = ") + i.value().toString() + QString("\n");
	     }
	     else
	     {
	    	 ++i;
	    	 continue;
	     }
	     ++i;
	 }
	return ;

}
