/***************************************************************************
 *   Copyright (C) 2011 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * ReticleView.cpp
 *  Created on: 04.01.2011
 */

#include "ReticleView.h"
#include <QGraphicsScene>
#include <QPainter>
#include <QStyleOption>
#include <QtGui>
#include <QLabel>
#include <QDebug>
#include "GraphicsOrthoLineItem.h"
#include <QGraphicsEllipseItem>

ReticleView::ReticleView(QWidget *parent , Qt::WindowFlags f ): QWidget(parent, f) {

	scene = new QGraphicsScene();
	scene->setSceneRect(QRectF(0.0, 0.0 , 200.0, 200.0));
	setting.x = 1.0;
	ok_y = false;
	ok_z = false;

	QGraphicsEllipseItem *circle = new QGraphicsEllipseItem();
	circle->setRect(0.0, 0.0, 200.0-1.0, 200.0-1.0);
	//QPen *circlepen = new QPen(QColor("black"), 1, Qt::SolidLine);
	//circle->setPen(*circlepen );
	circle->setBrush(Qt::white);
	circle->setZValue(-1.0);
	scene->addItem(circle);
	scene->setBackgroundBrush(Qt::black);
	connect(scene, SIGNAL(selectionChanged()),
        this, SLOT(selectionChanged()));

	QGraphicsView *view = new QGraphicsView(scene);
	view->setMinimumSize(205, 205);// etwas grösser wie die scene, damit keine scrollbalken angezeigt werden.


	QHBoxLayout *layout = new QHBoxLayout;
	layout->addWidget(view);
	this->setLayout(layout);
	//this->layout()->heightForWidth(200);
	this->layout()->setSizeConstraint(QLayout::QLayout::SetFixedSize);


	 //QGraphicsItem *item = scene->itemAt(50, 50);
	 // item == rect
	 this->setHidden(false);
}

ReticleView::~ReticleView() {
	// TODO Auto-generated destructor stub
}
void ReticleView::itemSelected(QGraphicsItem *item) {

	//qDebug() << "itemSelected";

}
void ReticleView::selectionChanged() {
	//qDebug() << "selectionChanged";
	QList<QGraphicsItem *> list = scene->selectedItems();
	if(list.isEmpty())
		return;
	GraphicsOrthoLineItem *line = static_cast<GraphicsOrthoLineItem*>( list.takeFirst()) ;


	QList<QGraphicsItem *> allItems = scene->items();
	QList<QGraphicsItem *>::iterator i ;
	for(i=allItems.begin(); i != allItems.end(); i++ ) {
		//qDebug() << (*i)->type()  << " " << GraphicsOrthoLineItem::Type;
		if(  (*i)->type() !=  GraphicsOrthoLineItem::Type) {
			continue;
		}
		GraphicsOrthoLineItem *l = static_cast<GraphicsOrthoLineItem*>(*i) ;
		//qDebug() << l->getDirection()  << " " << line->getDirection();

		if(l->getDirection() == line->getDirection() ) {
			l->markAsSelected(false);
		}
	}
	line->markAsSelected(true);
	if(GraphicsOrthoLineItem::horizontal == line->getDirection() ) {
		setting.y = line->setting;
		ok_y = true;
	}else{
		setting.z = line->setting;
		ok_z = true;
	}


	//QPen Selected(QColor("green"), 0, Qt::SolidLine);
	//line->setPen(*line->PenSelected);



}
void ReticleView::setCrossbow(CArmbrust crossbow) {
	if( ! crossbow.m_Zielgeraet.m_iTyp == BENUTZERDEF)
		return;

	unsigned int i;
	double diagram_width = 200.0;
	double middle_axis = diagram_width / 2.0;
	double max_y, min_y, max_z, min_z;
	double fact_y, fact_z;

	for(i=0; i<crossbow.m_Zielgeraet.m_marks.size(); i++ ) {
		if(i == 0) {
			min_y = crossbow.m_Zielgeraet.m_marks[i].y;
			max_y = crossbow.m_Zielgeraet.m_marks[i].y;
			min_z = crossbow.m_Zielgeraet.m_marks[i].z;
			max_z = crossbow.m_Zielgeraet.m_marks[i].z;
		}
		if(min_y > crossbow.m_Zielgeraet.m_marks[i].y) {
			min_y = crossbow.m_Zielgeraet.m_marks[i].y;
		}
		if(max_y < crossbow.m_Zielgeraet.m_marks[i].y) {
			max_y = crossbow.m_Zielgeraet.m_marks[i].y;
		}
		if(min_z > crossbow.m_Zielgeraet.m_marks[i].z) {
			min_z = crossbow.m_Zielgeraet.m_marks[i].z;
		}
		if(max_z < crossbow.m_Zielgeraet.m_marks[i].z) {
			max_z = crossbow.m_Zielgeraet.m_marks[i].z;
		}
	}

	double max;
	if(fabs(max_y) > fabs(min_y))
		max = fabs(max_y);
	else
		max = fabs(min_y);
	if( fabs(max) < 0.0001) {
		fact_y = 10000000.0;
	}else{
		fact_y = (diagram_width - 50.0) / 2.0 /  max  ;
	}

	if(fabs(max_z) > fabs(min_z))
		max = fabs(max_z);
	else
		max = fabs(min_z);
	if( fabs(max) < 0.0001) {
		fact_z = 10000000.0;
	}else{
		fact_z = (diagram_width - 50.0) / 2.0 /  max  ;
	}

	if(fact_y > 100000.0)
		fact_y = 100000.0;
	if(fact_z > 100000.0)
		fact_z = 100000.0;

	double distance_z; // Distance zur Mittelachse
	double distance_y;

	for(i=0; i<crossbow.m_Zielgeraet.m_marks.size(); i++ ) {
		GraphicsOrthoLineItem *line = new GraphicsOrthoLineItem();
		distance_z = 0.0 - crossbow.m_Zielgeraet.m_marks[i].z;
		distance_y = 0.0 - crossbow.m_Zielgeraet.m_marks[i].y;


		line->setLine(  middle_axis + distance_z * fact_z, 0.0, diagram_width , GraphicsOrthoLineItem::vertical);
		line->number = i;
		line->setting = crossbow.m_Zielgeraet.m_marks[i].z;
		line->setFlags(QGraphicsItem::ItemIsSelectable);
		scene->addItem(line);

		line = new GraphicsOrthoLineItem();
		line->setLine(0.0,  middle_axis + distance_y * fact_y  , diagram_width  , GraphicsOrthoLineItem::horizontal);
		line->number = i;
		line->setting = crossbow.m_Zielgeraet.m_marks[i].y;
		line->setFlags(QGraphicsItem::ItemIsSelectable);
		scene->addItem(line);

	}


}
