/***************************************************************************
 *   Copyright (C) 2011 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * SightSettingTable.cpp
 *  Created on: 21.08.2009
 */

#include "SightSettingTable.h"
#include <QTableWidget>
#include <QPushButton>
#include <QLabel>
#include <QLineEdit>
#include <QVBoxLayout>
#include <QTextStream>

#include "arrowmatcherdoc.h"
#include "DlgCsvSettings.h"

SightSettingTable::SightSettingTable(QWidget *parent) :
	QWidget(parent) {
	m_stepsize = 5.0;
	m_pdoc = NULL;
	m_savetrajectory_settings.field_sep = ",";
	m_savetrajectory_settings.field_textquote = "\"";

}

SightSettingTable::~SightSettingTable() {

}
void SightSettingTable::update(ArrowmatcherDoc * doc) {
	if (doc == NULL)
		return;
	m_pdoc = doc;
	m_csv_content.clear();
	QTextStream csv_content(&(this->m_csv_content));
	QString sep = m_savetrajectory_settings.field_sep;
	QString quote = m_savetrajectory_settings.field_textquote;

	lbl_stepsize->setText(tr("stepsize") + QString(" [") + m_pdoc->m_unitconverter.get_curr_unit_sym("SI_metre") + QString("]"));

	m_stepsize = txt_stepsize->text().toDouble();
	Unit unit_range;
	// fetch the current Unit for SI unit: metre
	m_pdoc->m_unitconverter.get_current_unit_system(unit_range, "SI_metre");
	// convert the current unit to SI unit (metre)
	m_pdoc->m_unitconverter.convert_to_SI(m_stepsize, unit_range.m_type_id);




	utility::vector_3d setting;
	utility::vector_3d targetpoint_crossbow;
	utility::vector_3d velocity_range;

	double time;
	double range;
	double energy;
	double velocity;
	m_list_targetpoint_sight.clear();
	m_list_targetpoint_crossbow.clear();
	m_list_velocity.clear();
	m_list_time.clear();
	m_list_range.clear();

	QStringList headlist;
	headlist << tr("range") + QString(" [") + m_pdoc->m_unitconverter.get_curr_unit_sym("SI_metre") + QString("]")
			<< tr("vertical") + QString(" [") + m_pdoc->m_unitconverter.get_curr_unit_sym("millimetre_per_metre") + QString("]")
			<< tr("horizonal") + QString(" [") + m_pdoc->m_unitconverter.get_curr_unit_sym("millimetre_per_metre") + QString(" ]")
			<< tr("vertical setting") + QString(" [Clicks]")
			<< tr("horizontal setting") + QString(" [Clicks]")
			<< tr("drop")  + QString(" [") + m_pdoc->m_unitconverter.get_curr_unit_sym("millimetre") + QString("]")
			<< tr("winddrift") + QString(" [") + m_pdoc->m_unitconverter.get_curr_unit_sym("millimetre") + QString("]")
			<< tr("velocity") + QString(" [") + m_pdoc->m_unitconverter.get_curr_unit_sym("SI_metre_per_second") + QString("]")
			<< tr("energy") + QString(" [") + m_pdoc->m_unitconverter.get_curr_unit_sym("SI_joule") + QString("]")
			<< tr("time") + QString(" [s]");

	// the header line for the csv-file:
	QStringList::const_iterator constIterator;
	 for (constIterator = headlist.constBegin(); constIterator != headlist.constEnd();
			++constIterator) {
		 csv_content << quote << (*constIterator).toUtf8() << quote << sep;
	 }
	 csv_content << endl;

	double d = doc->m_curve_targetpoint_sight.minX();

	for (range = 0.0; range <= doc->m_curve_targetpoint_sight.maxX(); range
			+= m_stepsize) {
		if (range > doc->m_curve_targetpoint_sight.maxX() || range
				< doc->m_curve_targetpoint_sight.minX())
			continue;
		if (doc->m_curve_targetpoint_sight.spline(setting, range)) {
			// Fehler;
		}
		if (doc->m_curve_targetpoint_crossbow.spline(targetpoint_crossbow,
				range)) {
			// Fehler;
		}
		if (doc->m_curve_velocity.spline(velocity_range, range)) {
			// Fehler;
		}
		if (doc->m_curve_time.spline(time, range)) {
			// Fehler;
		}

		setting /= setting.x;
		m_list_targetpoint_sight.push_back(setting);
		m_list_targetpoint_crossbow.push_back(targetpoint_crossbow);
		m_list_velocity.push_back(velocity_range);
		m_list_time.push_back(time);

		m_list_range.push_back(range);
	}

	wnd_table->setRowCount(m_list_targetpoint_sight.size());
	wnd_table->setColumnCount(10);
	wnd_table->setHorizontalHeaderLabels(headlist);
	double value;
	Unit unit;
	m_pdoc->m_unitconverter.convert_to_current_system(value, unit, 1.0, "SI_metre");

	for (int row = 0; row < m_list_targetpoint_sight.size(); row++) {
		QTableWidgetItem* item = NULL;
		velocity = sqrt(m_list_velocity[row].x * m_list_velocity[row].x + m_list_velocity[row].y
				* m_list_velocity[row].y + m_list_velocity[row].z * m_list_velocity[row].z);
		energy = doc->m_Arrow.m * velocity * velocity / 2.0;

		value = m_pdoc->m_unitconverter.get_curr_unit_val(m_list_range[row], "SI_metre");
		item = new QTableWidgetItem(QString().sprintf("%03.4f",
				value));
		csv_content << quote << value << quote << sep;
		wnd_table->setItem(row, 0, item);

		value = m_pdoc->m_unitconverter.get_curr_unit_val(m_list_targetpoint_sight[row].y * 1000.0, "millimetre_per_metre");
		item = new QTableWidgetItem(QString().sprintf("%03.4f",
				value));
		csv_content << quote << value << quote << sep;
		wnd_table->setItem(row, 1, item);

		value = m_pdoc->m_unitconverter.get_curr_unit_val(m_list_targetpoint_sight[row].z * 1000.0, "millimetre_per_metre");
		item = new QTableWidgetItem(QString().sprintf("%03.4f",
				value));
		csv_content << quote << value << quote << sep;
		wnd_table->setItem(row, 2, item);

		if (doc->m_Crossbow.m_Zielgeraet.m_iTyp == KIMME_U_KORN) {
			value = m_list_targetpoint_sight[row].y	/ doc->m_Crossbow.m_Zielgeraet.m_graduation.y;
			item = new QTableWidgetItem(QString().sprintf("%03.4f",value));
			csv_content << quote << value << quote << sep;
			wnd_table->setItem(row, 3, item);

			value = m_list_targetpoint_sight[row].z
							/ doc->m_Crossbow.m_Zielgeraet.m_graduation.z;
			item = new QTableWidgetItem(QString().sprintf("%03.4f", value));
			csv_content << quote << value << quote << sep;
			wnd_table->setItem(row, 4, item);
		}else{
			csv_content << quote <<  quote << sep;
			csv_content << quote <<  quote << sep;
		}

		value = m_pdoc->m_unitconverter.get_curr_unit_val(m_list_targetpoint_crossbow[row].y * 1000.0, "millimetre");
		//qDebug() << "metric:" << m_list_targetpoint_crossbow[row].y * 1000.0 << " english:" << value << "\n";
		item = new QTableWidgetItem(QString().sprintf("%03.4f",
				value));
		csv_content << quote << value << quote << sep;
		wnd_table->setItem(row, 5, item);

		value = m_pdoc->m_unitconverter.get_curr_unit_val(m_list_targetpoint_crossbow[row].z * 1000.0, "millimetre");
		item = new QTableWidgetItem(QString().sprintf("%03.4f",
				value));
		csv_content << quote << value << quote << sep;
		wnd_table->setItem(row, 6, item);

		value = m_pdoc->m_unitconverter.get_curr_unit_val(velocity, "SI_metre_per_second");
		item = new QTableWidgetItem(QString().sprintf("%03.4f",
				value));
		csv_content << quote << value << quote << sep;
		wnd_table->setItem(row, 7, item);


		value = m_pdoc->m_unitconverter.get_curr_unit_val(energy, "SI_joule");
		item = new QTableWidgetItem(QString().sprintf("%03.4f",
				value));
		csv_content << quote << value << quote << sep;
		wnd_table->setItem(row, 8, item);

		value = m_list_time[row];
		item = new QTableWidgetItem(QString().sprintf("%03.4f", value));
		csv_content << quote << value << quote << sep;
		wnd_table->setItem(row, 9, item);

		csv_content << endl;


	}
	wnd_table->resizeRowsToContents();

}

void SightSettingTable::addView(MainWindow* mainwindow) {
	this->m_pControler = mainwindow;
	//mainwindow->addView(this);
	mainwindow->m_content_col_main->addTab(this, tr("Sight Setting Table"));

	//m_pdoc->m_unitconverter.m_system = Unit::english;

	cmdOk = new QPushButton(tr("OK"));
	cmdOk->setMaximumWidth(100);
	cmdOk->setIcon(QPixmap(":/toolbar/button_ok.png"));

	cmdCsvExport = new QPushButton(tr("export csv"));
	cmdCsvExport->setMaximumWidth(100);

	txt_stepsize = new QLineEdit(QString::number(m_stepsize, 'f', 2));
	txt_stepsize->setMaximumWidth(100);

	lbl_stepsize = new QLabel(tr("stepsize [m]"));

	this->wnd_table = new QTableWidget;
	wnd_table->setMinimumSize(500, 200);

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(lbl_stepsize);
	layout->addWidget(txt_stepsize);
	layout->addWidget(cmdOk);
	layout->addWidget(cmdCsvExport);
	layout->addWidget(wnd_table);

	layout->setSizeConstraint(QLayout::SetMinimumSize);

	this->setLayout(layout);

	QDoubleValidator *v = new QDoubleValidator(this);
	txt_stepsize->setValidator(v);

	connect(cmdOk, SIGNAL(clicked()), this, SLOT(on_cmdOK_clicked()));
	connect(cmdCsvExport, SIGNAL(clicked()), this,
			SLOT(on_cmdCsvExport_clicked()));

}
void SightSettingTable::on_cmdOK_clicked() {
	//get_controler()->slotUpdateAllViews(this);

	update(m_pdoc);

}


void SightSettingTable::on_cmdCsvExport_clicked() {
	DlgCsvSettings *dlg = new DlgCsvSettings(this);
	dlg->init(&(m_savetrajectory_settings.field_sep),
			&(m_savetrajectory_settings.field_textquote));
	if (dlg->exec() == QDialog::Rejected) {
		return;
	}

	update(m_pdoc);

	QString fileName = QFileDialog::getSaveFileName(this, tr(
			"QFileDialog::getOpenFileName()"), QString(), tr(
			"csv (*.csv);;Text (*.txt);;All Files (*)") );
	if (fileName.isEmpty()) {
		QMessageBox::information(this, tr("Info"), tr("No File"));
		return;
	}
	QFile outputfile(fileName);
	if (!outputfile.open(QIODevice::WriteOnly | QIODevice::Text)) {
		QMessageBox::information(this, tr("Info"), tr("Cannot open file"));
		return;
	}
	QTextStream out(&outputfile);
	//qDebug() << this->m_csv_content;
	out << this->m_csv_content;
	outputfile.close();

}
