/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "UnitConverter.h"
#include "ballistic/utility.h"
#include <QDebug>

#define UNIT(system,name,symbol,type_id,si_type_id,k,d)  { \
	unit.m_unitsystem = (system); \
	unit.m_name = (name); \
	unit.m_symbol = (symbol); \
    unit.m_type_id = (type_id); \
    unit.m_SI_type_id = (si_type_id); \
    unit.m_k = (k); \
	unit.m_d = (d); \
    m_units.push_back(unit); \
    }

/** @see http://en.wikipedia.org/wiki/Foot-pound_%28energy%29
 *
 *
 */
UnitConverter::UnitConverter() {
	this->m_system = Unit::metric;

	Unit unit;
	// SI Units and derived SI:
	UNIT(Unit::metric, tr("meter"), "m", "SI_metre", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("square meter"), QObject::trUtf8("m²"), "SI_square_metre", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("cubicmeter"), QObject::trUtf8("m³"), "SI_cubic_metre", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("meter per second"), "m/s", "SI_metre_per_second", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("meter per second squared"), QObject::trUtf8("m/s²"), "SI_metre_per_second_squared", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("kilogram"), "kg", "SI_kilogram", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("radian"), "rad", "SI_radian", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("radian per second"), "1/s", "SI_radian_per_second", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("radian per second squared"), QObject::trUtf8("1/s²"), "SI_radian_per_second_squared", "", 1.0, 0.0)
	UNIT(Unit::metric, tr("kilogram square meter"),QObject::trUtf8("kgm²"),"SI_kilogram_square_metre","",1.0,0.0)
	UNIT(Unit::metric, tr("meter per meter"),QObject::trUtf8("m/m"),"SI_metre_per_metre","",1.0,0.0)
	UNIT(Unit::metric, tr("joule"),QObject::trUtf8("J"),"SI_joule","",1.0,0.0)
	UNIT(Unit::metric, tr("kilogram per square meter"),QObject::trUtf8("kg/m²"),"SI_kilogram_per_square_metre","",1.0,0.0)

	// other metric units:
	UNIT(Unit::metric, tr("degree"),QObject::trUtf8("°"),"degree","SI_radian",arw::utility::PI/180.0, 0.0)
	UNIT(Unit::metric, tr("milliradian"), "millirad","milliradian","SI_radian", 0.001, 0.0)
	UNIT(Unit::metric, tr("gram"),"g","gram","SI_kilogram", 0.001 , 0.0)
	UNIT(Unit::metric, tr("millimeter"), "mm", "millimetre", "SI_metre", 0.001, 0.0)
	UNIT(Unit::metric, tr("centimeter"), "cm", "centimetre", "SI_metre", 0.01, 0.0)
	UNIT(Unit::metric, tr("square centimeter"), QObject::trUtf8("cm²"), "square_centimetre", "SI_square_metre", 0.01*0.01, 0.0)
	UNIT(Unit::metric, tr("square millimeter"), QObject::trUtf8("mm²"), "square_millimetre", "SI_suare_metre", 0.001*0.001, 0.0)
	UNIT(Unit::metric, tr("millimeter per meter"),QObject::trUtf8("mm/m"),"millimetre_per_metre","SI_metre_per_metre",0.001,0.0)
	UNIT(Unit::metric, tr("kilometer per hour"), "km/h", "kilometre_per_hour", "SI_metre_per_second", 0.277778,  0.0)
	UNIT(Unit::metric, tr("minute of angle"), "MOA", "minute", "SI_radian", arw::utility::PI / 180.0 / 60.0,  0.0)
	UNIT(Unit::metric, tr("gram square centimeter"), QObject::trUtf8("g cm²"), "gram_square_centimetre", "SI_kilogram_square_metre", 1.0e-7 ,0.0)

	// english Units:
	UNIT(Unit::english, tr("grain"),"grain","grain","SI_kilogram", 0.06479891/1000.0 , 0.0)
	UNIT(Unit::english, tr("pound"),"lbm","pound","SI_kilogram", 0.45359237 , 0.0)
	UNIT(Unit::english, tr("yard"),"yd","yard","SI_metre", 0.9144 , 0.0)
	UNIT(Unit::english, tr("foot"),"ft","foot","SI_metre", 0.3048 , 0.0)
	UNIT(Unit::english, tr("feet per second"),"ft/s","feet_per_second","SI_metre_per_second", 0.3048 , 0.0)
	UNIT(Unit::english, tr("inch"),"in","inch","SI_metre", 25.4*0.001 , 0.0)
	UNIT(Unit::english, tr("square inch"),QObject::trUtf8("in²"),"square_inch","SI_square_metre", pow(25.4*0.001,2.0) , 0.0)
	UNIT(Unit::english, tr("inch per yard"),QObject::trUtf8("in/yd"),"inch_per_yard","SI_metre_per_metre", 25.4 / 914.4   , 0.0)
	//UNIT(Unit::english, tr("inch per 100 yards"),QObject::trUtf8("in/100*yd"),"inch_per_100_yard","SI_metre_per_metre", 25.4 / (0.01 * 914.4) , 0.0)
	UNIT(Unit::english, tr("yard per yards"),QObject::trUtf8("yd/yd"),"yard_per_yard","SI_metre_per_metre", 1.0 , 0.0)
	UNIT(Unit::english, tr("miles per hour"), "mph", "miles_per_hour", "SI_metre_per_second", 0.44704, 0.0)
	UNIT(Unit::english, tr("Foot-pound"), "ft-lbf", "foot_pound", "SI_joule", 1.3558179483314, 0.0)
	UNIT(Unit::english, tr("pound per square inch"), QObject::trUtf8("lbm/in²"), "pound_per_square_inch", "SI_kilogram_per_square_metre", 0.45359237 / ((25.4*0.001)*(25.4*0.001)), 0.0)
	UNIT(Unit::english, tr("pound square foot"), QObject::trUtf8("lbm ft²"), "pound_square_foot","SI_kilogram_square_metre", 0.45359237 * pow(0.3048, 2.0), 0.0)
	UNIT(Unit::english, tr("pound square inch"), QObject::trUtf8("lbm in²"), "pound_square_inch","SI_kilogram_square_metre", 0.45359237 * pow(25.4*0.001, 2.0), 0.0)

}
// Methods
bool UnitConverter::convert(double & value, QString dest_id, QString src_id) {
	double new_value = value;
	if(dest_id == src_id)
		return true;
	if (!convert_to_SI(new_value, src_id))
		return false;
	if (!convert_from_SI(new_value, dest_id))
		return false;
	value = new_value;
	return true;

}
//
bool UnitConverter::convert_to_SI(double & value, QString src_id) {
	Unit unit;
	if (!this->get_unit(unit, src_id))
		return false;
	if (unit.m_SI_type_id == "")
		return true; // no need to convert, because it is a SI
	value = unit.m_k * value + unit.m_d;
	return true;
}
//
bool UnitConverter::convert_from_SI(double & value, QString dest_id) {
	Unit unit;
	if (!this->get_unit(unit, dest_id))
		return false;
	if (unit.m_k == 0.0)
		return false;
	value = (value - unit.m_d) / unit.m_k;
	return true;
}
//
bool UnitConverter::get_unit(Unit & unit, QString unittype) {

	for (int i = 0; i < m_units.size(); ++i) {
		if (m_units.at(i).m_type_id == unittype) {
			unit = m_units.at(i);
			return true;
		}
	}
	return false;
}
bool UnitConverter::get_current_unit_system(Unit & dest, QString src_unit_id) {
	Unit src_unit, SI_unit;
	if (!this->get_unit(src_unit, src_unit_id))
		return false;
	if (src_unit.m_unitsystem == this->m_system) {
		dest = src_unit; // no need to convert
		return true;
	}
	if (this->m_system == Unit::english) {
		if (src_unit.m_type_id == "gram")
			return get_unit(dest, "grain");
		if (src_unit.m_type_id == "SI_metre")
			return get_unit(dest, "yard");
		if (src_unit.m_type_id == "SI_metre_per_second")
			return get_unit(dest, "feet_per_second");
		if (src_unit.m_type_id == "millimetre")
			return get_unit(dest, "inch");
		if (src_unit.m_type_id == "square_millimetre")
			return get_unit(dest, "square_inch");
		if (src_unit.m_type_id == "millimetre_per_metre")
			return get_unit(dest, "inch_per_yard");
		if (src_unit.m_type_id == "metre_per_metre")
			return get_unit(dest, "yard_per_yard");
		if (src_unit.m_type_id == "kilometre_per_hour")
			return get_unit(dest, "miles_per_hour");
		if (src_unit.m_type_id == "SI_joule")
			return get_unit(dest, "foot_pound");


	}

	dest = src_unit;
	return true;

}
QString UnitConverter::get_curr_unit_sym(QString src_unit_id) {
	Unit dest_unit;
	if( ! get_current_unit_system(dest_unit, src_unit_id)) {
		return QString("?");
	}
	return dest_unit.m_symbol;
}
double UnitConverter::get_curr_unit_val(double value, QString src_unit_id) {
	double dest_value = value;
	Unit dest_unit;
	this->convert_to_current_system(dest_value, dest_unit, value, src_unit_id);
	return dest_value;
}
//
bool UnitConverter::convert_to_current_system(double & dest_value,
		Unit & dest_unit, double src_value, QString src_unit_id) {

	Unit src_unit;
	if (!this->get_unit(src_unit, src_unit_id))
		return false;

	if (!this->get_current_unit_system(dest_unit, src_unit_id))
		return false;
	dest_value = src_value;

	//qDebug() << dest_unit.m_SI_type_id << src_unit_id;
	if( ! this->convert(dest_value, dest_unit.m_type_id, src_unit_id))
		return false;
	return true;
}

QList<Unit> UnitConverter::get_units(QString unit_id)
{
	QList<Unit> list;
	Unit unit;
	if( ! this->get_unit(unit, unit_id)) {
			return list;
	}
	QString si_unit_id;
	if(unit.m_SI_type_id == QString("")) {
		si_unit_id = unit.m_type_id; // is a SI unit
	}else {
		si_unit_id = unit.m_SI_type_id;
	}
	for (int i = 0; i < m_units.size(); ++i) {
		if (m_units.at(i).m_SI_type_id == si_unit_id
				|| m_units.at(i).m_type_id == si_unit_id)
		{
			list.push_back(m_units.at(i));
		}
	}
	return list;
}
QList<UnitSystem>  UnitConverter::get_list_UnitSystem(){
	  UnitSystem sys;
	  QList<UnitSystem> list;
	  sys.m_name = tr("metric");
	  sys.m_system = Unit::metric;
	  list.push_back(sys);
	  sys.m_name = tr("english");
	  sys.m_system = Unit::english;
	  list.push_back(sys);
	  return list;
}
