/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include "armbrustdlg.h"

#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QRadioButton>
#include "calc_crosshair_dlg.h"
#include "berechnespindelsteigungdlg.h"
#include "calc_energy_from_velocitydlg.h"
#include "calc_velocity_dlg.h"
#include "calc_inclination_of_sight_dlg.h"
#include <QStringList>
#include <QTemporaryFile>
#include "UnitInputWidget.h"

ArmbrustDlg::ArmbrustDlg(CArmbrust &a, ArrowTreeModel *p, QWidget * parent,
		Qt::WFlags f) :
	QDialog(parent, f), m_Armbrust(a), m_Pfeilliste(p) {
	ui.setupUi(this);

	ui.lstRaster_1->setColumnCount(2);
	QStringList header;
	header << QString("Y") << QString("Z");
	ui.lstRaster_1->setHorizontalHeaderLabels(header);
	ui.lstRaster_1->setSelectionMode(QAbstractItemView::SingleSelection);
	ui.lstRaster_1->setSelectionBehavior(QAbstractItemView::SelectRows);

	ui.uinpScrewY->setUnitstyle(UnitInputWidget::symbol);
	ui.uinpScrewZ->setUnitstyle(UnitInputWidget::symbol);

	connect(ui.cmdTeilstrich_berechnen_y, SIGNAL(clicked()), this,
			SLOT(cmdTeilstrich_berechnen_y_click()));
	connect(ui.cmdTeilstrich_berechnen_z, SIGNAL(clicked()), this,
			SLOT(cmdTeilstrich_berechnen_z_click()));
	connect(ui.cmdRasterwinkel_berechnen, SIGNAL(clicked()), this,
			SLOT(cmdRaster_berechnen_click()));
	connect(ui.cmdEnergie_Korrekturwert, SIGNAL(clicked()), this,
			SLOT(cmdEnergie_Korrekturwert_click()));
	connect(ui.cmdEnergy, SIGNAL(clicked()), this, SLOT(cmdEnergie_click()));
	connect(ui.cmdKorrekturwert, SIGNAL(clicked()), this,
			SLOT(cmdKorrekturwert_click()));
	connect(ui.cmdRasterNeu, SIGNAL(clicked()), this,
			SLOT(cmdRasterNeu_click()));
	connect(ui.cmdRasterLoeschen, SIGNAL(clicked()), this,
			SLOT(cmdRasterLoeschen_click()));
	connect(ui.cmdSetMildots, SIGNAL(clicked()), this,
			SLOT(cmdSetMildots_click()));

	connect(ui.uinpPosY, SIGNAL(editingFinished()), this,
			SLOT(on_unit_editingFinished()));


	ausfuellen(&m_Armbrust);

	QPixmap *image = new QPixmap(":/images/SightInclination.png");
	ui.imgSightInclination->setPixmap(*image);
	image = new QPixmap(":/images/SightPos.png");
	ui.imgSightPos->setPixmap(*image);
	image = new QPixmap(":/images/telescopic_sight.png");
	ui.lbl_image_crosshairs->setPixmap(*image);
	image = new QPixmap(":/images/micro_adjust.png");
	ui.lbl_image_microadjust->setPixmap(*image);


	CArmbrust default_crossbow;
	ui.uinpPosX->setDefaultValue(default_crossbow.m_scso.trans.x);
	ui.uinpPosY->setDefaultValue(default_crossbow.m_scso.trans.y);
	ui.uinpPosZ->setDefaultValue(default_crossbow.m_scso.trans.z);

	ui.uinpRotY->setDefaultValue(default_crossbow.m_scso.rot.y);
	ui.uinpRotZ->setDefaultValue(default_crossbow.m_scso.rot.z);

	ui.uinpEnergy->setDefaultValue(default_crossbow.m_energy);

	ui.uinpScrewY->setDefaultValue(default_crossbow.m_Zielgeraet.m_graduation.y);
	ui.uinpScrewZ->setDefaultValue(default_crossbow.m_Zielgeraet.m_graduation.z);


	gnuplot = NULL;

}

/**liest das Dialogfeld aus und speichert die Werte in Armbrust*/
int ArmbrustDlg::einlesen(CArmbrust &Armbrust) {

	double value;
	QString unit;

	Armbrust.setValue("title", ui.txtName->text());



	value = ui.uinpEnergy->text().toDouble();
	unit = ui.uinpEnergy->getUnitId();
	Armbrust.setValue("energy/status",(int)ui.uinpEnergy->getStatus() );
	Armbrust.setValue("energy/value", value);
	Armbrust.setValue("energy/unit", unit);



	// Translation
	value = ui.uinpPosX->text().toDouble();
	unit = ui.uinpPosX->getUnitId();
	Armbrust.setValue("scso/trans/x/status",(int)ui.uinpPosX->getStatus() );
	Armbrust.setValue("scso/trans/x/value", value);
	Armbrust.setValue("scso/trans/x/unit", unit);

	value = ui.uinpPosY->text().toDouble();
	unit = ui.uinpPosY->getUnitId();
	Armbrust.setValue("scso/trans/y/status",(int)ui.uinpPosY->getStatus());
	Armbrust.setValue("scso/trans/y/value", value);
	Armbrust.setValue("scso/trans/y/unit", unit);

	value = ui.uinpPosZ->text().toDouble();
	unit = ui.uinpPosZ->getUnitId();
	Armbrust.setValue("scso/trans/z/status",(int)ui.uinpPosZ->getStatus() );
	Armbrust.setValue("scso/trans/z/value", value);
	Armbrust.setValue("scso/trans/z/unit", unit);


	// Rotation
	value = ui.uinpRotY->text().toDouble();
	unit = ui.uinpRotY->getUnitId();
	Armbrust.setValue("scso/rot/y/status",(int) ui.uinpRotY->getStatus());
	Armbrust.setValue("scso/rot/y/value", value);
	Armbrust.setValue("scso/rot/y/unit", unit);

	value = ui.uinpRotZ->text().toDouble();
	unit = ui.uinpRotZ->getUnitId();
	Armbrust.setValue("scso/rot/z/status",(int)ui.uinpRotZ->getStatus() );
	Armbrust.setValue("scso/rot/z/value", value);
	Armbrust.setValue("scso/rot/z/unit", unit);




	value = ui.uinpScrewY->text().toDouble();
	unit = ui.uinpScrewY->getUnitId();
	Armbrust.setValue("sight/screw/y/status",(int)ui.uinpScrewY->getStatus() );
	Armbrust.setValue("sight/screw/y/value", value);
	Armbrust.setValue("sight/screw/y/unit", unit);

	value = ui.uinpScrewZ->text().toDouble();
	unit = ui.uinpScrewZ->getUnitId();
	Armbrust.setValue("sight/screw/z/status",(int) ui.uinpScrewZ->getStatus() );
	Armbrust.setValue("sight/screw/z/value", value);
	Armbrust.setValue("sight/screw/z/unit", unit);


	QMap<QString, QVariant> sight_type = Armbrust.m_data["sight"].toMap()["type"].toMap();
	if (ui.radSpindel->isChecked()) {
		//sight_type["value"] = QString("screw");
		Armbrust.setValue("sight/type/value", QString("screw"));
		Armbrust.setValue("sight/reticle/hidden", 1);
		Armbrust.setValue("sight/screw/hidden", 0);

		//Armbrust.m_Zielgeraet.m_iTyp = KIMME_U_KORN;
	} else {
		Armbrust.setValue("sight/type/value", QString("reticle"));
		Armbrust.setValue("sight/reticle/hidden", 0);
		Armbrust.setValue("sight/screw/hidden", 1);
		//Armbrust.m_Zielgeraet.m_iTyp = BENUTZERDEF;
	}



	Armbrust.removeAllChildren("sight/reticle");
	for (int row = 0; row < ui.lstRaster_1->rowCount(); row++) {
		QMap<QString, QVariant> mark;
		QMap<QString, QVariant> mark_y;
		QMap<QString, QVariant> mark_z;

		utility::vector_3d v;
		v.x = 1.0;

		UnitInputWidget *wy = static_cast<UnitInputWidget*>(ui.lstRaster_1->cellWidget(row, 0));
		UnitInputWidget *wz = static_cast<UnitInputWidget*>(ui.lstRaster_1->cellWidget(row, 1));

		mark["title"] = QString().setNum(row);
		mark["hidden"] = 0;

		value = wy->text().toDouble();
		unit = wy->getUnitId();
		mark_y["value"] = value;
		mark_y["unit"] = unit;
		mark_y["title"] = QString("Y");
		mark_y["status"] =  wy->getStatus();
		this->m_unitconverter.convert_to_SI(value, unit);
		v.y = value;

		value = wz->text().toDouble();
		unit = wz->getUnitId();
		mark_z["value"] = value;
		mark_z["unit"] = unit;
		mark_z["title"] = QString("Z");
		mark_z["status"] =  wz->getStatus();
		this->m_unitconverter.convert_to_SI(value, unit);
		v.z = value;

		mark["y"] = mark_y;
		mark["z"] = mark_z;

		Armbrust.setValue("sight/reticle/" + QString().number(row), mark);
	}

	//qDebug() << __LINE__ << " " << __FILE__ << " " <<Armbrust.toString();

	return 0;
}
void ArmbrustDlg::cmdRasterLoeschen_click() {

	ui.lstRaster_1->removeRow(ui.lstRaster_1->currentRow());
	ui.lstRaster_1->resizeRowsToContents();
	einlesen(m_Armbrust);
	this->ausfuellen(&m_Armbrust);
}

void ArmbrustDlg::cmdRasterNeu_click() {
	einlesen(m_Armbrust);
	QMap<QString, QVariant> reticle;
	int size = m_Armbrust.m_Zielgeraet.m_marks.size();
	int row = size;

	QMap<QString, QVariant> val_y, val_z, point;
	val_y["title"] = QString("Y");
	val_y["unit"] = "millimetre_per_metre";
	val_y["value"] = QString().number(0.0);// one Mil is a milliradian  @see http://en.wikipedia.org/wiki/Telescopic_sight#Mil-dot_reticles
	val_y["status"] = (int)UnitInputWidget::preset;
	val_y["hidden"] = 0;

	val_z["title"] = QString("Z");
	val_z["unit"] = "millimetre_per_metre";
	val_z["value"] = QString().number(0.0);
	val_z["status"] = (int)UnitInputWidget::preset;
	val_z["hidden"] = 0;

	point["title"] = QString().number(row);
	point["hidden"] = 0;
	point["y"] = val_y;
	point["z"] = val_z;

	m_Armbrust.setValue("sight/reticle/"+ QString().number(row ) , point);
	this->ausfuellen(&m_Armbrust);

}
/**
 * one Mil is a milliradian  @see http://en.wikipedia.org/wiki/Telescopic_sight#Mil-dot_reticles
 */
void ArmbrustDlg::cmdSetMildots_click() {
	einlesen(m_Armbrust);
	m_Armbrust.removeAllChildren("sight/reticle");

	//QMap<QString, QVariant> reticle;
	int row = 0;
	for(double d=5.0;d>= -5;d--) {

		QMap<QString, QVariant> val_y, val_z, point;
		val_y["title"] = QString("Y");
		val_y["unit"] = "millimetre_per_metre";
		val_y["value"] = QString().number(d);
		val_y["status"] = (int)UnitInputWidget::preset;
		val_y["hidden"] = 0;

		val_z["title"] = QString("Z");
		val_z["unit"] = "millimetre_per_metre";
		val_z["value"] = QString().number(d);
		val_z["status"] = (int)UnitInputWidget::preset;
		val_z["hidden"] = 0;

		point["title"] = QString().number(row);
		point["hidden"] = 0;
		point["y"] = val_y;
		point["z"] = val_z;

		//reticle[QString().number(row)] = point;
		m_Armbrust.setValue("sight/reticle/" + QString().number(row), point);

		row++;

	}
	//m_Armbrust.setValue("sight/reticle", reticle);

	this->ausfuellen(&m_Armbrust);
	//qDebug() << __LINE__ << " " << __FILE__ << " " << m_Armbrust.toString();


}


/**
* @brief writes the values of the m_data to the widgets of the dialog
*
*  @param   CArmbrust*
*
*  @return  int
*
*/
int ArmbrustDlg::ausfuellen(CArmbrust *ptrArmbrust) {

	if (ptrArmbrust == NULL) {
		return 0;
	}
	//qDebug() << __LINE__ << " " << __FILE__ << " " << ptrArmbrust->toString();

    ///////////////////////////////////////
	//
	// Write the values of the map to the widgets
	//
	QMap<QString, QVariant> map;

	ui.txtName->setText(ptrArmbrust->m_data["title"].toString() );

	double val;
	QString unit;
	int status;

	val = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["x"].toMap()["value"].toDouble();
	unit = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["x"].toMap()["unit"].toString();
	status = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["x"].toMap()["status"].toInt();
	ui.uinpPosX->setValue(val, unit, NULL, (UnitInputWidget::Value)status);

	val = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["y"].toMap()["value"].toDouble();
	unit = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["y"].toMap()["unit"].toString();
	status = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["y"].toMap()["status"].toInt();
	ui.uinpPosY->setValue(val, unit, NULL, (UnitInputWidget::Value)status);

	val = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["z"].toMap()["value"].toDouble();
	unit = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["z"].toMap()["unit"].toString();
	status = ptrArmbrust->m_data["scso"].toMap()["trans"].toMap()["z"].toMap()["status"].toInt();
	ui.uinpPosZ->setValue(val, unit, NULL, (UnitInputWidget::Value)status);


	val = ptrArmbrust->m_data["scso"].toMap()["rot"].toMap()["y"].toMap()["value"].toDouble();
	unit = ptrArmbrust->m_data["scso"].toMap()["rot"].toMap()["y"].toMap()["unit"].toString();
	status = ptrArmbrust->m_data["scso"].toMap()["rot"].toMap()["y"].toMap()["status"].toInt();
	ui.uinpRotY->setValue(val, unit, NULL, (UnitInputWidget::Value)status);

	val = ptrArmbrust->m_data["scso"].toMap()["rot"].toMap()["z"].toMap()["value"].toDouble();
	unit = ptrArmbrust->m_data["scso"].toMap()["rot"].toMap()["z"].toMap()["unit"].toString();
	status = ptrArmbrust->m_data["scso"].toMap()["rot"].toMap()["z"].toMap()["status"].toInt();
	ui.uinpRotZ->setValue(val, unit, NULL, (UnitInputWidget::Value)status);


	val = ptrArmbrust->m_data["energy"].toMap()["value"].toDouble();
	unit = ptrArmbrust->m_data["energy"].toMap()["unit"].toString();
	status = ptrArmbrust->m_data["energy"].toMap()["status"].toInt();
	ui.uinpEnergy->setValue(val, unit, NULL, (UnitInputWidget::Value)status);


	val = ptrArmbrust->m_data["sight"].toMap()["screw"].toMap()["y"].toMap()["value"].toDouble();
	unit = ptrArmbrust->m_data["sight"].toMap()["screw"].toMap()["y"].toMap()["unit"].toString();
	status = ptrArmbrust->m_data["sight"].toMap()["screw"].toMap()["y"].toMap()["status"].toInt();
	ui.uinpScrewY->setValue(val, unit, NULL, (UnitInputWidget::Value)status);

	val = ptrArmbrust->m_data["sight"].toMap()["screw"].toMap()["z"].toMap()["value"].toDouble();
	unit = ptrArmbrust->m_data["sight"].toMap()["screw"].toMap()["z"].toMap()["unit"].toString();
	status = ptrArmbrust->m_data["sight"].toMap()["screw"].toMap()["z"].toMap()["status"].toInt();
	ui.uinpScrewZ->setValue(val, unit, NULL, (UnitInputWidget::Value)status);

	ui.lstRaster_1->clearContents();

	ui.lstRaster_1->setRowCount(0);
	map =  ptrArmbrust->getValue("sight/reticle").toMap();

	for(int j=0; j<100;j++) {
		QString unitstr;
		double v;
		int status;
		if( ! map.contains(QString().number(j))) {
			break;
		}

		UnitInputWidget *w_y = new UnitInputWidget();
		UnitInputWidget *w_z = new UnitInputWidget();

		w_y->setUnitstyle(UnitInputWidget::symbol);
		w_z->setUnitstyle(UnitInputWidget::symbol);

		w_y->setDefaultValue(0.0);
		w_z->setDefaultValue(0.0);


		unitstr = ptrArmbrust->getValue("sight/reticle/" + QString().number(j) + "/y/unit").toString();
		v = ptrArmbrust->getValue("sight/reticle/" + QString().number(j) + "/y/value").toDouble();
		status = ptrArmbrust->getValue("sight/reticle/"+ QString().number(j) + "/y/status").toInt();
		w_y->setValue(v, unitstr, NULL, (UnitInputWidget::Value)status );

		unitstr = ptrArmbrust->getValue("sight/reticle/" + QString().number(j) + "/z/unit").toString();
		v = ptrArmbrust->getValue("sight/reticle/" + QString().number(j) + "/z/value").toDouble();
		status = ptrArmbrust->getValue("sight/reticle/"+ QString().number(j) + "/z/status").toInt();
		w_z->setValue(v, unitstr, NULL, (UnitInputWidget::Value)status );

		ui.lstRaster_1->insertRow(j);
		ui.lstRaster_1->setCellWidget(j,0,w_y);
		ui.lstRaster_1->setCellWidget(j,1,w_z);



	 }


	ui.lstRaster_1->resizeRowsToContents();

	QString type = ptrArmbrust->m_data["sight"].toMap()["type"].toMap()["value"].toString();
	if (type == QString("screw") ) {
		ui.radSpindel->setChecked(true);
	} else//BENUTERDEF
	{
		ui.radRaster->setChecked(true);
	}
	return 0;
}
void ArmbrustDlg::cmdEnergie_click() {
	einlesen(m_Armbrust);
	//CArmbrust temp_crossbow;
	Calc_Energy_from_VelocityDlg *dlg = new Calc_Energy_from_VelocityDlg(
			m_Pfeilliste, m_Armbrust, this);
	if (dlg->exec() != QDialog::Accepted)
		return;
	ausfuellen(&m_Armbrust);
}
void ArmbrustDlg::cmdKorrekturwert_click() {

	einlesen(m_Armbrust);
	calc_inclination_of_sight_dlg *d = new calc_inclination_of_sight_dlg(this);
	d->init(m_Armbrust, m_Pfeilliste);
	if (d->exec() != QDialog::Accepted)
		return;
	m_Armbrust = d->m_crossbow;
	ausfuellen(&m_Armbrust);
}
void ArmbrustDlg::cmdEnergie_Korrekturwert_click() {
	einlesen(m_Armbrust);

	calc_velocity_dlg *d = new calc_velocity_dlg(this);
	d->init(m_Armbrust, m_Pfeilliste);
	if (d->exec() != QDialog::Accepted)
		return;
	m_Armbrust = d->m_crossbow;
	ausfuellen(&m_Armbrust);

}
void ArmbrustDlg::accept() {
	einlesen(m_Armbrust);
	//qDebug() << m_Armbrust.toString();
	QDialog::accept();

}
void ArmbrustDlg::cmdTeilstrich_berechnen_y_click() {
	CArmbrust a = m_Armbrust;
	einlesen(a);
	BerechneSpindelsteigungDlg *dlg = new BerechneSpindelsteigungDlg( BerechneSpindelsteigungDlg::vertical,
			this);
	if (dlg->exec() == QDialog::Accepted) {
	    UnitConverter uv;
		QString unitstr = a.getValue("sight/screw/y/unit").toString();
		int status = a.getValue("sight/screw/y/status").toInt();
		double v = dlg->m_slope;
		uv.convert_from_SI(v, unitstr);
		ui.uinpScrewY->setValue(v, unitstr, NULL, UnitInputWidget::user);
	}

}
void ArmbrustDlg::cmdTeilstrich_berechnen_z_click() {
	CArmbrust a = m_Armbrust;
	einlesen(a);
	BerechneSpindelsteigungDlg *dlg = new BerechneSpindelsteigungDlg( BerechneSpindelsteigungDlg::horizontal,
			this);
	if (dlg->exec() == QDialog::Accepted) {
	    UnitConverter uv;
		QString unitstr = a.getValue("sight/screw/z/unit").toString();
		int status = a.getValue("sight/screw/z/status").toInt();
		double v = dlg->m_slope;
		uv.convert_from_SI(v, unitstr);
		ui.uinpScrewZ->setValue(v, unitstr, NULL, UnitInputWidget::user);
	}


}
void ArmbrustDlg::cmdRaster_berechnen_click() {
	CArmbrust a = m_Armbrust;
	einlesen(a);

	int row = ui.lstRaster_1->currentRow();
	if (row < 0) {
		QMessageBox::information(this, QString(), tr("No selected Value"),
				QMessageBox::Cancel, QMessageBox::NoButton);
		return;
	}

	calc_crosshair_dlg *dlg = new calc_crosshair_dlg(this);
	dlg->init(a, m_Pfeilliste);
	if (dlg->exec() != QDialog::Accepted)
		return;

	UnitInputWidget *wy = static_cast<UnitInputWidget*>(ui.lstRaster_1->cellWidget(row, 0));
	UnitInputWidget *wz = static_cast<UnitInputWidget*>(ui.lstRaster_1->cellWidget(row, 1));

    UnitConverter uv;

	QString unitstr;
	int status;
	double v;
	unitstr = a.getValue("sight/reticle/" + QString().number(row) + "/y/unit").toString();
	status = a.getValue("sight/reticle/" + QString().number(row) + "/y/unit").toInt();
	v = dlg->m_sight_setting.y / dlg->m_sight_setting.x;
	uv.convert_from_SI(v, unitstr);
	wy->setValue(v, unitstr, NULL, UnitInputWidget::user);

	unitstr = a.getValue("sight/reticle/" + QString().number(row) + "/z/unit").toString();
	status = a.getValue("sight/reticle/" + QString().number(row) + "/z/unit").toInt();
	v = dlg->m_sight_setting.z / dlg->m_sight_setting.x;
	uv.convert_from_SI(v, unitstr);
	wz->setValue(v, unitstr, NULL, UnitInputWidget::user);


}
void ArmbrustDlg::on_unit_editingFinished() {
	//qDebug() << "on_unit_editingFinished";

}


