/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "arrowmatcher_file.h"
#include <QMessageBox>
#include <QObject>
#include <QDebug>
#include "CrossbowTreeModel.h"
#include "ArrowTreeModel.h"
#include <QTextStream>
#include <ballistic/utility.h>

arrowmatcher_file::arrowmatcher_file() {
}

arrowmatcher_file::~arrowmatcher_file() {
}

/*!
 \fn arrowmatcher_file::read(QString filename)
 */
int arrowmatcher_file::read(QIODevice *device, ArrowmatcherDoc *doc) {
	QString errorStr;
	int errorLine;
	int errorColumn;
	QDomDocument domDocument;
	if (!domDocument.setContent(device, true, &errorStr, &errorLine,
			&errorColumn)) {
		QMessageBox::information(0, QObject::tr("Error"), QObject::tr(
				"Parse error at line %1, column %2:\n%3") .arg(errorLine) .arg(
				errorColumn) .arg(errorStr));
		return false;
	}

	QDomElement root = domDocument.documentElement();
	if (root.tagName() != "arrowmatcher") {
		QMessageBox::information(0, QObject::tr("Error"), QObject::tr(
				"The file is not an arrowmatcher file."));
		return false;
	} else if (root.hasAttribute("version") && root.attribute("version")
			!= "2.0") {
		QMessageBox::information(0, QObject::tr("Error"), QObject::tr(
				"The file is not an arrowmatcher version 2.0 "
					"file."));
		return false;
	}

	CArmbrust a;
	CPfeil p;
	CZielgeraet z;
	utility::vector_3d vector;

	doc->m_crossbow_list_model->clear();

	doc->m_arrow_list_model->clear();

	doc->m_Anglelist.clear();
	doc->m_Targetcircles.clear();

	QDomElement child = root.firstChildElement();
	while (!child.isNull()) {
		//std::string tag = child.tagName().latin1();
		//qDebug("Items in list: %s",tag.c_str() );
		if (child.tagName() == "crossbowlist") {
			QDomElement crossbow;
			crossbow = child.firstChildElement();
			while (!crossbow.isNull()) {
				if (0 == a.fromDomElement( crossbow)) {
					doc->m_crossbow_list_model->appendRow(a);

				}
				crossbow = crossbow.nextSiblingElement();
			}

		}
		if (child.tagName() == "arrowlist") {
			QDomElement arrow;
			arrow = child.firstChildElement();
			while (!arrow.isNull()) {
				if (0 == p.fromDomElement(arrow))
					doc->m_arrow_list_model->appendRow(p);

				arrow = arrow.nextSiblingElement();
			}
		}


		child = child.nextSiblingElement();
	}
	doc->m_Range = root.firstChildElement("range").text().toDouble();
	doc->m_Angle = root.firstChildElement("angle").text().toDouble();
	doc->m_windspeed = root.firstChildElement("windspeed").text().toDouble();
	doc->m_windangle = root.firstChildElement("windangle").text().toDouble();

	child = root.firstChildElement("arrow");
	//readArrow(doc->m_Arrow, child);
	doc->m_Arrow.fromDomElement(child);

	child = root.firstChildElement("crossbow");
	//readCrossbow(doc->m_Crossbow, child);
	doc->m_Crossbow.fromDomElement(child);

	child = root.firstChildElement("anglelist");
	QDomElement angle = child.firstChildElement("angle");
	while( !angle.isNull()){
		doc->m_Anglelist.push_back(angle.text().toDouble());
		angle = angle.nextSiblingElement("angle");
	}

	child = root.firstChildElement("targetcircles");
	QDomElement circle = child.firstChildElement("circle");
	while( !circle.isNull()){
		doc->m_Targetcircles.push_back(circle.text().toDouble());
		circle = circle.nextSiblingElement("circle");
	}
	return true;
}

/*!
 \fn arrowmatcher_file::write(QString filename)
 */
int arrowmatcher_file::write(QIODevice *device, ArrowmatcherDoc *doc) {


	CPfeil p;
	p.fromString(doc->m_Arrow.toString());
	//qDebug() << QString("dom Arrow neu=") << p.toString();




	QDomDocument domDocument("arrowmatcher");
	domDocument.setContent(
			QString("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"));
	QDomElement root = domDocument.createElement("arrowmatcher");
	root.setAttribute("version", "2.0");
	domDocument.appendChild(root);
	QDomElement dom_crossbowlist = domDocument.createElement("crossbowlist");

	std::list<CArmbrust>::iterator c_iter;
	for(int i=0; i< doc->m_crossbow_list_model->rootItem->childItems.size(); i++){
		CArmbrust a = *(doc->m_crossbow_list_model->rootItem->childItems.at(i)->base_item);
		QDomElement dom_crossbow = domDocument.createElement("crossbow");
		a.toDomElement(dom_crossbow);
		dom_crossbowlist.appendChild(dom_crossbow);
	}
	root.appendChild(dom_crossbowlist);

	QDomElement dom_arrowlist = domDocument.createElement("arrowlist");
	std::list<CPfeil>::iterator a_iter;
	for(int i=0; i< doc->m_arrow_list_model->rootItem->childItems.size(); i++){
		CPfeil p = *(doc->m_arrow_list_model->rootItem->childItems.at(i)->base_item);
		QDomElement dom_arrow = domDocument.createElement("arrow");
		p.toDomElement(dom_arrow);
		dom_arrowlist.appendChild(dom_arrow);

	}
	root.appendChild(dom_arrowlist);


	QDomElement dom_element = domDocument.createElement("range");
	QDomText dom_text = domDocument.createTextNode(QString().setNum(doc->m_Range));
	dom_element.appendChild(dom_text);
	root.appendChild(dom_element);

	dom_element = domDocument.createElement("angle");
	dom_text = domDocument.createTextNode(QString().setNum(doc->m_Angle));
	dom_element.appendChild(dom_text);
	root.appendChild(dom_element);

	dom_element = domDocument.createElement("arrow");
	doc->m_Arrow.toDomElement(dom_element);
	root.appendChild(dom_element);

	dom_element = domDocument.createElement("crossbow");
	doc->m_Crossbow.toDomElement(dom_element);
	root.appendChild(dom_element);



	QDomElement dom_anglelist = domDocument.createElement("anglelist");
	for (int i = 0; i< doc->m_Anglelist.size(); i++) {
		QDomElement dom_angle = domDocument.createElement("angle");
		dom_text = domDocument.createTextNode(QString().setNum(doc->m_Anglelist[i]));
		dom_angle.appendChild(dom_text);
		dom_anglelist.appendChild(dom_angle);
	}
	root.appendChild(dom_anglelist);

	QDomElement dom_targetcircles = domDocument.createElement("targetcircles");
	for (int i = 0; i< doc->m_Targetcircles.size(); i++) {
		QDomElement dom_circle = domDocument.createElement("circle");
		dom_text = domDocument.createTextNode(QString().setNum(doc->m_Targetcircles[i]));
		dom_circle.appendChild(dom_text);
		dom_targetcircles.appendChild(dom_circle);
	}
	root.appendChild(dom_targetcircles);



	dom_element = domDocument.createElement("windspeed");
	dom_text = domDocument.createTextNode(QString().setNum(doc->m_windspeed));
	dom_element.appendChild(dom_text);
	root.appendChild(dom_element);

	dom_element = domDocument.createElement("windangle");
	dom_text = domDocument.createTextNode(QString().setNum(doc->m_windangle));
	dom_element.appendChild(dom_text);
	root.appendChild(dom_element);


	QTextStream out(device);
	domDocument.save(out, 4);
	return 1;
}


int arrowmatcher_file::read_vector(utility::vector_3d &v,
		const QDomElement &dom_vector) {
	v.x = dom_vector.firstChildElement("x").text().toDouble();
	v.y = dom_vector.firstChildElement("y").text().toDouble();
	v.z = dom_vector.firstChildElement("z").text().toDouble();
	return 0;
}
int arrowmatcher_file::write_vector(QDomElement &dom_vector,
		QDomDocument &domDocument, const utility::vector_3d & v) {
	//QDomElement dom_rot = domDocument.createElement("rot");
	//dom_element.appendChild(dom_rot);

	QDomElement dom_x = domDocument.createElement("x");
	QDomText dom_text = domDocument.createTextNode(QString().setNum(v.x));
	dom_x.appendChild(dom_text);
	dom_vector.appendChild(dom_x);
	QDomElement dom_y = domDocument.createElement("y");
	dom_text = domDocument.createTextNode(QString().setNum(v.y));
	dom_y.appendChild(dom_text);
	dom_vector.appendChild(dom_y);
	QDomElement dom_z = domDocument.createElement("z");
	dom_text = domDocument.createTextNode(QString().setNum(v.z));
	dom_z.appendChild(dom_text);
	dom_vector.appendChild(dom_z);
	//dom_element.appendChild(dom_rot);


	return 0;
}

int arrowmatcher_file::writeMap(QDomElement &dom_map, QDomDocument &domDocument, const QMap<QString, QVariant> &v)
{

	QMap<QString, QVariant>::const_iterator i = v.constBegin();

	while (i != v.constEnd()) {
	     QDomElement dom_element;
	     QDomText dom_text;
		 if(i.value().type() == QVariant::Map)
	     {
			 dom_element = domDocument.createElement("map");
			 dom_element.setAttribute("name", i.key());
			 writeMap(dom_element, domDocument, i.value().toMap() );
	     }
	     else if(i.value().type() == QVariant::Double)
	     {
	    	 dom_element = domDocument.createElement("double");
			 dom_element.setAttribute("name", i.key());
	    	 dom_text = domDocument.createTextNode(QString().setNum(i.value().toDouble()) );
	    	 dom_element.appendChild(dom_text);
	     }
	     else if(i.value().type() == QVariant::Int)
	     {
	    	 dom_element = domDocument.createElement("int");
			 dom_element.setAttribute("name", i.key());
	    	 dom_text = domDocument.createTextNode(QString().setNum(i.value().toInt()) );
	    	 dom_element.appendChild(dom_text);
	     }
	     else if(i.value().type() == QVariant::String)
	     {
	    	 dom_element = domDocument.createElement("string");
			 dom_element.setAttribute("name", i.key());
	    	 dom_text = domDocument.createTextNode(i.value().toString());
	    	 dom_element.appendChild(dom_text);
	     }
	     else
	     {
	    	 ++i;
	    	 continue;
	     }
		 dom_map.appendChild(dom_element);
		 //std::cout << i.key() << ": " << i.value().toString().latin1() << endl;
	     ++i;
	 }
	return 0;
}
int  arrowmatcher_file::readMap(QMap<QString, QVariant> &v, const QDomElement &dom_map)
{
	QDomElement dom_element = dom_map.firstChildElement();
	while (!dom_element.isNull()) {
		QString name = dom_element.attribute("name");
		//std::cout << "name=" << name.toStdString().c_str() << " value=";
		if (dom_element.tagName() == "map") {
			QMap<QString, QVariant> map ;
			this->readMap(map, dom_element );
			v[name] = map;
			//std::cout << "map.-" << std::endl;
		}
		else if (dom_element.tagName() == "double") {
			double d = dom_element.text().toDouble();
			v[name] = d;
			//std::cout  << d ;
		}
		else if (dom_element.tagName() == "string") {
			QString s = dom_element.text();
			v[name] = s;
			//std::cout  << s.toStdString().c_str() ;
		}
		else if (dom_element.tagName() == "int") {
			int i = dom_element.text().toInt();
			v[name] = i;
			//std::cout  << i ;
		}
		//std::cout << std::endl;
		dom_element = dom_element.nextSiblingElement();

	}
	return 0;
}

