/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// application specific includes
#include "arrowmatcherview.h"
#include "arrowmatcherdoc.h"
#include "mainwindow.h"
#include <ballistic/utility.h>
#include "sight_diagram_y.h"
#include "sight_diagram_z.h"

#include <QPainter>
#include <QTextStream>
#include <QPaintEvent>
#include <QTextDocument>
#include <QRegion>
#include <QMessageBox>
#include <QLayout>
#include <QPixmap>
#include <QPrinter>
#include <QPaintEvent>
#include <QLabel>

#include <vector>
#include <list>
#include <string>
#include <iostream>
#include <stdio.h>
#include <algorithm>
#include <QSlider>



void ArrowmatcherView::update(ArrowmatcherDoc *doc) {
	m_doc = doc;
	if (m_doc == NULL)
		return;
	m_vd->m_doc = m_doc;
	m_bd->m_doc = m_doc;


	m_vd->setUpdatesEnabled(false);
	m_bd->setUpdatesEnabled(false);

	//m_bd->m_doc = m_doc;
	m_vd->recreate(m_doc);
	m_bd->recreate(m_doc);

	m_Pixmap->fill(Qt::white);
	m_fontheight = 12;//12
	QPainter paint(m_Pixmap);
	QPen background(QColor(255,255,255));
	paint.setPen(background);
	//paint.setBackgroundColor(QColor(255, 255, 255));//Hintergrundfarbe von Schrift etc. ...
	QRect r(0, 0, 1000, 3000);
	paint.setWindow(0, 0, 100, 100);
	paint.setViewport(0, 0, 100, 100);

	zeichnen(paint, r);

	m_vd->setUpdatesEnabled(true);
	m_bd->setUpdatesEnabled(true);
	repaint();
}

ArrowmatcherView::ArrowmatcherView(QWidget *parent) :
	QWidget(parent) {

//	 QSlider *slider = new QSlider(Qt::Horizontal, this);
//	 slider->setFocusPolicy(Qt::StrongFocus);
//	 slider->setTickPosition(QSlider::TicksBothSides);
//	 slider->setTickInterval(10);
//	 slider->setSingleStep(1);
//	 slider->setGeometry(0,1100, 500, 30  );
//	 slider->setHidden(false);




	//this->setBackgroundMode(Qt::OpaqueMode);
	this->setBackgroundRole(QPalette::NoRole);
	m_zoom = 1;//faktor zur vergr�erung des absehens, kann ver�dert werden
	m_pan = 0;//gibt die vertikale verschiebung des Bildes an
	//m_ptrPixmap = NULL;
	m_fontheight = 12;

	m_dEntfernung = 10;
	m_dZielwinkel = 0;
	m_doc = NULL;
	double width = 210.0;
	double height = 300.0;

	this->setFixedSize(830, 830 * height / width);

	m_Pixmap = new QPixmap(this->width(), this->height());
	//qDebug() << "Pixmap width=" << this->width() << " height="  << this->height();
	m_Pixmap->fill();// fill with white color

	m_vd = new sight_diagram_y(this);
	m_bd = new sight_diagram_z(this);


	QRect diagram(0, 0, 820, 500);//diagram(0,0,970,550 );
	diagram.translate(0, 30);
	m_vd->setGeometry(diagram);
	diagram.translate(0, 505);
	diagram.translate(0, 30);
	m_bd->setGeometry(diagram);

	//m_vd->hide();
	//m_bd->hide();

}

ArrowmatcherView::~ArrowmatcherView() {

}
void ArrowmatcherView::addView(MainWindow* mainwindow){
	this->m_pControler = mainwindow;
	QScrollArea *scrollview = new QScrollArea;
	scrollview->setWidget(this);
	mainwindow->m_content_col_main->addTab(scrollview, tr("Sight Setting Graph"));

	actPrint = new QAction(QIcon(":/toolbar/fileprint"), tr("Print"), this);
	actPrint->setStatusTip(tr("Print"));
	connect(actPrint, SIGNAL(triggered()), this, SLOT(on_actPrint_triggered()));
	mainwindow->m_editToolBar->addAction(this->actPrint);
}
ArrowmatcherDoc *ArrowmatcherView::getDocument() const {
	return m_doc;
}

void ArrowmatcherView::print(QPrinter *pPrinter) {

	QPainter paint;
	//paint.drawPixmap(0,0,*m_Pixmap);

	paint.begin(pPrinter);

	paint.setWindow(0, 0, 100, 100);
	paint.setViewport(0, 0, 70, 70);

	//QPainter temp(this);
	//paint.drawPixmap(0, 0, *m_Pixmap);
	//return;


	//m_ptrPixmap = new QPixmap(contentsWidth(), contentsHeight() );
	m_vd->setUpdatesEnabled(false);
	m_bd->setUpdatesEnabled(false);
	//m_doc->m_views_need_recreate = true;

	//setUpdatesEnabled(true);

	m_vd->m_doc = m_doc;
	m_bd->m_doc = m_doc;
	m_vd->recreate(m_doc);
	m_bd->recreate(m_doc);

	//m_Pixmap->fill(Qt::white);
	//m_fontheight = 12;//12
	//QPainter paint(m_Pixmap);
	paint.setPen(QColor(255, 255, 255));//Hintergrundfarbe von Schrift etc. ...

	//paint.begin(m_Pixmap);
	QPoint offset(0, 0);
	offset += (QPoint(0, 0));
	QRect diagram(0, 0, 820, 500);//diagram(0,0,970,550 );
	diagram.translated(offset.x(), offset.y());
	//m_vd->zeichnen(paint,diagram);


	paint.drawPixmap(offset.x(), offset.y(), *(m_vd->m_Pixmap));
	offset += QPoint(0, 505);

	paint.drawPixmap(offset.x(), offset.y(), *(m_bd->m_Pixmap));
	//QRect r(0,0,1500,3000 );
	QRect r(0, 0, 1000, 3000);
	zeichnen(paint, r);
	m_vd->setUpdatesEnabled(true);
	m_bd->setUpdatesEnabled(true);

	paint.end();

}

void ArrowmatcherView::paintEvent(QPaintEvent *paintEvent) {

	QPainter temp(this);
	temp.drawPixmap(0, 0, *m_Pixmap);

}


int ArrowmatcherView::zeichnen(QPainter &dc, QRect &Rechteck) {
	ArrowmatcherDoc *doc = m_doc;
	if (doc == NULL)
		return -1;
	//qDebug() << "ArrowmatcherView::zeichnen";
	QPen originalPen = dc.pen();
	QFont originalFont = dc.font();
	QBrush originalBrush = dc.brush();
	QFont fontText("courier", m_fontheight, QFont::Bold);//Entfernungsangeben der Querbalken
	QPen black_thin_solid(QColor("black"), 0, Qt::SolidLine);
	//int zellheight = dc.fontInfo().pointSize()+ 5;



	dc.setPen(black_thin_solid);
	dc.setFont(fontText);
	//dc.setBackgroundMode(Qt::TransparentMode);//OpaqueMode  bringt beim ausdrucken nicht

	//dc.setPen(QColor(255, 255, 255));//Hintergrundfarbe von Schrift etc. ...

	QRect diagram_header(0, 0, 820, 30);
	QString header;

	header = tr("vertical");
	dc.drawText(diagram_header, Qt::AlignLeft, header);

	QRect diagram(0, 0, 820, 500);//diagram(0,0,970,550 );

	dc.translate(0, 30);

	//drawDiagram(dc, diagram);
	dc.translate(0, 505);
	//draw_trajectory(dc, diagram);

	header = tr("horizontal");
	dc.drawText(diagram_header, Qt::AlignLeft, header);

	dc.translate(0, 30);

	dc.translate(0, 505);

	QString s;
	std::vector<double> crosshair;
	QString s_1 = "";
	QTextStream output(&s_1);


	//output.setf(std::ios::fixed);
	//output.precision(6);
	output << "<table width=100% border=0 cellspacing=0 cellpadding=0><tr><td>"

	<< "<table width=100% border=1 cellspacing=0 cellpadding=0>" << "<tr>"
			<< "<th colspan=2 bgcolor=yellow><b>" << tr("crossbow")  << "</th>"
			<< "</tr><tr>"
			<< "<td>" << tr("title") << "</td>"
			<< "<td>" << doc->m_Crossbow.m_name << "</td>"
			<< "</tr><tr>"
			<< "<td>" << tr("Energy") << "</td>"
			<< "<td>" << doc->m_Crossbow.m_energy << " J" << "</td>"
			<< "</tr><tr>"
			<< "<td>" << tr("Height of the sight") << "</td>" << "<td>"
			<< doc->m_Crossbow.m_scso.trans.y << " mm" << "</td>"
			<< "</tr>";
	if (doc->m_Crossbow.m_Zielgeraet.m_iTyp == KIMME_U_KORN) {
		output << "<tr>"
		        << "<td>" << tr("Graduation") << "</td>" << "<td>"
				<< doc->m_Crossbow.m_Zielgeraet.m_graduation.y
				<< QString(" mm/m") << "</td>"
				<< "</tr>";
	}
	output << "</table>";
	output << "</td><td>";

	output << "<table width=100% border=1 cellspacing=0 cellpadding=0>" << "<tr>"
			<< "<th colspan=2 bgcolor=yellow><b>" << tr("arrow")  << "</th>"
			<< "</tr><tr>"
			<< "<td>" << tr("title") << "</td>"
			<< "<td>" << doc->m_Arrow.m_data["title"].toString() << "</td>"
			<< "</tr><tr>"
			<< "<td>" << tr("mass") << "</td>"
			<< "<td>" << doc->m_Arrow.m << " kg" << "</td>"

	    <<  "</tr></table>";

	output << "</td></tr></table>";

	QTextDocument *richtext = new QTextDocument(this);
	richtext->setHtml(s_1);
	//richtext->drawContents(&dc, QRect(0,0, diagram.width(), 500));
	dc.setPen(originalPen);
	dc.setFont(originalFont);
	dc.setBrush(originalBrush);

	return 0;
}


void ArrowmatcherView::on_actPrint_triggered()
{
	QPrinter printer;

	QPrintDialog *dialog = new QPrintDialog(&printer, this);
	if (dialog->exec() != QDialog::Accepted)
		return;

	this->print(&printer);
	/// @todo improve printing

}

