/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * calc_crosshair_dlg.cpp
 *  Created on: 30.08.2009
 */

#include "calc_crosshair_dlg.h"
#include <QMessageBox>
#include "ArrowTreeModel.h"
#include <QTextStream>

calc_crosshair_dlg::calc_crosshair_dlg(QWidget *parent) :
	calc_velocity_dlg(parent) {
	ui.txt_wide_setting_z->hide();
	ui.txt_wide_setting_y->hide();
	ui.txt_small_setting_z->hide();
	ui.txt_small_setting_y->hide();
	ui.lbl_small_setting_y->hide();
	ui.lbl_small_setting_z->hide();
	ui.lbl_wide_setting_y->hide();
	ui.lbl_wide_setting_z->hide();

	ui.selReticle_small->hide();
	ui.selReticle_wide->hide();
	//ui.grp_wide_range->hide();
	//ui.grp_small_range->setTitle(QString());
	ui.lbl_setting->hide();
	ui.lbl_crosshair->hide();

	ui.lbl_wide_setting_y->hide();
	ui.lbl_wide_setting_z->hide();
	ui.txt_wide_setting_y->hide();
	ui.txt_wide_setting_z->hide();
	ui.lbl_wide_devitation_y->hide();
	ui.lbl_wide_setting_z_2->hide();
	ui.txt_wide_devitation_y->hide();
	ui.txt_wide_devitation_z->hide();
	ui.lbl_wide_range->hide();
	ui.txt_wide_range->hide();
	ui.line->hide();

}

calc_crosshair_dlg::~calc_crosshair_dlg() {

}
void calc_crosshair_dlg::set_values(CArmbrust &a) {
	calc_velocity_dlg::set_values(a);

}
void calc_crosshair_dlg::init(CArmbrust &a, ArrowTreeModel *p) {
	calc_velocity_dlg::init(a,p);
	set_values(m_crossbow);


}
void calc_crosshair_dlg::print_result(CArmbrust &a, CPfeil &p) {
	QString s;
	QTextStream outputstream(&s);
	double v = sqrt(2.0 * a.m_energy / p.m);

	outputstream << "<table cellpadding=0 cellspacing=0><tr><td>"
			<< tr("Setting") << "</td><td>" << "<table>"
			<< "<tr><td>Y&nbsp;" << m_sight_setting.y * 1000.0 / m_sight_setting.x << "</td></tr>"
			<< "<tr><td>Z&nbsp;" << m_sight_setting.z * 1000.0 / m_sight_setting.x<< "</td></tr>"
			<< "</table></td><td>" << tr("mm/m") << "</td></tr>";

	outputstream << "</table>";
	ui.lbl_solution->setText(s);
}

void calc_crosshair_dlg::on_cmdCalc_clicked() {
	CArmbrust a = m_crossbow;

	double small_range = ui.txt_small_range->text().toDouble();

	utility::vector_3d devitation_small;
	devitation_small.x = 0.0;
	devitation_small.y = ui.txt_small_devitation_y->text().toDouble() / 1000.0;
	devitation_small.z = ui.txt_small_devitation_z->text().toDouble() / 1000.0;


	QItemSelectionModel *selection_model = ui.lst_arrows->selectionModel();
	if (selection_model == NULL) {
		QMessageBox::information(this, tr("Error"), tr("select an arrow"));//MessageBox("einen Pfeil aus der Pfeilliste ausw�len!", "Arrowmatcher", MB_ICONEXCLAMATION);
		return;
	}
	QModelIndex index = selection_model->currentIndex();

	CPfeil *p_arrow = m_arrow_list_model->getItem(index);//= ui.lstPfeile->GetSelPfeil();
	if (p_arrow == NULL) {
		QMessageBox::information(this, tr("Error"), tr("select an arrow"));//MessageBox("einen Pfeil aus der Pfeilliste ausw�len!", "Arrowmatcher", MB_ICONEXCLAMATION);
		return;
	}

	flightparam fp;
	Ballistic::flightparam_init(fp, *p_arrow);
	Ballistic::flightparam_gravity(fp, 0.0);
	Ballistic::flightparam_wind(fp, 0.0, 0.0);

	flight_snapshot endparam, beginparam;
	utility::vector_3d setting;

	int iFehler = a.sight_setting(setting, // Richtungsvector vom zielgerätkoordinatensystem aus, x=1.0
			endparam, beginparam, small_range, //m
			fp);
	if (0 != iFehler){
		QMessageBox::information(this, tr("Error"), tr("Cannot calculate the sight setting"));//MessageBox("einen Pfeil aus der Pfeilliste ausw�len!", "Arrowmatcher", MB_ICONEXCLAMATION);
		return;
	}
	//////////////////////////////////////////////////
	// The real setting of the sight
	// include the devitation. (sight coordinaten system)

	m_sight_setting.x = setting.x;
	m_sight_setting.y = setting.y - devitation_small.y;
	m_sight_setting.z = setting.z - devitation_small.z;



	print_result(m_crossbow, *p_arrow);

}
