/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "calc_velocity_dlg.h"
#include "ArrowTreeModel.h"
#include <QMessageBox>
#include <iostream>
#include <QTextStream>

calc_velocity_dlg::calc_velocity_dlg(QWidget *parent) :
	QDialog(parent) {
	ui.setupUi(this);
	QPixmap *image = new QPixmap(":/images/targetpoint.png");
	ui.lbl_image->setPixmap(*image);

}

calc_velocity_dlg::~calc_velocity_dlg() {

}
void calc_velocity_dlg::init(CArmbrust &a, ArrowTreeModel *p) {
	m_crossbow = a;
	m_arrow_list_model = p;
	ui.lst_arrows->setModel(p);
	ui.lst_arrows->setSelectionMode(QAbstractItemView::SingleSelection);
	ui.lst_arrows->setSelectionBehavior(QAbstractItemView::SelectRows);
	set_values(m_crossbow);
	QPixmap *image = new QPixmap(":/images/targetpoint.png");
	ui.lbl_image->setPixmap(*image);

	ui.txt_small_range->setUnitstyle(UnitInputWidget::symbol);
	ui.txt_small_range->setValue(0.0, "SI_metre", NULL, UnitInputWidget::preset);

	ui.txt_wide_range->setUnitstyle(UnitInputWidget::symbol);
	ui.txt_wide_range->setValue(0.0, "SI_metre", NULL, UnitInputWidget::preset);

	ui.txt_wide_devitation_y->setUnitstyle(UnitInputWidget::symbol);
	ui.txt_wide_devitation_y->setValue(0.0, "millimetre", NULL, UnitInputWidget::preset);

	ui.txt_wide_devitation_z->setUnitstyle(UnitInputWidget::symbol);
	ui.txt_wide_devitation_z->setValue(0.0, "millimetre", NULL, UnitInputWidget::preset);

	ui.txt_small_devitation_y->setUnitstyle(UnitInputWidget::symbol);
	ui.txt_small_devitation_y->setValue(0.0, "millimetre", NULL, UnitInputWidget::preset);

	ui.txt_small_devitation_z->setUnitstyle(UnitInputWidget::symbol);
	ui.txt_small_devitation_z->setValue(0.0, "millimetre", NULL, UnitInputWidget::preset);
	//ui.txt_small_range->setUnit("SI_metre");

}
void calc_velocity_dlg::set_values(CArmbrust &a) {


	if (a.m_Zielgeraet.m_iTyp == BENUTZERDEF) {
		ui.txt_wide_setting_z->hide();
		ui.txt_wide_setting_y->hide();
		ui.txt_small_setting_z->hide();
		ui.txt_small_setting_y->hide();
		ui.lbl_small_setting_y->hide();
		ui.lbl_small_setting_z->hide();
		ui.lbl_wide_setting_y->hide();
		ui.lbl_wide_setting_z->hide();
		ui.lbl_setting->hide();

		ui.selReticle_small->setCrossbow(a);
		ui.selReticle_wide->setCrossbow(a);


	} else if (a.m_Zielgeraet.m_iTyp == KIMME_U_KORN) {
		ui.selReticle_wide->hide();
		ui.selReticle_small->hide();
		ui.lbl_crosshair->hide();

	}

}
void calc_velocity_dlg::get_values(CArmbrust &a) {
	a = m_crossbow;

}
void calc_velocity_dlg::on_cmdCalc_clicked() {
	CArmbrust a = m_crossbow;
	get_values(a);

	bool ok;
	double small_range = ui.txt_small_range->getSIValue(ok);
	double wide_range = ui.txt_wide_range->getSIValue(ok);

	utility::vector_3d setting_small;
	utility::vector_3d setting_wide;

	utility::vector_3d devitation_small;
	devitation_small.x = 0.0;
	devitation_small.y = ui.txt_small_devitation_y->getSIValue(ok);
	devitation_small.z = ui.txt_small_devitation_z->getSIValue(ok);

	// vector points from the targetpoint to the impactpoint (x = 0)
	utility::vector_3d devitation_wide;
	devitation_wide.x = 0.0;
	devitation_wide.y = ui.txt_wide_devitation_y->getSIValue(ok);
	devitation_wide.z = ui.txt_wide_devitation_z->getSIValue(ok);

	if (a.m_Zielgeraet.m_iTyp == BENUTZERDEF) {
		bool ok_small, ok_wide;
		setting_small = ui.selReticle_small->getSetting(ok_small);
		setting_wide = ui.selReticle_wide->getSetting(ok_wide);
		if( ! (ok_small && ok_wide)){
			QMessageBox::information(this, tr("Error"), tr("select a vertical and a horizontal line of the reticle by mouse click"));//MessageBox("einen Pfeil aus der Pfeilliste ausw�len!", "Arrowmatcher", MB_ICONEXCLAMATION);
			return;
		}
		//setting_small = a.m_Zielgeraet.m_marks[index_small];
		//setting_wide = a.m_Zielgeraet.m_marks[index_wide];

	} else if (a.m_Zielgeraet.m_iTyp == KIMME_U_KORN) {
		setting_small.x = 1.0;
		setting_small.y = ui.txt_small_setting_y->text().toDouble()
				* a.m_Zielgeraet.m_graduation.y;
		setting_small.z = ui.txt_small_setting_z->text().toDouble()
				* a.m_Zielgeraet.m_graduation.z;

		setting_wide.x = 1.0;
		setting_wide.y = ui.txt_wide_setting_y->text().toDouble()
				* a.m_Zielgeraet.m_graduation.y;
		setting_wide.z = ui.txt_wide_setting_z->text().toDouble()
				* a.m_Zielgeraet.m_graduation.z;
	}

	QItemSelectionModel *selection_model = ui.lst_arrows->selectionModel();
	if (selection_model == NULL) {
		QMessageBox::information(this, tr("Error"), tr("select an arrow"));
		return;
	}
	QModelIndex index = selection_model->currentIndex();

	CPfeil *p_arrow = m_arrow_list_model->getItem(index);//= ui.lstPfeile->GetSelPfeil();
	if (p_arrow == NULL) {
		QMessageBox::information(this, tr("Error"), tr("select an arrow"));
		return;
	}

	flightparam fp;
	Ballistic::flightparam_init(fp, *p_arrow);
	Ballistic::flightparam_gravity(fp, 0.0);
	Ballistic::flightparam_wind(fp, 0.0, 0.0);

	double v;
	utility::vector_3d scso_rot;
	int err = 0;
	err = a.calc_velocity(v, scso_rot, small_range, wide_range, setting_small,
			setting_wide, devitation_small, devitation_wide, fp);
	if (err) {
		QMessageBox::critical(this, tr("Error"), tr("Cannot calculate speed"),
				QMessageBox::Ok);
		return;
	}
	double energy = fp.arrow.m * v * v / 2.0;
	//a.m_scso.rot = scso_rot;

	QString unitstr ;
	//double v ;
	int status ;
	UnitConverter uv;

	unitstr = a.getValue("energy/unit").toString();
	status = a.getValue("energy/status").toInt();
	v = energy;
	uv.convert_from_SI(v, unitstr);
	a.setValue("energy/status", (int)Datastore::user );
	a.setValue("energy/value", v);

	unitstr = a.getValue("scso/rot/y/unit").toString();
	status = a.getValue("scso/rot/y/status").toInt();
	v = scso_rot.y;
	uv.convert_from_SI(v, unitstr);
	a.setValue("scso/rot/y/status", (int)Datastore::user );
	a.setValue("scso/rot/y/value", v);

	unitstr = a.getValue("scso/rot/z/unit").toString();
	status = a.getValue("scso/rot/z/status").toInt();
	v = scso_rot.z;
	uv.convert_from_SI(v, unitstr);
	a.setValue("scso/rot/z/status", (int)Datastore::user );
	a.setValue("scso/rot/z/value", v);

	m_crossbow = a;
	//set_values(m_crossbow);
	print_result(m_crossbow, *p_arrow);

}
void calc_velocity_dlg::accept() {
	//get_values(m_crossbow);
	QDialog::accept();
}
void calc_velocity_dlg::print_result(CArmbrust &a, CPfeil &p) {
	QString s;
	QTextStream outputstream(&s);
	double v = sqrt(2.0 * a.m_energy / p.m);

	outputstream << "<table cellpadding=0 cellspacing=\"4\"><tr><td>" << tr(
			"Velocity") << "</td><td>" << v << "</td><td>" << tr("m/s")
			<< "</td></tr><tr><td>" << tr("Energy") << "</td><td>"
			<< a.m_energy << "</td><td>" << tr("J") << "</td></tr><tr><td>"
			<< tr("Angle") << "</td><td>" << "<table>"
			<< "<tr><td>Z&nbsp;" << (a.m_scso.rot.z * 1000.0) << "</td></tr>"
			<< "<tr><td>Y&nbsp;" << (a.m_scso.rot.y*1000.0) << "</td></tr>"
			<< "</table></td><td>" << tr(
			"millirad") << "</td></tr>";

	outputstream << "</table>";
	ui.lbl_solution->setText(s);

}
