/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 *  Created on: 28.10.2009
 */
#include "dlgarrowtabdimensions.h"
#include <math.h>
#include <ballistic/utility.h>

DlgArrowTabDimensions::DlgArrowTabDimensions(QWidget *parent) :
	QWidget(parent) {
	m_pArrow = NULL;
	ui.setupUi(this);
	QIcon *shaft = new QIcon(":/toolbar/tlb_shaft");
	QIcon *point = new QIcon(":/toolbar/tlb_point");
	QIcon *nock = new QIcon(":/toolbar/tlb_nock");
	QIcon *fletches = new QIcon(":/toolbar/tlb_fletches");

	ui.toolBox->setItemIcon(0, *shaft);
	ui.toolBox->setItemIcon(1, *fletches);
	ui.toolBox->setItemIcon(2, *point);
	ui.toolBox->setItemIcon(3, *nock);

}

DlgArrowTabDimensions::~DlgArrowTabDimensions() {

}

void DlgArrowTabDimensions::update(CPfeil *arrow) {

	this->blockSignals(true);

	m_pArrow = arrow;

	//////////////////////
	// set values:
	ui.txtShaftweight->setText(
			m_pArrow->m_data["txtShaftweight"].toMap()["value"].toString());
	ui.txtShaftlength->setText(
			m_pArrow->m_data["txtShaftlength"].toMap()["value"].toString());
	ui.txtShaftcode->setText(
			m_pArrow->m_data["txtShaftcode"].toMap()["value"].toString());
	ui.txtShaftdia->setText(
			m_pArrow->m_data["txtShaftdia"].toMap()["value"].toString());
	ui.txtShaftthickness->setText(
			m_pArrow->m_data["txtShaftthickness"].toMap()["value"].toString());
	ui.txtShaftdisplacement->setText(
			m_pArrow->m_data["txtShaftdisplacement"].toMap()["value"].toString());

	ui.txtFletchesarea->setText(
			m_pArrow->m_data["txtFletchesarea"].toMap()["value"].toString());
	ui.txtFletcheslength->setText(
			m_pArrow->m_data["txtFletcheslength"].toMap()["value"].toString());
	ui.txtFletchesweight->setText(
			m_pArrow->m_data["txtFletchesweight"].toMap()["value"].toString());
	ui.txtFletchesdisplacement->setText(
			m_pArrow->m_data["txtFletchesdisplacement"].toMap()["value"].toString());

	ui.txtPointweight->setText(
			m_pArrow->m_data["txtPointweight"].toMap()["value"].toString());
	ui.txtPointdisplacement->setText(
			m_pArrow->m_data["txtPointdisplacement"].toMap()["value"].toString());

	ui.txtNockweight->setText(
			m_pArrow->m_data["txtNockweight"].toMap()["value"].toString());
	ui.txtNockdisplacement->setText(
			m_pArrow->m_data["txtNockdisplacement"].toMap()["value"].toString());

	///////////////////////////////
	// Units for Comboboxes
	this->set_unit_id(
			m_pArrow->m_data["txtShaftweight"].toMap()["unit"].toString(),
			ui.cboShaftweight);
	this->set_unit_id(
			m_pArrow->m_data["txtShaftlength"].toMap()["unit"].toString(),
			ui.cboShaftlength);
	this->set_unit_id(
			m_pArrow->m_data["txtShaftdia"].toMap()["unit"].toString(),
			ui.cboShaftdia);
	this->set_unit_id(
			m_pArrow->m_data["txtShaftthickness"].toMap()["unit"].toString(),
			ui.cboShaftthickness);
	this->set_unit_id(
			m_pArrow->m_data["txtShaftdisplacement"].toMap()["unit"].toString(),
			ui.cboShaftdisplacement);
	this->set_unit_id(
			m_pArrow->m_data["txtFletchesarea"].toMap()["unit"].toString(),
			ui.cboFletchesarea);
	this->set_unit_id(
			m_pArrow->m_data["txtFletcheslength"].toMap()["unit"].toString(),
			ui.cboFletcheslength);
	this->set_unit_id(
			m_pArrow->m_data["txtFletchesweight"].toMap()["unit"].toString(),
			ui.cboFletchesweight);
	this->set_unit_id(
			m_pArrow->m_data["txtFletchesdisplacement"].toMap()["unit"].toString(),
			ui.cboFletchesdisplacement);
	this->set_unit_id(
			m_pArrow->m_data["txtPointweight"].toMap()["unit"].toString(),
			ui.cboPointweight);
	this->set_unit_id(
			m_pArrow->m_data["txtPointdisplacement"].toMap()["unit"].toString(),
			ui.cboPointdisplacement);
	this->set_unit_id(
			m_pArrow->m_data["txtNockweight"].toMap()["unit"].toString(),
			ui.cboNockweight);
	this->set_unit_id(
			m_pArrow->m_data["txtNockdisplacement"].toMap()["unit"].toString(),
			ui.cboNockdisplacement);

	////////////////////////
	// Styles:
	QString style_calculated = "QLineEdit {color: rgb(0, 150, 0);}";
	QString style_default = "QLineEdit {color: rgb(0, 0, 150);}";

	switch (m_pArrow->m_data["txtShaftweight"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtShaftweight->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtShaftweight->setStyleSheet(style_default);
		break;
	default:
		ui.txtShaftweight->setStyleSheet("");
	}

	switch (m_pArrow->m_data["txtShaftlength"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtShaftlength->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtShaftlength->setStyleSheet(style_default);
		break;
	default:
		ui.txtShaftlength->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtShaftcode"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtShaftcode->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtShaftcode->setStyleSheet(style_default);
		break;
	default:
		ui.txtShaftcode->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtShaftdia"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtShaftdia->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtShaftdia->setStyleSheet(style_default);
		break;
	default:
		ui.txtShaftdia->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtShaftthickness"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtShaftthickness->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtShaftthickness->setStyleSheet(style_default);
		break;
	default:
		ui.txtShaftthickness->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtShaftdisplacement"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtShaftdisplacement->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtShaftdisplacement->setStyleSheet(style_default);
		break;
	default:
		ui.txtShaftdisplacement->setStyleSheet("");
	}

	switch (m_pArrow->m_data["txtFletchesarea"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtFletchesarea->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtFletchesarea->setStyleSheet(style_default);
		break;
	default:
		ui.txtFletchesarea->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtFletcheslength"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtFletcheslength->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtFletcheslength->setStyleSheet(style_default);
		break;
	default:
		ui.txtFletcheslength->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtFletchesweight"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtFletchesweight->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtFletchesweight->setStyleSheet(style_default);
		break;
	default:
		ui.txtFletchesweight->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtFletchesdisplacement"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtFletchesdisplacement->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtFletchesdisplacement->setStyleSheet(style_default);
		break;
	default:
		ui.txtFletchesdisplacement->setStyleSheet("");
	}

	switch (m_pArrow->m_data["txtPointweight"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtPointweight->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtPointweight->setStyleSheet(style_default);
		break;
	default:
		ui.txtPointweight->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtPointdisplacement"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtPointdisplacement->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtPointdisplacement->setStyleSheet(style_default);
		break;
	default:
		ui.txtPointdisplacement->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtNockweight"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtNockweight->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtNockweight->setStyleSheet(style_default);
		break;
	default:
		ui.txtNockweight->setStyleSheet("");
	}
	switch (m_pArrow->m_data["txtNockdisplacement"].toMap()["status"].toInt()) {
	case DlgArrowModel::calculated:
		ui.txtNockdisplacement->setStyleSheet(style_calculated);
		break;
	case DlgArrowModel::defaultvalue:
		ui.txtNockdisplacement->setStyleSheet(style_default);
		break;
	default:
		ui.txtNockdisplacement->setStyleSheet("");
	}
	this->blockSignals(false);

}

void DlgArrowTabDimensions::calc(CPfeil *arrow) {
	this->blockSignals(true);
	m_pArrow = arrow;
	//DlgArrowModel::Statusflag status = DlgArrowModel::defaultvalue;


	///////////////////////////////////
	// create the values in the map
//	if (!m_pArrow->m_data.contains("txtShaftweight")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_kilogram";
//		map["title"] = tr("shaft weight");
//		m_pArrow->m_data["txtShaftweight"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtShaftlength")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_metre";
//		map["title"] = tr("shaft length");
//		m_pArrow->m_data["txtShaftlength"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtShaftcode")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["title"] = tr("shaft code");
//		m_pArrow->m_data["txtShaftcode"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtShaftdia")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_metre";
//		map["title"] = tr("shaft diameter");
//		m_pArrow->m_data["txtShaftdia"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtShaftthickness")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_metre";
//		map["title"] = tr("shaft thickness");
//		m_pArrow->m_data["txtShaftthickness"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtShaftdisplacement")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_metre";
//		map["title"] = tr("shaft displacement");
//		m_pArrow->m_data["txtShaftdisplacement"] = map;
//	}
//
//	if (!m_pArrow->m_data.contains("txtFletchesarea")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_square_metre";
//		map["title"] = tr("fletches area");
//		m_pArrow->m_data["txtFletchesarea"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtFletcheslength")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_metre";
//		map["title"] = tr("fletches length");
//		m_pArrow->m_data["txtFletcheslength"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtFletchesweight")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_kilogram";
//		map["title"] = tr("fletches weight");
//		m_pArrow->m_data["txtFletchesweight"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtFletchesdisplacement")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_metre";
//		map["title"] = tr("fletches displacement");
//		m_pArrow->m_data["txtFletchesdisplacement"] = map;
//	}
//
//	if (!m_pArrow->m_data.contains("txtPointweight")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_kilogram";
//		map["title"] = tr("point weight");
//		m_pArrow->m_data["txtPointweight"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtPointdisplacement")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_metre";
//		map["title"] = tr("point displacement");
//		m_pArrow->m_data["txtPointdisplacement"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtNockweight")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_kilogram";
//		map["title"] = tr("nock weight");
//		m_pArrow->m_data["txtNockweight"] = map;
//	}
//	if (!m_pArrow->m_data.contains("txtNockdisplacement")) {
//		QMap<QString, QVariant> map;
//		map["value"] = "";
//		map["status"] = DlgArrowModel::defaultvalue;
//		map["unit"] = "SI_metre";
//		map["title"] = tr("nock displacement");
//		m_pArrow->m_data["txtNockdisplacement"] = map;
//	}
//	///////////////////////////////////
//	// Defaultvalues:
//	QMap<QString, QVariant> map;
//	if (m_pArrow->m_data["txtShaftdia"].toMap().value("status").toInt()
//			!= DlgArrowModel::modified_by_user) {
//		map = m_pArrow->m_data["txtShaftdia"].toMap();
//		map["value"] = "";
//		m_pArrow->m_data["txtShaftdia"] = map;
//	}
//	if (m_pArrow->m_data["txtShaftthickness"].toMap().value("status").toInt()
//			!= DlgArrowModel::modified_by_user) {
//		map = m_pArrow->m_data["txtShaftthickness"].toMap();
//		map["value"] = "";
//		m_pArrow->m_data["txtShaftthickness"] = map;
//	}
//	if (m_pArrow->m_data["txtShaftweight"].toMap().value("status").toInt()
//			!= DlgArrowModel::modified_by_user) {
//		map = m_pArrow->m_data["txtShaftweight"].toMap();
//		map["value"] = "";
//		m_pArrow->m_data["txtShaftweight"] = map;
//	}
//	if (m_pArrow->m_data["txtFletchesweight"].toMap().value("status").toInt()
//			!= DlgArrowModel::modified_by_user) {
//		map = m_pArrow->m_data["txtFletchesweight"].toMap();
//		map["value"] = "";
//		m_pArrow->m_data["txtFletchesweight"] = map;
//	}
//	if (m_pArrow->m_data["txtFletchesarea"].toMap().value("status").toInt()
//			!= DlgArrowModel::modified_by_user) {
//		map = m_pArrow->m_data["txtFletchesarea"].toMap();
//		map["value"] = "";
//		m_pArrow->m_data["txtFletchesarea"] = map;
//	}

	///////////////////////
	// calc missing values:
	if (m_pArrow->m_data["txtShaftdia"].toMap().value("status").toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftcode"].toMap().value("value").toString().length()
					> 3

	) {
		double dia;
		if (this->calc_diameter(dia,
				m_pArrow->m_data["txtShaftcode"].toMap()["value"].toString())) {
			this->set_value_from_SI("txtShaftdia",dia);
			m_pArrow->setValue("txtShaftdia/status",  DlgArrowModel::calculated);
		}
	}
	if (m_pArrow->m_data["txtShaftthickness"].toMap().value("status").toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftcode"].toMap().value("value").toString().length()
					> 3

	) {
		double thickness;
		if (this->calc_thickness(thickness,
				m_pArrow->m_data["txtShaftcode"].toMap()["value"].toString())) {
			this->set_value_from_SI("txtShaftthickness", thickness);
			m_pArrow->setValue("txtShaftthickness/status", DlgArrowModel::calculated);
		}
	}

	if (m_pArrow->m_data["txtShaftweight"].toMap().value("status").toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftlength"].toMap().value("value").toDouble()
					> 0.0 && m_pArrow->m_data["txtShaftdia"].toMap().value(
			"value").toDouble() > 0.0
			&& m_pArrow->m_data["txtShaftthickness"].toMap().value("value").toDouble()
					> 0.0) {

		double weight;
		double length;
		double thickness;
		double dia;
		this->get_SI_value(length, m_pArrow->m_data["txtShaftlength"].toMap());
		this->get_SI_value(dia, m_pArrow->m_data["txtShaftdia"].toMap());
		this->get_SI_value(thickness,
				m_pArrow->m_data["txtShaftthickness"].toMap());

		if (this->calc_shaftweight(weight, dia, thickness, length)) {
			this->set_value_from_SI("txtShaftweight",weight);
			m_pArrow->setValue("txtShaftweight/status", DlgArrowModel::calculated);
		}

	}

	if (m_pArrow->m_data["txtFletchesweight"].toMap().value("status").toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtFletcheslength"].toMap()["value"].toDouble()
					> 0.0) {
		double weight;
		double length;
		this->get_SI_value(length,
				m_pArrow->m_data["txtFletcheslength"].toMap());
		this->calc_fletchesweight(weight, length);
		this->set_value_from_SI("txtFletchesweight",weight);
		m_pArrow->setValue("txtFletchesweight/status", DlgArrowModel::calculated);
	}
	if (m_pArrow->m_data["txtFletchesarea"].toMap().value("status").toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtFletcheslength"].toMap()["value"].toDouble()
					> 0.0) {
		double area;
		double length;
		this->get_SI_value(length,
				m_pArrow->m_data["txtFletcheslength"].toMap());
		this->calc_fletchesarea(area, length);
		this->set_value_from_SI("txtFletchesarea",area);
		m_pArrow->setValue("txtFletchesarea/status", DlgArrowModel::calculated);
	}

	////////////////////////////////////////
	// calc the values for the "required Tab":
	if (m_pArrow->m_data["txtMass"].toMap().value("status").toInt()
			!= DlgArrowModel::modified_by_user) {
		double weight = 0.0;
		double part_weight;
		this->get_SI_value(part_weight,
				m_pArrow->m_data["txtShaftweight"].toMap());
		weight += part_weight;
		this->get_SI_value(part_weight,
				m_pArrow->m_data["txtFletchesweight"].toMap());
		weight += part_weight * 3.0;
		this->get_SI_value(part_weight,
				m_pArrow->m_data["txtPointweight"].toMap());
		weight += part_weight;
		this->get_SI_value(part_weight,
				m_pArrow->m_data["txtNockweight"].toMap());
		weight += part_weight;
		if (weight > 0.0) {
			this->set_value_from_SI("txtMass", weight);
			m_pArrow->setValue("txtMass/status", DlgArrowModel::calculated);
		}
	}
	if (m_pArrow->m_data["txtAreaFrontal"].toMap()["status"].toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftdia"].toMap()["value"].toDouble()
					> 0.0) {
		double dia;
		this->get_SI_value(dia, m_pArrow->m_data["txtShaftdia"].toMap());
		double d = dia * dia * utility::PI / 4.0;
		this->set_value_from_SI("txtAreaFrontal", d);
		m_pArrow->setValue("txtAreaFrontal/status", DlgArrowModel::calculated);

	}
	if (m_pArrow->m_data["txtAreaCrossSection"].toMap()["status"].toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftdia"].toMap()["value"].toDouble()
					> 0.0
			&& m_pArrow->m_data["txtFletchesarea"].toMap()["value"].toDouble()
					> 0.0
			&& m_pArrow->m_data["txtShaftlength"].toMap()["value"].toDouble()
					> 0.0) {

		double dia;
		double shaftlength;
		double fletchesarea;
		this->get_SI_value(dia, m_pArrow->m_data["txtShaftdia"].toMap());
		this->get_SI_value(shaftlength,
				m_pArrow->m_data["txtShaftlength"].toMap());
		this->get_SI_value(fletchesarea,
				m_pArrow->m_data["txtFletchesarea"].toMap());

		double d = dia * shaftlength + 0.000135/* pointarea: d * l => 0.009 * 0.015  */;
		d += 1.2 * fletchesarea;

		this->set_value_from_SI("txtAreaCrossSection", d);
		m_pArrow->setValue("txtAreaCrossSection/status", DlgArrowModel::calculated);
	}

	if (m_pArrow->m_data["txtPointdisplacement"].toMap()["status"].toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftlength"].toMap()["value"].toDouble()
					> 0.0) {
		double shaftlength;
		double shaft_displace;
		get_SI_value(shaftlength, m_pArrow->m_data["txtShaftlength"].toMap());
		get_SI_value(shaft_displace,
				m_pArrow->m_data["txtShaftdisplacement"].toMap());
		double d = shaftlength / 2.0 + shaft_displace;
		set_value_from_SI("txtPointdisplacement",  d);
		m_pArrow->setValue("txtPointdisplacement/status",  DlgArrowModel::calculated);
	}
	if (m_pArrow->m_data["txtNockdisplacement"].toMap()["status"].toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftlength"].toMap()["value"].toDouble()
					> 0.0) {
		double shaftlength;
		double shaft_displace;
		get_SI_value(shaftlength, m_pArrow->m_data["txtShaftlength"].toMap());
		get_SI_value(shaft_displace,
				m_pArrow->m_data["txtShaftdisplacement"].toMap());
		double d = (-1) * shaftlength / 2.0 + shaft_displace;
		set_value_from_SI("txtNockdisplacement",  d);
		m_pArrow->setValue("txtNockdisplacement/status",  DlgArrowModel::calculated);
	}
	if (m_pArrow->m_data["txtFletchesdisplacement"].toMap()["status"].toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftlength"].toMap()["value"].toDouble()
					> 0.0) {
		double shaftlength;
		double shaft_displace;
		double fletches_length;
		get_SI_value(shaftlength, m_pArrow->m_data["txtShaftlength"].toMap());
		get_SI_value(shaft_displace,
				m_pArrow->m_data["txtShaftdisplacement"].toMap());
		get_SI_value(fletches_length,
				m_pArrow->m_data["txtFletcheslength"].toMap());
		double d = (-1) * shaftlength / 2.0 + shaft_displace + fletches_length
				/ 2.0 + 0.02;
		set_value_from_SI("txtFletchesdisplacement", d);
		m_pArrow->setValue("txtFletchesdisplacement/status",  DlgArrowModel::calculated);
	}

	if (m_pArrow->m_data["txtCenterOfPresure"].toMap()["status"].toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftdia"].toMap()["value"].toDouble()
					> 0.0
			&& m_pArrow->m_data["txtShaftlength"].toMap()["value"].toDouble()
					> 0.0) {
		double point_area, dia;
		double shaft_area, shaft_length;
		double fletches_area;
		get_SI_value(dia, m_pArrow->m_data["txtShaftdia"].toMap());
		point_area = dia * 0.02; // length of the point
		get_SI_value(shaft_length, m_pArrow->m_data["txtShaftlength"].toMap());
		shaft_area = shaft_length * dia;
		get_SI_value(fletches_area, m_pArrow->m_data["txtFletchesarea"].toMap());
		fletches_area *= 1.2;
		double nock_area = dia * 0.005 /* length of the nock */;

		double point_displace;
		double shaft_displace;
		double fletches_displace;
		double nock_displace;
		get_SI_value(point_displace,
				m_pArrow->m_data["txtPointdisplacement"].toMap());
		get_SI_value(shaft_displace,
				m_pArrow->m_data["txtShaftdisplacement"].toMap());
		get_SI_value(fletches_displace,
				m_pArrow->m_data["txtFletchesdisplacement"].toMap());
		get_SI_value(nock_displace,
				m_pArrow->m_data["txtNockdisplacement"].toMap());

		double center_of_area = (point_area * point_displace + shaft_area
				* shaft_displace + fletches_area * fletches_displace
				+ nock_area * nock_displace) / (point_area + shaft_area
				+ fletches_area + nock_area);

		set_value_from_SI("txtCenterOfPresure", center_of_area);
		m_pArrow->setValue("txtCenterOfPresure/status",  DlgArrowModel::calculated);
	}

	if (m_pArrow->m_data["txtMomentOfInertia"].toMap()["status"].toInt()
			!= DlgArrowModel::modified_by_user
			&& m_pArrow->m_data["txtShaftlength"].toMap().value("value").toDouble()
					> 0.0

	) {
		double shaftlength;
		get_SI_value(shaftlength, m_pArrow->m_data["txtShaftlength"].toMap());

		double point_m;
		double shaft_m;
		double fletches_m;
		double nock_m;
		get_SI_value(point_m, m_pArrow->m_data["txtPointweight"].toMap());
		get_SI_value(shaft_m, m_pArrow->m_data["txtShaftweight"].toMap());
		get_SI_value(fletches_m, m_pArrow->m_data["txtFletchesweight"].toMap());
		get_SI_value(nock_m, m_pArrow->m_data["txtNockweight"].toMap());

		double point_displace;
		double shaft_displace;
		double fletches_displace;
		double nock_displace;
		get_SI_value(point_displace,
				m_pArrow->m_data["txtPointdisplacement"].toMap());
		get_SI_value(shaft_displace,
				m_pArrow->m_data["txtShaftdisplacement"].toMap());
		get_SI_value(fletches_displace,
				m_pArrow->m_data["txtFletchesdisplacement"].toMap());
		get_SI_value(nock_displace,
				m_pArrow->m_data["txtNockdisplacement"].toMap());
		point_displace = fabs(point_displace);
		shaft_displace = fabs(shaft_displace);
		fletches_displace = fabs(fletches_displace);
		nock_displace = fabs(nock_displace);

		double shaft_J = shaft_m * shaftlength * shaftlength / 12.0; // Näherungsweisse für einen dünnen Stab

		shaft_J = shaft_J + shaft_m * shaft_displace * shaft_displace;// Steineranteil dazurechnen
		double point_J = point_m * point_displace * point_displace;// nur Steineranteil
		double fletches_J = 3.0 * fletches_m * fletches_displace
				* fletches_displace; // nur Steineranteil
		double nock_J = nock_m * nock_displace * nock_displace;// nur Steineranteil

		double J = shaft_J + point_J + fletches_J + nock_J;

		set_value_from_SI("txtMomentOfInertia",  J);
		m_pArrow->setValue("txtMomentOfInertia/status",  DlgArrowModel::calculated);
	}
	this->blockSignals(false);


}
void DlgArrowTabDimensions::addView(DlgArrowModel *dlg) {

	this->m_pControler = dlg;
	QList<Unit> list = m_pControler->m_unitconverter.get_units("SI_metre");
	for (int i = 0; i < list.size(); i++) {
		ui.cboShaftdisplacement->addItem(list.at(i).m_name,
				list.at(i).m_type_id);
		ui.cboShaftdia->addItem(list.at(i).m_name, list.at(i).m_type_id);
		ui.cboShaftthickness->addItem(list.at(i).m_name, list.at(i).m_type_id);
		ui.cboShaftlength->addItem(list.at(i).m_name, list.at(i).m_type_id);
		ui.cboFletchesdisplacement->addItem(list.at(i).m_name,
				list.at(i).m_type_id);
		ui.cboFletcheslength->addItem(list.at(i).m_name, list.at(i).m_type_id);
		ui.cboNockdisplacement->addItem(list.at(i).m_name, list.at(i).m_type_id);
		ui.cboPointdisplacement->addItem(list.at(i).m_name,
				list.at(i).m_type_id);
	}

	list = m_pControler->m_unitconverter.get_units("SI_kilogram");
	for (int i = 0; i < list.size(); i++) {
		ui.cboShaftweight->addItem(list.at(i).m_name, list.at(i).m_type_id);
		ui.cboFletchesweight->addItem(list.at(i).m_name, list.at(i).m_type_id);
		ui.cboNockweight->addItem(list.at(i).m_name, list.at(i).m_type_id);
		ui.cboPointweight->addItem(list.at(i).m_name, list.at(i).m_type_id);

	}
	list = m_pControler->m_unitconverter.get_units("SI_square_metre");
	for (int i = 0; i < list.size(); i++) {
		ui.cboFletchesarea->addItem(list.at(i).m_name, list.at(i).m_type_id);
	}
	QPixmap *image = new QPixmap(":/images/arrowmeasures.png");
	ui.lblImage->setPixmap(*image);

}

bool DlgArrowTabDimensions::calc_diameter(double &dia, const QString shaftcode) {
	if (shaftcode.length() < 4)
		return false;
	QString shaftdia_code = shaftcode.mid(0, 2);
	double dia_inch = shaftdia_code.toDouble() / 64.0;
	double dia_metric = dia_inch * 25.4 / 1000.0;
	if (dia_metric <= 0.0)
		return false;
	dia = dia_metric;
	return true;

}
bool DlgArrowTabDimensions::calc_thickness(double &thickness,
		const QString shaftcode) {
	if (shaftcode.length() < 4)
		return false;
	QString thickness_code = shaftcode.mid(2, 2);
	double thickness_inch = thickness_code.toDouble() / 1000.0;
	double thickness_metric = thickness_inch * 25.4 / 1000.0;
	if (thickness_metric <= 0.0)
		return false;
	thickness = thickness_metric;
	return true;

}
/**
 * @depreciate
 */
bool DlgArrowTabDimensions::calc_shaftweight(double &weight,
		const QString shaftcode, const double length) {
	double dia;
	double thickness;
	if (!this->calc_diameter(dia, shaftcode))
		return false;
	if (!this->calc_thickness(thickness, shaftcode))
		return false;
	dia *= 10.0;
	thickness *= 10.0;
	double Dmittel = dia - thickness;
	double U = Dmittel * utility::PI;
	double A = U * thickness;
	double V = A * length * 10.0;
	// spez. weight = 2.7 kg/dm³
	weight = V * 2.766;
	return true;

}
bool DlgArrowTabDimensions::calc_shaftweight(double &weight, double dia,
		double thickness, const double length) {
	dia *= 10.0;
	thickness *= 10.0;
	double Dmittel = dia - thickness;
	double U = Dmittel * utility::PI;
	double A = U * thickness;
	double V = A * length * 10.0;
	// spez. weight = 2.7 kg/dm³
	weight = V * 2.766;
	return true;
}

bool DlgArrowTabDimensions::calc_fletchesweight(double &weight,
		const double length) {
	/*
	 the weight of one vane:
	 0.408g)    3"(0.076m)
	 0.518g)    4"(0.102m)
	 0.745g)    5"(0.127m)
	 *
	 * 1 Grain = 0.0648 Gramm
	 */

	SplineCurve fletches_weight;
	fletches_weight.add(0.076, 0.0004);
	fletches_weight.add(0.102, 0.00052);
	fletches_weight.add(0.127, 0.00075);
	fletches_weight.calcSpline();
	weight = fletches_weight.spline(length);
	return true;

}

bool DlgArrowTabDimensions::calc_fletchesarea(double &area, const double length) {
	/*

	 one vane:
	 3"(0.076m)         460.057mm² = 4.600cm² = 0.046dm² = 0.00046m²


	 */

	area = length * 0.00046 / 0.076;
	return true;

}

void DlgArrowTabDimensions::on_txtShaftweight_editingFinished() {
	if (ui.txtShaftweight->text()
			== m_pArrow->m_data["txtShaftweight"].toMap()["value"])
		return;
	if (ui.txtShaftweight->text().length() <= 0) {
		m_pArrow->setValue("txtShaftweight/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtShaftweight/value",  "");
	} else {
		m_pArrow->setValue("txtShaftweight/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtShaftweight/value",  ui.txtShaftweight->text() );
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtShaftlength_editingFinished() {
	if (ui.txtShaftlength->text()
			== m_pArrow->m_data["txtShaftlength"].toMap()["value"])
		return;
	if (ui.txtShaftlength->text().length() <= 0) {
		m_pArrow->setValue("txtShaftlength/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtShaftlength/value",   "");
	} else {
		m_pArrow->setValue("txtShaftlength/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtShaftlength/value",  ui.txtShaftlength->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);

}
void DlgArrowTabDimensions::on_txtShaftcode_editingFinished() {
	if (ui.txtShaftcode->text()
			== m_pArrow->m_data["txtShaftcode"].toMap()["value"])
		return;
	if (ui.txtShaftcode->text().length() <= 0) {
		m_pArrow->setValue("txtShaftcode/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtShaftcode/value",   "");
	} else {
		m_pArrow->setValue("txtShaftcode/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtShaftcode/value",  ui.txtShaftcode->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}

void DlgArrowTabDimensions::on_txtShaftdia_editingFinished() {
	if (ui.txtShaftdia->text()
			== m_pArrow->m_data["txtShaftdia"].toMap()["value"])
		return;
	if (ui.txtShaftdia->text().length() <= 0) {
		m_pArrow->setValue("txtShaftdia/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtShaftdia/value",   "");
	} else {
		m_pArrow->setValue("txtShaftdia/status", 	 DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtShaftdia/value", 	 ui.txtShaftdia->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtShaftthickness_editingFinished() {
	if (ui.txtShaftthickness->text()
			== m_pArrow->m_data["txtShaftthickness"].toMap()["value"]) {
		return;
	}
	if (ui.txtShaftthickness->text().length() <= 0) {
		m_pArrow->setValue("txtShaftthickness/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtShaftthickness/value",   "");
	} else {
		m_pArrow->setValue("txtShaftthickness/status", 	 DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtShaftthickness/value", 	 ui.txtShaftthickness->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtShaftdisplacement_editingFinished() {
	if (ui.txtShaftdisplacement->text()
			== m_pArrow->m_data["txtShaftdisplacement"].toMap()["value"])
		return;
	if (ui.txtShaftdisplacement->text().length() <= 0) {
		m_pArrow->setValue("txtShaftdisplacement/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtShaftdisplacement/value",   "");
	} else {
		m_pArrow->setValue("txtShaftdisplacement/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtShaftdisplacement/value",  ui.txtShaftdisplacement->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtFletchesarea_editingFinished() {
	if (ui.txtShaftdisplacement->text()
			== m_pArrow->m_data["txtShaftdisplacement"].toMap()["value"])
		return;
	if (ui.txtFletchesarea->text().length() <= 0) {
		m_pArrow->setValue("txtFletchesarea/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtFletchesarea/value",   "");
	} else {
		m_pArrow->setValue("txtFletchesarea/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtFletchesarea/value",  ui.txtFletchesarea->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtFletcheslength_editingFinished() {
	if (ui.txtFletcheslength->text()
			== m_pArrow->m_data["txtFletcheslength"].toMap()["value"])
		return;
	if (ui.txtFletcheslength->text().length() <= 0) {
		m_pArrow->setValue("txtFletcheslength/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtFletcheslength/value",   "");
	} else {
		m_pArrow->setValue("txtFletcheslength/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtFletcheslength/value",  ui.txtFletcheslength->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtFletchesweight_editingFinished() {
	if (ui.txtFletchesweight->text()
			== m_pArrow->m_data["txtFletchesweight"].toMap()["value"])
		return;
	if (ui.txtFletchesweight->text().length() <= 0) {
		m_pArrow->setValue("txtFletchesweight/status", 	 DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtFletchesweight/value",   "");
	} else {
		m_pArrow->setValue("txtFletchesweight/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtFletchesweight/value",  ui.txtFletchesweight->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtFletchesdisplacement_editingFinished() {
	if (ui.txtFletchesdisplacement->text()
			== m_pArrow->m_data["txtFletchesdisplacement"].toMap()["value"])
		return;
	if (ui.txtFletchesdisplacement->text().length() <= 0) {
		m_pArrow->setValue("txtFletchesdisplacement/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtFletchesdisplacement/value",   "");
	} else {
		m_pArrow->setValue("txtFletchesdisplacement/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtFletchesdisplacement/value",  ui.txtFletchesdisplacement->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtPointweight_editingFinished() {
	if (ui.txtPointweight->text()
			== m_pArrow->m_data["txtPointweight"].toMap()["value"])
		return;
	if (ui.txtPointweight->text().length() <= 0) {
		m_pArrow->setValue("txtPointweight/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtPointweight/value",   "");
	} else {
		m_pArrow->setValue("txtPointweight/status", DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtPointweight/value", 	 ui.txtPointweight->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtPointdisplacement_editingFinished() {
	if (ui.txtPointdisplacement->text()
			== m_pArrow->m_data["txtPointdisplacement"].toMap()["value"])
		return;
	if (ui.txtPointdisplacement->text().length() <= 0) {
		m_pArrow->setValue("txtPointdisplacement/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtPointdisplacement/value",   "");
	} else {
		m_pArrow->setValue("txtPointdisplacement/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtPointdisplacement/value",  ui.txtPointdisplacement->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtNockweight_editingFinished() {
	if (ui.txtNockweight->text()
			== m_pArrow->m_data["txtNockweight"].toMap()["value"])
		return;
	if (ui.txtNockweight->text().length() <= 0) {
		m_pArrow->setValue("txtNockweight/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtNockweight/value",   "");
	} else {
		m_pArrow->setValue("txtNockweight/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtNockweight/value",  ui.txtNockweight->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::on_txtNockdisplacement_editingFinished() {
	if (ui.txtNockdisplacement->text()
			== m_pArrow->m_data["txtNockdisplacement"].toMap()["value"])
		return;
	if (ui.txtNockdisplacement->text().length() <= 0) {
		m_pArrow->setValue("txtNockdisplacement/status",  DlgArrowModel::defaultvalue);
		m_pArrow->setValue("txtNockdisplacement/value",   "");
	} else {
		m_pArrow->setValue("txtNockdisplacement/status",  DlgArrowModel::modified_by_user);
		m_pArrow->setValue("txtNockdisplacement/value",  ui.txtNockdisplacement->text());
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
////////////////////////////////////////////////////////////////////////////////////
// Units comboboxes

void DlgArrowTabDimensions::on_cboShaftweight_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtShaftweight"))
		return;
	unit_changed("txtShaftweight",  ui.cboShaftweight,index);
	/////////////////
}
void DlgArrowTabDimensions::on_cboShaftlength_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtShaftlength"))
		return;
	unit_changed("txtShaftlength",  ui.cboShaftlength,	index);

}
//void on_cboShaftcode_currentIndexChanged(int index){  }
void DlgArrowTabDimensions::on_cboShaftdia_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtShaftdia"))
		return;
	unit_changed("txtShaftdia",  ui.cboShaftdia, index);

}
void DlgArrowTabDimensions::on_cboShaftthickness_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtShaftthickness"))
		return;
	unit_changed("txtShaftthickness", 	ui.cboShaftthickness, index);

}
void DlgArrowTabDimensions::on_cboShaftdisplacement_currentIndexChanged(
		int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtShaftdisplacement"))
		return;
	unit_changed("txtShaftdisplacement", ui.cboShaftdisplacement, index);

}
void DlgArrowTabDimensions::on_cboFletchesarea_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtFletchesarea"))
		return;
	unit_changed("txtFletchesarea", ui.cboFletchesarea, index);

}
void DlgArrowTabDimensions::on_cboFletcheslength_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtFletcheslength"))
		return;
	unit_changed("txtFletcheslength", 	ui.cboFletcheslength, index);

}
void DlgArrowTabDimensions::on_cboFletchesweight_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtFletchesweight"))
		return;
	unit_changed("txtFletchesweight",
			ui.cboFletchesweight, index);

}
void DlgArrowTabDimensions::on_cboFletchesdisplacement_currentIndexChanged(
		int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtFletchesdisplacement"))
		return;
	unit_changed("txtFletchesdisplacement", ui.cboFletchesdisplacement, index);

}
void DlgArrowTabDimensions::on_cboPointweight_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtPointweight"))
		return;
	unit_changed("txtPointweight",  ui.cboPointweight,
			index);

}
void DlgArrowTabDimensions::on_cboPointdisplacement_currentIndexChanged(
		int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtPointdisplacement"))
		return;
	unit_changed("txtPointdisplacement", ui.cboPointdisplacement, index);

}
void DlgArrowTabDimensions::on_cboNockweight_currentIndexChanged(int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtNockweight"))
		return;
	unit_changed("txtNockweight",  ui.cboNockweight,	index);

}
void DlgArrowTabDimensions::on_cboNockdisplacement_currentIndexChanged(
		int index) {
	if (m_pArrow == NULL)
		return;
	if (!m_pArrow->m_data.contains("txtNockdisplacement"))
		return;
	unit_changed("txtNockdisplacement", 	ui.cboNockdisplacement, index);

}
///////////////////////////////////////////////////////////

bool DlgArrowTabDimensions::get_unit_id(QString &unit_id, int index,
		QComboBox *combo) {
	if (index < 0)
		return false;
	unit_id = combo->itemData(index).toString();
	return true;
}
bool DlgArrowTabDimensions::set_unit_id(QString unit_id, QComboBox *combo) {
	QVariant var(unit_id);
	int index = combo->findData(var);
	combo->setCurrentIndex(index);
	return true;

}
bool DlgArrowTabDimensions::get_SI_value(double & value, const QMap<QString,
		QVariant> &map) {
	value = map["value"].toDouble();
	QString unit_id = map["unit"].toString();
	return this->m_pControler->m_unitconverter.convert_to_SI(value, unit_id);
}
bool DlgArrowTabDimensions::set_value_from_SI(QMap<QString, QVariant> &map,
		double value) {
	m_pControler->m_unitconverter.convert_from_SI(value, map["unit"].toString());// if conversion fails, because of an unknown unit, the SI-value is written
	map["value"] = QString::number(value);
	return true;

}
bool DlgArrowTabDimensions::set_value_from_SI(QString path_to_map,
		double value) {
	QString unit_id = m_pArrow->getValue(path_to_map + "/unit").toString();
	m_pControler->m_unitconverter.convert_from_SI(value, unit_id);// if conversion fails, because of an unknown unit, the SI-value is written
	m_pArrow->setValue(path_to_map + "/value",  QString::number(value));
	return true;

}
void DlgArrowTabDimensions::unit_changed(QMap<QString, QVariant> &map,
		QComboBox *combo, int index) {
	QString unit_id;
	if (!get_unit_id(unit_id, index, combo))
		return;
	QString prev_unit_id = map["unit"].toString();
	map["unit"] = unit_id;
	if (map["value"].toString().length() > 0) {
		double value = map["value"].toDouble();
		m_pControler->m_unitconverter.convert(value, unit_id, prev_unit_id);
		map["value"] = QString::number(value);
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
void DlgArrowTabDimensions::unit_changed(QString path_to_map,
		QComboBox *combo, int index) {
	QString unit_id;
	if (!get_unit_id(unit_id, index, combo))
		return;
	QString prev_unit_id = m_pArrow->getValue(path_to_map +"/unit").toString();
	m_pArrow->setValue(path_to_map + "/unit", unit_id);
	QString old_val = m_pArrow->getValue(path_to_map + "/value").toString();
	if (old_val.length() > 0) {
		double value = old_val.toDouble();
		m_pControler->m_unitconverter.convert(value, unit_id, prev_unit_id);
		m_pArrow->setValue(path_to_map + "/value",  QString::number(value) );
	}
	this->m_pControler->slotUpdateAllViews(NULL);
}
