/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
#include <QProcess>
#include <QTemporaryFile>
#include <QScrollArea>

#include "mainwindow.h"

#include "trajectoryview1.h"
#include "arrowmatcherview.h"
#include "SightSettingTable.h"
#include "crossbows_arrows_settingsdlg.h"

#include "armbrustdlg.h"
#include "sightdiagram_appearancedlg.h"
#include "DlgCsvSettings.h"
#include "aboutdlg.h"

#include "view.h"
#include "preferences.h"



MainWindow::MainWindow() {
	m_pdoc = NULL;
	m_gnuplot_io_file = NULL;
	m_gnuplot = NULL;
	m_views_need_recreate = false;
    m_splitter_main = NULL;
    m_content_col_main = NULL;
    m_content_col_left = NULL;
	m_pViewList = new QList<View*> ();
	isUntitled = true;
	m_helpBrowser = NULL;
	m_last_save_path = QString("");
}

void MainWindow::slotUpdateAllViews(View *sender) {
	if (m_pViewList) {
		m_views_need_recreate = true;
		for (int i = 0; i < m_pViewList->size(); ++i) {
			if (m_pViewList->at(i) != sender) {
				m_pViewList->at(i)->update(m_pdoc);
			}
		}
		m_views_need_recreate = false;
	}
}
bool MainWindow::views_need_recreate() {
	return m_views_need_recreate;

}
void MainWindow::addView(View *view) {
	bool already_added = false;
	for (int i = 0; i < m_pViewList->size(); ++i) {
		if (m_pViewList->at(i) == view) {
			already_added = true;
		}
	}
	if(! already_added)
		m_pViewList->append(view);
}

void MainWindow::removeView(View *view) {
	/** @todo implement */

	//pViewList->remove(view);
}

void MainWindow::closeEvent(QCloseEvent *event) {
	if (maybeSave()) {
		writeSettings();
		if (m_pViewList) {
			for (int i = 0; i < m_pViewList->size(); ++i) {
					m_pViewList->at(i)->removeView(this);
			}
		}
		event->accept();
	} else {
		event->ignore();
	}
}

void MainWindow::slotNewFile() {
	MainWindow *other = new MainWindow;
	other->init(QString());
	other->move(x() + 40, y() + 40);
	other->show();
}

void MainWindow::open() {

	QString fileName = QFileDialog::getOpenFileName(this, tr("Open"),
			m_last_save_path, "*.arw");
	if (fileName.isEmpty())
		return;
	MainWindow *existing = findMainWindow(fileName);
	if (existing) {
		existing->show();
		existing->raise();
		existing->activateWindow();
		return;
	}

	if (isUntitled && !m_pdoc->isModified()) {

		loadFile(fileName);
		addRecentFile(fileName);

		slotUpdateAllViews(0);

	} else {
		MainWindow *other = new MainWindow();
		other->init(fileName);
		if (other->isUntitled) {
			delete other;
		}
		other->move(x() + 40, y() + 40);
		other->show();
	}

}

bool MainWindow::save() {

	bool ret;
	QApplication::setOverrideCursor(Qt::WaitCursor);

	if (isUntitled)
		ret = saveAs();
	else
		ret = saveFile(m_pdoc->m_filename);

	QApplication::restoreOverrideCursor();

	statusBar()->showMessage(tr("File saved"), 2000);

	return ret;
}

bool MainWindow::saveAs() {
	QString local_file = m_pdoc->m_filename;

	QFileInfo info(local_file);
	if (info.isRelative()) {
			local_file = m_last_save_path + QDir::separator() + local_file;
	}

	QString fileName = QFileDialog::getSaveFileName(this, tr("Save As"),
			local_file);
	if (fileName.isEmpty())
		return false;

	QFileInfo saveFileInfo(fileName);
	m_last_save_path = saveFileInfo.path();
	return saveFile(fileName);
}

void MainWindow::about() {
	QDir documentationDir = QDir(QLibraryInfo::location(
			QLibraryInfo::DocumentationPath));
	QString str = QLibraryInfo::location(QLibraryInfo::DocumentationPath);

	aboutDlg *dlg = new aboutDlg(this);
	dlg->exec();
}

void MainWindow::documentWasModified() {
	setWindowModified(true);
}

void MainWindow::init(const QString &fileName) {

	m_pdoc = new ArrowmatcherDoc(this);


	m_content_col_main = new QTabWidget;
	m_content_col_left = new QTabWidget;
	m_splitter_main = new QSplitter(this);



	setAttribute(Qt::WA_DeleteOnClose);
	setWindowTitle(tr("%1[*] - %2").arg(m_pdoc->m_filename).arg(tr(
			"Arrowmatcher2")));
	setWindowIcon(QIcon(":/toolbar/arrowmatcher"));



	m_splitter_main->addWidget(m_content_col_left);
	m_splitter_main->addWidget(m_content_col_main);
	setCentralWidget(m_splitter_main);

	readSettings();

	createActions();
	createMenus();
	createToolBars();
	createStatusBar();


	/////////////
	// create all views
	//
	crossbows_arrows_settingsdlg* cas = new crossbows_arrows_settingsdlg(this);
	addView(cas);

	ArrowmatcherView *sight_diagram = new ArrowmatcherView(this);
	addView(sight_diagram);

	SightSettingTable *sst = new SightSettingTable;
	addView(sst);

	TrajectoryView1 *trajectory_view = new TrajectoryView1;
	addView(trajectory_view);


	if (m_pViewList) {
		for (int i = 0; i < m_pViewList->size(); ++i) {
				m_pViewList->at(i)->addView(this);
		}
	}
	/////////////////////////////////////////////
	setCurrentFile("");
	if (!fileName.isEmpty()) {
		loadFile(fileName);
	}
	//qDebug() << "mainwindow unitsystem=" << (int)m_pdoc->m_unitconverter.m_system;
	QList<QAction*> unitsystemActions = unitsystemActGroup->actions();
	QAction* tempAct;
	for(int i=0; i<unitsystemActions.size(); i++) {
		tempAct = unitsystemActions.at(i);
		if(tempAct->data().toInt() == (int)m_pdoc->m_unitconverter.m_system ) {
			tempAct->setChecked(true);
		}else {
			tempAct->setChecked(false);
		}
	}

	slotUpdateAllViews(0);

}

void MainWindow::createActions() {
	newAct = new QAction(QIcon(":/toolbar/filenew.png"), tr("&New"), this);
	newAct->setShortcut(tr("Ctrl+N"));
	newAct->setStatusTip(tr("Create a new file"));
	connect(newAct, SIGNAL(triggered()), this, SLOT(slotNewFile()));

	openAct
			= new QAction(QIcon(":/toolbar/fileopen.png"), tr("&Open..."), this);
	openAct->setShortcut(tr("Ctrl+O"));
	openAct->setStatusTip(tr("Open an existing file"));
	connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

	m_recentFilesActions = new QActionGroup(this);
	m_recentFilesActions->setExclusive(false);
	QAction *act;
	int MaxRecentFiles = 10;
	// Need to insert this into the QAction.
	for (int i = 0; i < MaxRecentFiles; ++i) {
		act = new QAction(this);
		act->setVisible(false);
		connect(act, SIGNAL(triggered()), this, SLOT(openRecentForm()));
		m_recentFilesActions->addAction(act);
	}
	updateRecentFileActions();

	saveAct = new QAction(QIcon(":/toolbar/filesave.png"), tr("&Save"), this);
	saveAct->setShortcut(tr("Ctrl+S"));
	saveAct->setStatusTip(tr("Save the document to disk"));
	connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

	saveAsAct = new QAction(QIcon(":/toolbar/filesaveas.png"),
			tr("Save &As..."), this);
	saveAsAct->setStatusTip(tr("Save the document under a new name"));
	connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

	closeAct = new QAction(tr("&Close"), this);
	closeAct->setShortcut(tr("Ctrl+W"));
	closeAct->setStatusTip(tr("Close this window"));
	connect(closeAct, SIGNAL(triggered()), this, SLOT(close()));

	exitAct = new QAction(QIcon(":/toolbar/exit.png"), tr("E&xit"), this);
	exitAct->setShortcut(tr("Ctrl+Q"));
	exitAct->setStatusTip(tr("Exit the application"));
	connect(exitAct, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));

	aboutAct = new QAction(tr("&About"), this);
	aboutAct->setStatusTip(tr("Show the application's About box"));
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

	aboutQtAct = new QAction(tr("About &Qt"), this);
	aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
	connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

	helpAct = new QAction(tr("Help"), this);
	helpAct->setStatusTip(tr("Show help browser"));
	connect(helpAct, SIGNAL(triggered()), this, SLOT(slotHelp()));




	//edit_angle_list
	SettingsSightDiagram_appearance
			= new QAction(QIcon(":/toolbar/tlb_visier"), tr(
					"Sight Diagram Appearance"), this);
	connect(SettingsSightDiagram_appearance, SIGNAL(triggered()), this,
			SLOT(slotEditSight_Diagram_appearance()));


	plotTrajectory = new QAction(QIcon(":/toolbar/tlb_edit_targetcircles"), tr(
			"Trajectory"), this);
	plotTrajectory->setStatusTip(tr("plot the trajectory"));
	connect(plotTrajectory, SIGNAL(triggered()), this,
			SLOT(slotPlotTrajectory()));


	showPreferencesDlg = new QAction(tr("Preferences"), this);
	connect(showPreferencesDlg, SIGNAL(triggered()), this, SLOT(slotShowPreferencesDlg()) );


	unitsystemActGroup = new QActionGroup(this);
	unitsystemActGroup->setExclusive(true);
	UnitConverter uc;
	QList<UnitSystem> list = uc.get_list_UnitSystem();
	QAction* tempAct;
	for(int i=0; i<list.count(); i++) {
		tempAct = new QAction(list.at(i).m_name, this);
		tempAct->setCheckable(true);
		tempAct->setData((int)list.at(i).m_system);
		unitsystemActGroup->addAction(tempAct);
		connect(unitsystemActGroup, SIGNAL(triggered(QAction*)), this, SLOT(slotunitsystemActGroup(QAction*)) );
		//ui.cboUnitsystem->addItem(list.at(i).m_name,(int)list.at(i).m_system);
	}




}

void MainWindow::createMenus() {
	m_fileMenu = menuBar()->addMenu(tr("&File"));
	m_fileMenu->addAction(newAct);
	m_fileMenu->addAction(openAct);
	QMenu *recentFilesMenu = m_fileMenu->addMenu(tr("&Recent Files"));
	// Pop the "Recent Files" stuff in here.
	foreach(QAction *recentAction, m_recentFilesActions->actions())
			recentFilesMenu->addAction(recentAction);

	m_fileMenu->addAction(saveAct);
	m_fileMenu->addAction(saveAsAct);
	m_fileMenu->addSeparator();
	m_fileMenu->addAction(closeAct);
	m_fileMenu->addAction(exitAct);

	m_editMenu = menuBar()->addMenu(tr("&Edit"));
	m_editMenu->addAction(SettingsSightDiagram_appearance);
	m_editMenu->addAction(showPreferencesDlg);

	QList<QAction*> unitsystemActions = unitsystemActGroup->actions();
	QAction* tempAct;
	for(int i=0; i<unitsystemActions.size(); i++) {
		tempAct = unitsystemActions.at(i);
		m_editMenu->addAction(tempAct);


	}


	menuBar()->addSeparator();

	m_helpMenu = menuBar()->addMenu(tr("&Help"));
	m_helpMenu->addAction(aboutAct);
	m_helpMenu->addAction(aboutQtAct);
	m_helpMenu->addAction(helpAct);
}

void MainWindow::createToolBars() {
	m_fileToolBar = addToolBar(tr("File"));
	m_fileToolBar->addAction(newAct);
	m_fileToolBar->addAction(openAct);
	m_fileToolBar->addAction(saveAct);

	m_editToolBar = addToolBar(tr("Edit"));
	m_editToolBar->addAction(SettingsSightDiagram_appearance);
	//editToolBar->addAction(SettingsDiagram_appearance);

	//gnuplotToolBar = addToolBar(tr("diagrams"));
	//gnuplotToolBar->addAction(plotTrajectory);

}

void MainWindow::createStatusBar() {
	statusBar()->showMessage(tr("Ready"));
}

void MainWindow::readSettings() {
	QSettings settings;
	QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
	QSize size = settings.value("size", QSize(400, 400)).toSize();
	move(pos);
	resize(size);
	m_splitter_main->restoreState(settings.value("splitter_main").toByteArray());

	m_path2gnuplot = settings.value("path2gnuplot").value<QString> () ;
	m_last_save_path = settings.value("lastSavePath").value<QString> ();
	int recentFiles_count = settings.beginReadArray("recentFiles");
	for (int i = 0; i < recentFiles_count; ++i) {
		settings.setArrayIndex(i);
		QString str = settings.value("filename").value<QString> ();
		m_recentFiles.append(str);
	}
	settings.endArray();

	int us;
	us = settings.value("unitsystem").value<int> ();
	m_pdoc->m_unitconverter.m_system = (Unit::UnitSystem)us;
	//qDebug() << "mainwindow unitsystem=" << (int)m_pdoc->m_unitconverter.m_system;


}

void MainWindow::writeSettings() {
	QSettings settings;
	settings.setValue("pos", pos());
	settings.setValue("size", size());
	settings.setValue("splitter_main", m_splitter_main->saveState());

	settings.setValue("lastSavePath", m_last_save_path);

	settings.beginWriteArray("recentFiles");
	for (int i = 0; i < m_recentFiles.size() && i < 10; ++i) {
		settings.setArrayIndex(i);
		settings.setValue("filename", m_recentFiles.at(i));
	}
	settings.endArray();



}

bool MainWindow::maybeSave() {
	if (this->m_pdoc->isModified()) {
		int ret = QMessageBox::warning(this, tr("Arrowmatcher"), tr(
				"The document has been modified.\n"
					"Do you want to save your changes?"), QMessageBox::Yes
				| QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel
				| QMessageBox::Escape);
		if (ret == QMessageBox::Yes)
			return save();
		else if (ret == QMessageBox::Cancel)
			return false;
	}
	return true;
}

void MainWindow::loadFile(const QString &fileName) {

	QFile file(fileName);
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
		QMessageBox::warning(this, tr("Arrowmatcher"), tr(
				"Cannot read file %1:\n%2.") .arg(fileName) .arg(
				file.errorString()));
		return;
	}

	m_pdoc->openDocument(fileName);
	addRecentFile(fileName);
	setCurrentFile(fileName);
	slotUpdateAllViews(0);

	statusBar()->showMessage(tr("File loaded"), 2000);
}

bool MainWindow::saveFile(const QString &fileName) {
	QApplication::setOverrideCursor(Qt::WaitCursor);

	if (!m_pdoc->saveDocument(fileName)) {
		QMessageBox::warning(this, tr("arrowmatcher"), tr(
				"Cannot write file %1:\n%2.") .arg(fileName) .arg(tr(
				"cannot write file")));

		return false;

	}
	m_recentFiles.push_front(fileName);
	setCurrentFile(fileName);
	QApplication::restoreOverrideCursor();
	statusBar()->showMessage(tr("File saved"), 2000);
	return true;
}

void MainWindow::setCurrentFile(const QString &fileName) {
	static int sequenceNumber = 1;

	isUntitled = m_pdoc->m_filename.isEmpty();
	if (isUntitled) {
		m_pdoc->m_filename = QString("untitled%1.arw").arg(sequenceNumber++);
	} else {
		m_pdoc->m_filename = QFileInfo(fileName).canonicalFilePath();
	}

	m_pdoc->setModified(false);
	setWindowModified(false);
	setWindowTitle(tr("%1[*] - %2").arg(m_pdoc->m_filename).arg(tr(
			"arrowmatcher")));
}

QString MainWindow::strippedName(const QString &fullFileName) {
	return QFileInfo(fullFileName).fileName();
}

MainWindow *MainWindow::findMainWindow(const QString &fileName) {
	foreach (QWidget *widget, qApp->topLevelWidgets())
		{
			MainWindow *mainWin = qobject_cast<MainWindow *> (widget);
			if (mainWin && mainWin->m_pdoc && mainWin->m_pdoc->m_filename
					== fileName)
				return mainWin;
		}
	return 0;
}
/** shows assistant */
void MainWindow::slotHelp() {

	if(m_helpBrowser == NULL){
		m_helpBrowser = new QProcess;
		if(m_helpBrowser->state() != QProcess::Running){
			QStringList args;
			QString locale = QLocale::system().name();
			QString lang = locale.left(2);
			QString url;
			QString collectionfile;

			QStringList path_to_qhc;
			path_to_qhc
			        << "doc"          // relative
					<< "../doc"       // relative
					<< "../../doc"    //relative
					<< "/share/doc/packages/arrowmatcher2"    //ubuntu
					<< "/usr/share/doc/packages/arrowmatcher2" // opensuse
					<< "/usr/local/share/doc/packages/arrowmatcher2" // "make install" opensuse
					<< "/usr/share/doc/arrowmatcher2";         // fedora

			bool found_qhc = false;
			for(int i=0; i< path_to_qhc.size(); i++) {
				collectionfile = path_to_qhc.at(i) + QString("/arrowmatcher2.qhc");
				if(QFile::exists(collectionfile)) {
					found_qhc = true;
					break;
				}
			}
			if(! found_qhc) {
				//qDebug() << "collection File not found in:" << path_to_qhc;
	             QMessageBox::critical(0, QObject::tr("arrowmatcher2"),
	                 QObject::tr("Unable to find the .qhc file of the documentation in\n %1").arg(path_to_qhc.join(QString("\n"))));
			}



			url = QString("qthelp://arrowmatcher2.") + lang + QString("/doc/arrowmatcher_2.html");


			args << QString("-collectionFile").toLatin1()
			<< QString(collectionfile).toLatin1()
			<< QString("-hide").toLatin1() <<  QString("index").toLatin1()
			<< QString("-hide").toLatin1() <<  QString("bookmarks").toLatin1()
			<< QString("-hide").toLatin1() <<  QString("contents").toLatin1()
			<< QString("-showUrl").toLatin1()
			<< QString(url.toLatin1()).toLatin1();

			//qDebug() << "url=" << url;
			//qDebug() << args;
	        QString app = QLibraryInfo::location(QLibraryInfo::BinariesPath) + QDir::separator();



#if defined(Q_OS_WIN)
			QStringList assistant;
			// First looks in the QT Directory and then in the start directory of arrowmatcher:
			assistant << QLibraryInfo::location(QLibraryInfo::BinariesPath) + QDir::separator() + QLatin1String("assistant.exe")
					<< QLatin1String("assistant.exe");
			for(int i=0; i< assistant.size(); i++) {
				if(QFile::exists(assistant.at(i))) {
					app = assistant.at(i);
					break;
				}
			}


#elif defined(Q_OS_MAC)
            app += QLatin1String("Assistant.app/Contents/MacOS/Assistant");
#else
	        app += QLatin1String("assistant");
#endif
            //qDebug() << app;
			m_helpBrowser->start(app, args);
			if (!m_helpBrowser->waitForStarted()){
	             QMessageBox::critical(0, QObject::tr("arrowmatcher2"),
	                 QObject::tr("Unable to launch Qt Assistant (%1 %2)").arg(app, args.join(QString(" "))));
				return;
			}

			connect(m_helpBrowser, SIGNAL(finished(int, QProcess::ExitStatus)), this,
					SLOT(helpBrowser_finished ( int , QProcess::ExitStatus)));


		}
	}
}



/** No descriptions */
void MainWindow::slotEditSight_Diagram_appearance() {
	sightdiagram_appearanceDlg *dlg = new sightdiagram_appearanceDlg(this);
	if (m_pdoc != NULL)
		dlg->init(&m_pdoc->m_Targetcircles, &m_pdoc->m_Anglelist,
				m_pdoc->m_diagram_max_range, m_pdoc->m_diagram_max_angle);
	else
		return;
	if (dlg->exec() == QDialog::Accepted) {
		slotUpdateAllViews(NULL);

		//syncronize();
	}
}
//void MainWindow::slotEditDiagram_appearance() {

//}


void MainWindow::updateRecentFileActions() {
	int originalSize = m_recentFiles.size();
	//int MaxRecentFiles = 10;
	int numRecentFiles = qMin(m_recentFiles.size(), int(MaxRecentFiles));
	QList<QAction *> recentFilesActs = m_recentFilesActions->actions();

	for (int i = 0; i < numRecentFiles; ++i) {
		QFileInfo fi(m_recentFiles[i]);
		// If the file doesn't exist anymore, just remove it from the list so
		// people don't get confused.
		if (!fi.exists()) {
			m_recentFiles.removeAt(i);
			--i;
			numRecentFiles = qMin(m_recentFiles.size(), int(MaxRecentFiles));
			continue;
		}
		QString text = fi.fileName();
		recentFilesActs[i]->setText(text);
		recentFilesActs[i]->setIconText(m_recentFiles[i]);
		recentFilesActs[i]->setVisible(true);
	}

	for (int j = numRecentFiles; j < MaxRecentFiles; ++j)
		recentFilesActs[j]->setVisible(false);

	// If there's been a change, right it back
	//if (originalSize != m_recentFiles.size())
	//    settings.setRecentFilesList(m_recentFiles);
}

void MainWindow::openRecentForm() {
	if (QAction * action = qobject_cast<QAction *> (sender())) {
		QString fileName = action->iconText();
		loadFile(fileName);

	}
}

void MainWindow::clearRecentFiles() {
	m_recentFiles = QStringList();// overwrite with an emty list
	updateRecentFileActions();
}

QActionGroup *MainWindow::recentFilesActions() const {
	return m_recentFilesActions;
}

void MainWindow::addRecentFile(const QString &fileName) {
	m_recentFiles.removeAll(fileName);
	m_recentFiles.prepend(fileName);
	while (m_recentFiles.size() > MaxRecentFiles)
		m_recentFiles.removeLast();

	//settings.setRecentFilesList(files);
	updateRecentFileActions();
}



void MainWindow::slotPlotTrajectory() {

	if (m_gnuplot_io_file != NULL) {
		delete m_gnuplot_io_file;
	}

	m_gnuplot_io_file = new QTemporaryFile();
	if (!m_gnuplot_io_file->open()) {
		std::cout << "filename:"
				<< m_gnuplot_io_file->fileName().toLatin1().data() << std::endl;
		return;
	}

	flightparam fp;

	std::vector<flight_snapshot> global_table, crossbow_table, sight_table, line_of_sight_table;

	Ballistic::flightparam_init(fp, m_pdoc->m_Arrow);
	Ballistic::flightparam_gravity(fp, m_pdoc->m_Angle);
	Ballistic::flightparam_wind(fp, m_pdoc->m_windangle, m_pdoc->m_windspeed);
	Ballistic b(fp);

//	for(double d=10.0; d< 5000.0; d+=30.0){
//		std::cout << d << ": " << b.m_cd.spline(d) << "\n";
//	}


	m_pdoc->m_Crossbow.calc_flight_table(global_table,
			crossbow_table,
			sight_table,
			line_of_sight_table,
			20.0, // max fight time
			m_pdoc->m_diagram_max_range, // max range
			m_pdoc->m_Range,
			fp);

	QTextStream out(m_gnuplot_io_file);

	std::vector<flight_snapshot>::iterator global_iter, crossbow_iter, sight_iter, line_of_sight_iter;

	for(global_iter = global_table.begin(), crossbow_iter = crossbow_table.begin(), sight_iter = sight_table.begin(), line_of_sight_iter = line_of_sight_table.begin();
	            global_iter != global_table.end();
	            global_iter++, crossbow_iter++, sight_iter++, line_of_sight_iter++)
	{
		out << (*global_iter).t << " "
		      << (*global_iter).s.x << " "
		      << (*global_iter).s.y << " "
		      << (*global_iter).s.z << " "
		      << (*global_iter).v.x << " "
		      << (*global_iter).v.y << " "
		      << (*global_iter).v.z << " "
		      << (*global_iter).phi.x << " "
		      << (*global_iter).phi.y << " "
		      << (*global_iter).phi.z << " "
		      << (*global_iter).omega.x << " "
		      << (*global_iter).omega.y << " "
		      << (*global_iter).omega.z << "           "

		      << (*crossbow_iter).s.x << " "
		      << (*crossbow_iter).s.y << " "
		      << (*crossbow_iter).s.z << " "
		      << (*crossbow_iter).v.x << " "
		      << (*crossbow_iter).v.y << " "
		      << (*crossbow_iter).v.z << " "
		      << (*crossbow_iter).phi.x << " "
		      << (*crossbow_iter).phi.y << " "
		      << (*crossbow_iter).phi.z << " "
		      << (*crossbow_iter).omega.x << " "
		      << (*crossbow_iter).omega.y << " "
		      << (*crossbow_iter).omega.z << "              "

		      << (*sight_iter).s.x << " "
		      << (*sight_iter).s.y << " "
		      << (*sight_iter).s.z << " "
		      << (*sight_iter).v.x << " "
		      << (*sight_iter).v.y << " "
		      << (*sight_iter).v.z << " "
		      << (*sight_iter).phi.x << " "
		      << (*sight_iter).phi.y << " "
		      << (*sight_iter).phi.z << " "
		      << (*sight_iter).omega.x << " "
		      << (*sight_iter).omega.y << " "
		      << (*sight_iter).omega.z << "               "

		      << (*line_of_sight_iter).s.x << " "
		      << (*line_of_sight_iter).s.y << " "
			  << (*line_of_sight_iter).s.z << " "
			  << (*line_of_sight_iter).v.x << " "
			  << (*line_of_sight_iter).v.y << " "
			  << (*line_of_sight_iter).v.z << " "
			  << (*line_of_sight_iter).phi.x << " "
			  << (*line_of_sight_iter).phi.y << " "
			  << (*line_of_sight_iter).phi.z << " "
			  << (*line_of_sight_iter).omega.x << " "
			  << (*line_of_sight_iter).omega.y << " "
			  << (*line_of_sight_iter).omega.z << " "


		      << "\n";

	}

	out.flush();
	if(m_gnuplot == NULL){
		m_gnuplot = new QProcess();
		m_gnuplot->start("gnuplot");
	}
	if (!m_gnuplot->waitForStarted())
		return;
	QString gnuplotcmd_string;
	QTextStream gnuplotcmd(&gnuplotcmd_string);

	gnuplotcmd << "set term png size 1000, 500;\n";
	//		<< "set output '/home/herbert/gnuplottest.png'\n";

	gnuplotcmd << "set multiplot;\n";
	gnuplotcmd << "set autoscale\n";
	gnuplotcmd << "set size 1,0.5;\n";

	gnuplotcmd << "set origin 0.0, 0.0;\n"
				<< "set title '" << tr("angle of the arrow") << "';\n"
				<< "set xlabel '" << tr("range") << " [m]" << "';\n"
				<< "set xzeroaxis;\n"
				<< "set grid;\n"
				<< "set ylabel '[rad]';\n";
	gnuplotcmd << "plot '" << m_gnuplot_io_file->fileName() << "' using 2:10 title 'z-" << tr("axis") << "' with lines"
	           << ", '"    << m_gnuplot_io_file->fileName() << "' using 2:9  title 'y-" << tr("axis") << "' with lines;\n";

	gnuplotcmd << "set origin 0.0, 0.5;\n"
	           << "set title '" << tr("path of the arrow") << "';\n"
				<< "set xlabel '" << tr("range") << " [m]" << "';\n"
				<< "set xzeroaxis;\n"
				<< "set grid;\n"
				<< "set ylabel '[m]';\n";
	gnuplotcmd << "plot '" << m_gnuplot_io_file->fileName() << "' using 38:39 title 'y-" << tr("axis") << "' with lines"
	           << ", '"    << m_gnuplot_io_file->fileName() << "' using 38:40 title 'z-" << tr("axis") << "' with lines;\n";

	gnuplotcmd << "unset multiplot;\n";
	std::cout <<  gnuplotcmd_string.toStdString().c_str() << std::endl;

	m_gnuplot->write(gnuplotcmd_string.toAscii() );

}
void MainWindow::slotShowPreferencesDlg()
{
	Preferences *dlg = new Preferences(this);
	dlg->m_path2gnuplot = m_path2gnuplot;
	dlg->init();
	if (dlg->exec() == QDialog::Rejected) {
		return;
	}
	m_path2gnuplot = dlg->m_path2gnuplot;
	//slotUpdateAllViews(NULL);


}
void MainWindow::slotunitsystemActGroup(QAction* act) {
	if(m_pdoc == NULL) {
		return;
	}
	//qDebug() << "slotunitsystemActGroup";
	if(act->isChecked()) {
		if(act->data().toInt() == (int)Unit::metric) {
			m_pdoc->m_unitconverter.m_system = Unit::metric;
		}
		else if(act->data().toInt() == (int)Unit::english) {
			m_pdoc->m_unitconverter.m_system = Unit::english;
		}
		//qDebug() << act->data().toInt();
	}
	slotUpdateAllViews(NULL);


}



void MainWindow::helpBrowser_finished(int exitCode,
		QProcess::ExitStatus exitStatus) {
	m_helpBrowser = NULL;
}
